@@ -192,6 +192,15 @@ public:
 	VImage( im__IMAGE *image );
 	VImage() throw( VError );
 
+	// Convert to a disc file, eg:
+	// 	VImage fred = VImage::convert2disc( "im_jpeg2vips", 
+	// 		"file.jpg", "temp.v" );
+	// Runs im_jpeg2vips to the temp file, then opens that and returns
+	// it. Useful for opening very large files without using a lot of RAM.
+	// FIXME ... what a hack, replace this with something better
+	static VImage convert2disc( const char* convert, 
+		const char* in, const char* disc ) throw( VError );
+
 	// Copy constructor 
 	VImage( const VImage &a );
 
@@ -389,7 +398,7 @@ public:
 
 VIPS_NAMESPACE_END
 
-// Other VIPS protos
+// Other VIPS protos we need 
 extern "C" {
 extern int im_init_world( const char *argv0 ); 
 extern void im__print_all(); 
