@@ -34,13 +34,9 @@
 extern "C" {
 #endif /*__cplusplus*/
 
-/* Need List.
- */
-#include <vips/list.h>
-
 /* All open image descriptors ... see im_init() and im_close().
  */
-extern List *im__open_images;
+extern GSList *im__open_images;
 
 /* Print one line for each descriptor, complete dump for one descriptor.
  */
