@@ -4,6 +4,10 @@
 #ifndef IM_VIPS_INTL_H
 #define IM_VIPS_INTL_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif /*__cplusplus*/
+
 const char *im__gettext( const char *msgid );
 const char *im__ngettext( const char *msgid,
 	const char *plural, unsigned long int n );
@@ -39,6 +43,10 @@ const char *im__ngettext( const char *msgid,
 
 #endif /* ENABLE_NLS */
 
+#ifdef __cplusplus
+}
+#endif /*__cplusplus*/
+
 #endif /* IM_VIPS_INTL_H */
 
 
