@@ -74,6 +74,10 @@ typedef void *(*im_header_map_fn)( IMAGE *, const char *, GValue *, void * );
 int im_init_world( const char *argv0 );
 GOptionGroup *im_get_option_group( void );
 
+/* Turn progress feedback on and off.
+ */
+void im_progress_set( int progress );
+
 const char *im_error_buffer( void );
 int im_debugim( IMAGE * );
 int im_printlines( IMAGE * );
@@ -149,6 +153,8 @@ int im_ismagick( const char * );
 int im_isanalyze( const char *filename );
 
 int im_add_close_callback( IMAGE *, im_callback_fn, void *, void * );
+int im_add_preclose_callback( IMAGE *, im_callback_fn, void *, void * );
+int im_add_evalstart_callback( IMAGE *, im_callback_fn, void *, void * );
 int im_add_eval_callback( IMAGE *, im_callback_fn, void *, void * );
 int im_add_evalend_callback( IMAGE *, im_callback_fn, void *, void * );
 
@@ -264,12 +270,11 @@ int im_convsub( IMAGE *, IMAGE *, INTMASK *, int, int );
 int im_grad_x( IMAGE *in, IMAGE *out );
 int im_grad_y( IMAGE *in, IMAGE *out );
 
+int im_phasecor_fft( IMAGE *in1, IMAGE *in2, IMAGE *out );
 int im_fastcor( IMAGE *, IMAGE *, IMAGE * );
 int im_fastcor_raw( IMAGE *, IMAGE *, IMAGE * );
 int im_spcor( IMAGE *, IMAGE *, IMAGE * );
 int im_spcor_raw( IMAGE *, IMAGE *, IMAGE * );
-int im_spcor2( IMAGE *, IMAGE *, IMAGE * );
-int im_spcor2_raw( IMAGE *, IMAGE *, IMAGE * );
 int im_gradcor( IMAGE *, IMAGE *, IMAGE * );
 int im_gradcor_raw( IMAGE *, IMAGE *, IMAGE * );
 int im_contrast_surface( IMAGE *, IMAGE *, int, int );
@@ -343,6 +348,7 @@ int im_fav4( IMAGE **, IMAGE * );
 int im_gadd( double, IMAGE *, double, IMAGE *, double, IMAGE *);
 int im_litecor( IMAGE *, IMAGE *, IMAGE *, int, double );
 int im_bandmean( IMAGE *in, IMAGE *out );
+int im_cross_phase( IMAGE *a, IMAGE *b, IMAGE *out );
 
 /* boolean
  */
@@ -358,6 +364,13 @@ int im_eor_vec( IMAGE *, IMAGE *, int, double * );
 int im_shiftleft( IMAGE *, IMAGE *, int );
 int im_shiftright( IMAGE *, IMAGE *, int );
 
+/* cimg
+ */
+int im_greyc_mask( IMAGE *in, IMAGE *out, IMAGE *mask, 
+	int iterations, float amplitude, float sharpness, float anisotropy, 
+	float alpha, float sigma, float dl, float da, float gauss_prec, 
+	int interpolation, int fast_approx );
+
 /* histogram
  */
 int im_maplut( IMAGE *, IMAGE *, IMAGE * );
@@ -493,6 +506,7 @@ int im_replicate( IMAGE *in, IMAGE *out, int across, int down );
 int im_grid( IMAGE *in, IMAGE *out, int tile_height, int across, int down );
 int im_msb ( IMAGE * in, IMAGE * out );
 int im_msb_band ( IMAGE * in, IMAGE * out, int band );
+int im_wrap( IMAGE *in, IMAGE *out, int x, int y );
 
 /* colour
  */
@@ -650,6 +664,9 @@ int im_correl( IMAGE *ref, IMAGE *sec,
 int im_remosaic( IMAGE *in, IMAGE *out,
 	const char *old_str, const char *new_str );
 
+int im_align_bands( IMAGE *in, IMAGE *out );
+int im_maxpos_subpel( IMAGE *in, double *x, double *y );
+
 /* inplace
  */
 int im_plotmask( IMAGE *, int, int, PEL *, PEL *, Rect * );
