@@ -41,10 +41,16 @@ extern "C" {
 
 #define g_thread_init im__g_thread_init
 #define g_thread_join im__g_thread_join
+#define g_thread_self im__g_thread_self
 #define g_thread_create_full im__g_thread_create_full
 
+/* We don't need a shadow imlementation of g_thread_create(), even though we
+ * use it, because it's just a macro over g_thread_create_full().
+ */
+
 void im__g_thread_init( GThreadFunctions *vtable );
 gpointer im__g_thread_join( GThread * );
+gpointer im__g_thread_self( void );
 GThread *im__g_thread_create_full( GThreadFunc, 
 	gpointer, gulong, gboolean, gboolean, GThreadPriority, GError ** );
 
