@@ -47,17 +47,6 @@ extern "C" {
  */
 #define IM__DEFAULT_STACK_SIZE (2 * 1024 * 1024)
 
-/* Default tile geometry.
- */
-extern int im__tile_width;
-extern int im__tile_height;
-extern int im__fatstrip_height;
-extern int im__thinstrip_height;
-
-/* Default n threads.
- */
-extern int im__concurrency;
-
 /* A work function.
  */
 typedef int (*im__work_fn)( REGION *, void *, void *, void * );
@@ -109,6 +98,8 @@ typedef struct im__threadgroup_t {
 #endif /*DEBUG_HIGHWATER*/
 
 	int kill;		/* Set this to stop threadgroup early */
+
+	int progress;		/* Set this to get eval progress feedback */
 } im_threadgroup_t;
 
 void im_concurrency_set( int concurrency );
@@ -127,6 +118,12 @@ void im_threadgroup_trigger( im_thread_t *thr );
  */
 int im_prepare_thread( im_threadgroup_t *tg, REGION *oreg, Rect *r );
 
+/* Threaded, double-buffered eval to file.
+ */
+typedef int (*im_wbuffer_fn)( REGION *region, Rect *area, void *a, void *b );
+int im_wbuffer( im_threadgroup_t *tg, 
+	im_wbuffer_fn write_fn, void *a, void *b );
+
 #ifdef __cplusplus
 }
 #endif /*__cplusplus*/
