@@ -42,9 +42,9 @@ extern "C" {
  */
 #define IM_PI (3.14159265358979323846)
 
-/* Don't define if IM_NO_VIPS7_COMPAT is set.
+/* Only define if IM_ENABLE_DEPRECATED is set.
  */
-#ifndef IM_NO_VIPS7_COMPAT
+#ifdef IM_ENABLE_DEPRECATED
 
 #ifndef MAX
 #define MAX(A,B) ((A)>(B)?(A):(B))
@@ -143,7 +143,7 @@ extern "C" {
 
 #define CLIP_NONE( V, SEQ ) {}
 
-#endif /*IM_NO_VIPS7_COMPAT*/
+#endif /*IM_ENABLE_DEPRECATED*/
 
 #define IM_MAX(A,B) ((A)>(B)?(A):(B))
 #define IM_MIN(A,B) ((A)<(B)?(A):(B))
@@ -160,6 +160,17 @@ extern "C" {
         } \
 } while( 0 )
 #define IM_FREE( A ) IM_FREEF( im_free, A )
+#define IM_SETSTR( S, V ) do { \
+        if( (S) != (V) ) { \
+                if( !(S) || !(V) || strcmp( (S), (V) ) != 0 ) { \
+                        const char *sst = (V); \
+                        \
+                        IM_FREE( S ); \
+                        if( sst ) \
+                                (S) = im_strdup( NULL, sst ); \
+                } \
+        } \
+} while( 0 ) 
 
 /* Duff's device. Do OPERation N times in a 16-way unrolled loop.
  */
@@ -203,7 +214,7 @@ extern "C" {
 		(SEQ)->underflow++;   \
 		(V) = 0;   \
 	}  \
-	if( (V) > UCHAR_MAX ) {   \
+	else if( (V) > UCHAR_MAX ) {   \
 		(SEQ)->overflow++;   \
 		(V) = UCHAR_MAX;   \
 	}  \
@@ -214,7 +225,7 @@ extern "C" {
 		(SEQ)->underflow++;   \
 		(V) = 0;   \
 	}  \
-	if( (V) > USHRT_MAX ) {   \
+	else if( (V) > USHRT_MAX ) {   \
 		(SEQ)->overflow++;   \
 		(V) = USHRT_MAX;   \
 	}  \
@@ -225,7 +236,7 @@ extern "C" {
 		(SEQ)->underflow++;   \
 		(V) = SCHAR_MIN;   \
 	}  \
-	if( (V) > SCHAR_MAX ) {   \
+	else if( (V) > SCHAR_MAX ) {   \
 		(SEQ)->overflow++;   \
 		(V) = SCHAR_MAX;   \
 	}  \
@@ -236,7 +247,7 @@ extern "C" {
 		(SEQ)->underflow++;   \
 		(V) = SHRT_MIN;   \
 	}  \
-	if( (V) > SHRT_MAX ) {   \
+	else if( (V) > SHRT_MAX ) {   \
 		(SEQ)->overflow++;   \
 		(V) = SHRT_MAX;   \
 	}  \
@@ -250,8 +261,11 @@ extern "C" {
 	((IMAGE *) im_local( (IM), \
 		(im_construct_fn) im_open, (im_callback_fn) im_close, \
 		(void *) (NAME), (void *) (MODE), NULL ))
+
+/* Strange double cast stops bogus warnings from gcc 4.1
+ */
 #define im_open_local_array( IM, OUT, N, NAME, MODE ) \
-	(im_local_array( (IM), (void **)(OUT), (N),\
+	(im_local_array( (IM), (void **)((void*)(OUT)), (N),\
 		(im_construct_fn) im_open, (im_callback_fn) im_close, \
 		(void *) (NAME), (void *) (MODE), NULL ))
 
@@ -259,28 +273,17 @@ extern "C" {
  */
 char *im__break_token( char *str, char *brk );
 
-/* Directory separator.
- */
-#ifdef OS_WIN32
-#define IM_DIR_SEP '\\'
-#define IM_DIR_SEP_STR "\\"
-#define IM_PATH_SEP ';'
-#define IM_PATH_SEP_STR ";"
-#else /*!OS_WIN32*/
-#define IM_DIR_SEP '/'
-#define IM_DIR_SEP_STR "/"
-#define IM_PATH_SEP ':'
-#define IM_PATH_SEP_STR ":"
-#endif /*OS_WIN32*/
-
 /* Like GFunc, but return a value.
  */
 typedef void *(*VSListMap2Fn)( void *, void *, void * );
+typedef void *(*VSListMap4Fn)( void *, void *, void *, void *, void * );
 typedef void *(*VSListFold2Fn)( void *, void *, void *, void * );
 
 gboolean im_slist_equal( GSList *l1, GSList *l2 );
 void *im_slist_map2( GSList *list, VSListMap2Fn fn, void *a, void *b );
 void *im_slist_map2_rev( GSList *list, VSListMap2Fn fn, void *a, void *b );
+void *im_slist_map4( GSList *list, 
+	VSListMap4Fn fn, void *a, void *b, void *c, void *d );
 void *im_slist_fold2( GSList *list, void *start, 
 	VSListFold2Fn fn, void *a, void *b );
 GSList *im_slist_filter( GSList *list, VSListMap2Fn fn, void *a, void *b );
@@ -294,7 +297,8 @@ char *im_strdup( IMAGE *im, const char *str );
 gboolean im_ispostfix( const char *a, const char *b );
 gboolean im_isprefix( const char *a, const char *b );
 int im_vsnprintf( char *str, size_t size, const char *format, va_list ap );
-int im_snprintf( char *str, size_t size, const char *format, ... );
+int im_snprintf( char *str, size_t size, const char *format, ... )
+	__attribute__((format(printf, 3, 4)));
 char *im_break_token( char *str, const char *brk );
 
 const char *im_skip_dir( const char *filename );
@@ -310,8 +314,9 @@ int im_local_array( IMAGE *im, void **out, int n,
 	im_construct_fn cons, im_callback_fn dest, void *a, void *b, void *c );
 
 gint64 im_file_length( int fd );
-char *im__file_read( FILE *fp, const char *name, int *length_out );
-char *im__file_read_name( const char *name, int *length_out );
+
+char *im__file_read( FILE *fp, const char *name, unsigned int *length_out );
+char *im__file_read_name( const char *name, unsigned int *length_out );
 #ifdef __cplusplus
 }
 #endif /*__cplusplus*/
