@@ -29,6 +29,8 @@
  * 27/9/04
  *	- im__cast_and_call() now matches bands as well
  *	- ... so 1 band + 4 band image -> 4 band image
+ * 8/12/06
+ * 	- add liboil support
  */
 
 /*
@@ -68,6 +70,11 @@
 #include <assert.h>
 
 #include <vips/vips.h>
+#include <vips/internal.h>
+
+#ifdef HAVE_LIBOIL
+#include <liboil/liboil.h>
+#endif /*HAVE_LIBOIL*/
 
 #ifdef WITH_DMALLOC
 #include <dmalloc.h>
@@ -125,7 +132,16 @@ add_buffer( PEL **in, PEL *out, int width, IMAGE *im )
         case IM_BANDFMT_USHORT: 	rloop( unsigned short ); break; 
         case IM_BANDFMT_INT: 		rloop( signed int ); break; 
         case IM_BANDFMT_UINT: 		rloop( unsigned int ); break; 
-        case IM_BANDFMT_FLOAT: 		rloop( float ); break; 
+
+        case IM_BANDFMT_FLOAT: 		
+#ifdef HAVE_LIBOIL
+		oil_add_f32( (float *) out, 
+			(float *) in[0], (float *) in[1], sz );
+#else /*!HAVE_LIBOIL*/
+		rloop( float ); 
+#endif /*HAVE_LIBOIL*/
+		break; 
+
         case IM_BANDFMT_DOUBLE:		rloop( double ); break; 
         case IM_BANDFMT_COMPLEX:	cloop( float ); break;
         case IM_BANDFMT_DPCOMPLEX:	cloop( double ); break;
@@ -161,21 +177,6 @@ static int iformat[6][6] = {
 /* I */  { I,  I,  I,  I,  I,  I }
 };
 
-/* Find the Bbits for a BandFmt.
- */
-static int bbits[10] = {
-	IM_BBITS_BYTE,		/* UCHAR */ 
-	IM_BBITS_BYTE, 		/* CHAR */
-	IM_BBITS_SHORT, 	/* USHORT */
-	IM_BBITS_SHORT, 	/* SHORT */
-	IM_BBITS_INT, 		/* UINT */
-	IM_BBITS_INT, 		/* INT */
-	IM_BBITS_FLOAT, 	/* FLOAT */ 
-	IM_BBITS_COMPLEX,	/* COMPLEX */  
-	IM_BBITS_DOUBLE, 	/* DOUBLE */
-	IM_BBITS_DPCOMPLEX	/* DPCOMPLEX */
-};
-
 /* Make an n-band image. Input 1 or n bands.
  */
 static int
@@ -215,7 +216,7 @@ im__cast_and_call( IMAGE *in1, IMAGE *in2, IMAGE *out,
 
 	/* Make sure bbits is set correctly.
 	 */
-	out->Bbits = bbits[ out->BandFmt ];
+	out->Bbits = im_bits_of_fmt( out->BandFmt );
 
 	/* Cast our input images up to the same type as the output.
 	 */
@@ -254,7 +255,7 @@ im_add( IMAGE *in1, IMAGE *in2, IMAGE *out )
 		im_error( "im_add", _( "not uncoded" ) );
 		return( -1 );
 	}
-	if( im_cp_desc( out, in1 ) )
+	if( im_cp_descv( out, in1, in2, NULL ) )
 		return( -1 );
 
 	/* What number of bands will we write?
