@@ -9,6 +9,8 @@
  *
  * 20/6/02 JC
  *	- adapted from im_abs()
+ * 8/12/06
+ * 	- add liboil support
  */
 
 /*
@@ -48,6 +50,11 @@
 #include <assert.h>
 
 #include <vips/vips.h>
+#include <vips/internal.h>
+
+#ifdef HAVE_LIBOIL
+#include <liboil/liboil.h>
+#endif /*HAVE_LIBOIL*/
 
 #ifdef WITH_DMALLOC
 #include <dmalloc.h>
@@ -71,7 +78,14 @@ floor_gen( PEL *in, PEL *out, int width, IMAGE *im )
 	int sz = width * im->Bands;
 
         switch( im->BandFmt ) {
-        case IM_BANDFMT_FLOAT: 		floor_loop(float); break; 
+        case IM_BANDFMT_FLOAT: 		
+#ifdef HAVE_LIBOIL
+		oil_floor_f32( (float *) out, (float *) in, sz );
+#else /*!HAVE_LIBOIL*/
+		floor_loop(float); 
+#endif /*HAVE_LIBOIL*/
+		break; 
+
         case IM_BANDFMT_DOUBLE:		floor_loop(double); break; 
         case IM_BANDFMT_COMPLEX:	sz *= 2; floor_loop(float); break;
         case IM_BANDFMT_DPCOMPLEX:	sz *= 2; floor_loop(double); break;
