@@ -30,6 +30,8 @@
  *	- cast up more quickly to help accuracy
  * 27/9/04
  *	- updated for 1 band $op n band image -> n band image case
+ * 8/12/06
+ * 	- add liboil support
  */
 
 /*
@@ -69,6 +71,11 @@
 #include <assert.h>
 
 #include <vips/vips.h>
+#include <vips/internal.h>
+
+#ifdef HAVE_LIBOIL
+#include <liboil/liboil.h>
+#endif /*HAVE_LIBOIL*/
 
 #ifdef WITH_DMALLOC
 #include <dmalloc.h>
@@ -126,7 +133,15 @@ subtract_buffer( PEL **in, PEL *out, int width, IMAGE *im )
         case IM_BANDFMT_USHORT: 	rloop( unsigned short ); break; 
         case IM_BANDFMT_INT: 		rloop( signed int ); break; 
         case IM_BANDFMT_UINT: 		rloop( unsigned int ); break; 
-        case IM_BANDFMT_FLOAT: 		rloop( float ); break; 
+        case IM_BANDFMT_FLOAT: 		
+#ifdef HAVE_LIBOIL
+		oil_subtract_f32( (float *) out, 
+			(float *) in[0], (float *) in[1], sz );
+#else /*!HAVE_LIBOIL*/
+		rloop( float ); 
+#endif /*HAVE_LIBOIL*/
+		break; 
+
         case IM_BANDFMT_DOUBLE:		rloop( double ); break; 
         case IM_BANDFMT_COMPLEX:	cloop( float ); break;
         case IM_BANDFMT_DPCOMPLEX:	cloop( double ); break;
@@ -171,7 +186,7 @@ im_subtract( IMAGE *in1, IMAGE *in2, IMAGE *out )
 		im_error( "im_subtract", _( "not uncoded" ) );
 		return( -1 );
 	}
-	if( im_cp_desc( out, in1 ) )
+	if( im_cp_descv( out, in1, in2, NULL ) )
 		return( -1 );
 
 	/* What number of bands will we write?
