@@ -98,7 +98,7 @@ andconst_vec( im_object *argv )
 /* Description of im_andconst.
  */ 
 static im_function andconst_desc = {
-	"im_andconst", 			/* Name */
+	"im_andimageconst", 			/* Name */
 	"bitwise and of an image with a constant",
 	IM_FN_PTOP | IM_FN_PIO,			/* Flags */
 	andconst_vec, 			/* Dispatch function */
@@ -119,7 +119,7 @@ and_vec_vec( im_object *argv )
 /* Description of im_andconst.
  */ 
 static im_function and_vec_desc = {
-	"im_and_vec", 		/* Name */
+	"im_andimage_vec", 		/* Name */
 	"bitwise and of an image with a vector constant",
 	IM_FN_PTOP | IM_FN_PIO,		/* Flags */
 	and_vec_vec, 		/* Dispatch function */
@@ -159,7 +159,7 @@ orconst_vec( im_object *argv )
 /* Description of im_orconst.
  */ 
 static im_function orconst_desc = {
-	"im_orconst", 			/* Name */
+	"im_orimageconst", 			/* Name */
 	"bitwise or of an image with a constant",
 	IM_FN_PTOP | IM_FN_PIO,			/* Flags */
 	orconst_vec, 			/* Dispatch function */
@@ -180,7 +180,7 @@ or_vec_vec( im_object *argv )
 /* Description of im_orconst.
  */ 
 static im_function or_vec_desc = {
-	"im_or_vec", 		/* Name */
+	"im_orimage_vec", 		/* Name */
 	"bitwise or of an image with a vector constant",
 	IM_FN_PTOP | IM_FN_PIO,		/* Flags */
 	or_vec_vec, 		/* Dispatch function */
@@ -220,7 +220,7 @@ eorconst_vec( im_object *argv )
 /* Description of im_eorconst.
  */ 
 static im_function eorconst_desc = {
-	"im_eorconst", 			/* Name */
+	"im_eorimageconst", 			/* Name */
 	"bitwise eor of an image with a constant",
 	IM_FN_PTOP | IM_FN_PIO,			/* Flags */
 	eorconst_vec, 			/* Dispatch function */
@@ -241,7 +241,7 @@ eor_vec_vec( im_object *argv )
 /* Description of im_eorconst.
  */ 
 static im_function eor_vec_desc = {
-	"im_eor_vec", 		/* Name */
+	"im_eorimage_vec", 		/* Name */
 	"bitwise eor of an image with a vector constant",
 	IM_FN_PTOP | IM_FN_PIO,		/* Flags */
 	eor_vec_vec, 		/* Dispatch function */
