@@ -92,7 +92,7 @@ im_dECMC_fromLab( IMAGE *im1, IMAGE *im2, IMAGE *out )
 
 	/* Prepare the output image 
 	 */
-	if( im_cp_desc( out, im1 ) ) 
+	if( im_cp_descv( out, im1, im2, NULL ) )
 		return( -1 );
 	out->Bbits = IM_BBITS_FLOAT;
 	out->Bands = 1;
