@@ -16,6 +16,11 @@
  * 	- ANSI
  * 31/8/93 JC
  *	- calculation of offset now includes scale
+ * 8/5/06
+ * 	- set Type on output too
+ * 16/10/06
+ * 	- what? no, don't set Type, useful to be able to scale histograms, for
+ * 	  example
  */
 
 /*
@@ -85,9 +90,8 @@ im_scale( IMAGE *in, IMAGE *out )
 
 	/* Transform!
 	 */
-	if( im_lintra( scale, in, offset, t ) )
-		return( -1 );
-	if( im_clip( t, out ) )
+	if( im_lintra( scale, in, offset, t ) ||
+		im_clip( t, out ) )
 		return( -1 );
 
 	return( 0 );
