@@ -1,4 +1,4 @@
-/* Convert 1 or 3-band 8-bit VIPS images to/from JPEG.
+/* Convert 8-bit VIPS images to/from JPEG.
  *
  * 28/11/03 JC
  *	- better no-overshoot on tile loop
@@ -21,6 +21,10 @@
  * 	- add </libexif/ prefix if required
  * 19/1/07
  * 	- oop, libexif confusion
+ * 2/11/07
+ * 	- use im_wbuffer() API for BG writes
+ * 15/2/08
+ * 	- write CMYK if Bands == 4 and Type == CMYK
  */
 
 /*
@@ -188,11 +192,11 @@ typedef struct {
 	IMAGE *in;
 	struct jpeg_compress_struct cinfo;
         ErrorManager eman;
-	REGION *reg;
 	im_threadgroup_t *tg;
 	JSAMPROW *row_pointer;
 	char *profile_bytes;
 	unsigned int profile_length;
+	IMAGE *inverted;
 } Write;
 
 static void
@@ -200,11 +204,11 @@ write_destroy( Write *write )
 {
 	jpeg_destroy_compress( &write->cinfo );
 	IM_FREEF( im_threadgroup_free, write->tg );
-	IM_FREEF( im_region_free, write->reg );
 	IM_FREEF( im_close, write->in );
 	IM_FREEF( fclose, write->eman.fp );
 	IM_FREE( write->row_pointer );
 	IM_FREE( write->profile_bytes );
+	IM_FREEF( im_close, write->inverted );
 	im_free( write );
 }
 
@@ -217,30 +221,23 @@ write_new( IMAGE *in )
 		return( NULL );
 	memset( write, 0, sizeof( Write ) );
 
-	if( !(write->in = im__convert_saveable( in, FALSE )) ) {
+	if( !(write->in = im__convert_saveable( in, IM__RGB_CMYK )) ) {
 		im_error( "im_vips2jpeg", 
-			_( "unable to convert to RGB for save" ) );
+			_( "unable to convert to saveable format" ) );
 		write_destroy( write );
 		return( NULL );
 	}
 
-	write->reg = im_region_create( write->in );
-	write->tg = im_threadgroup_create( write->in );
-
-	write->row_pointer = IM_ARRAY( NULL, write->tg->nlines, JSAMPROW );
-
+	write->tg = NULL;
+	write->row_pointer = NULL;
         write->cinfo.err = jpeg_std_error( &write->eman.pub );
 	write->eman.pub.error_exit = new_error_exit;
 	write->eman.pub.output_message = new_output_message;
 	write->eman.fp = NULL;
 	write->profile_bytes = NULL;
 	write->profile_length = 0;
+	write->inverted = NULL;
 
-	if( !write->reg || !write->tg || !write->row_pointer ) {
-		write_destroy( write );
-		return( NULL );
-	}
-	
         return( write );
 }
 
@@ -515,21 +512,44 @@ write_profile_meta( Write *write )
 	return( 0 );
 }
 
+static int
+write_jpeg_block( REGION *region, Rect *area, void *a, void *b )
+{
+	Write *write = (Write *) a;
+	int i;
+
+	/* We are running in a background thread. We need to catch longjmp()s
+	 * here instead.
+	 */
+	if( setjmp( write->eman.jmp ) ) 
+		return( -1 );
+
+	for( i = 0; i < area->height; i++ )
+		write->row_pointer[i] = (JSAMPROW) 
+			IM_REGION_ADDR( region, 0, area->top + i );
+
+	jpeg_write_scanlines( &write->cinfo, write->row_pointer, area->height );
+
+	return( 0 );
+}
+
 /* Write a VIPS image to a JPEG compress struct.
  */
 static int
 write_vips( Write *write, int qfac, const char *profile )
 {
-	IMAGE *in = write->in;
+	IMAGE *in;
+	J_COLOR_SPACE space;
 
-	Rect area;
-	int y, i;
+	/* The image we'll be writing ... can change, see CMYK.
+	 */
+	in = write->in;
 
 	/* Should have been converted for save.
 	 */
         assert( in->BandFmt == IM_BANDFMT_UCHAR );
 	assert( in->Coding == IM_CODING_NONE );
-        assert( in->Bands == 1 || in->Bands == 3 );
+        assert( in->Bands == 1 || in->Bands == 3 || in->Bands == 4 );
 
         /* Check input image.
          */
@@ -544,14 +564,32 @@ write_vips( Write *write, int qfac, const char *profile )
 	 */
         write->cinfo.image_width = in->Xsize;
         write->cinfo.image_height = in->Ysize;
-	if( in->Bands == 3 ) {
-		write->cinfo.input_components = 3;
-		write->cinfo.in_color_space = JCS_RGB; 
-	}
-	else if( in->Bands == 1 ) {
-		write->cinfo.input_components = 1;
-		write->cinfo.in_color_space = JCS_GRAYSCALE; 
+	write->cinfo.input_components = in->Bands;
+	if( in->Bands == 4 && in->Type == IM_TYPE_CMYK ) {
+		space = JCS_CMYK;
+		/* IJG always sets an Adobe marker, so we should invert CMYK.
+		 */
+		if( !(write->inverted = im_open( "vips2jpeg_invert", "p" )) ||
+			im_invert( in, write->inverted ) )
+			return( -1 );
+		in = write->inverted;
 	}
+	else if( in->Bands == 3 )
+		space = JCS_RGB;
+	else if( in->Bands == 1 )
+		space = JCS_GRAYSCALE;
+	else 
+		/* Use luminance compression for all channels.
+		 */
+		space = JCS_UNKNOWN;
+	write->cinfo.in_color_space = space; 
+
+	/* Build VIPS output stuff now we know the image we'll be writing.
+	 */
+	write->tg = im_threadgroup_create( in );
+	write->row_pointer = IM_ARRAY( NULL, write->tg->nlines, JSAMPROW );
+	if( !write->tg || !write->row_pointer ) 
+		return( -1 );
 
 	/* Rest to default. 
 	 */
@@ -577,24 +615,15 @@ write_vips( Write *write, int qfac, const char *profile )
 		write_profile_meta( write ) )
 		return( -1 );
 
-	/* Write data.
+	/* Write data. Note that the write function grabs the longjmp()!
 	 */
-	for( y = 0; y < in->Ysize; y += write->tg->nlines ) {
-		area.left = 0;
-		area.top = y;
-		area.width = in->Xsize;
-		area.height = IM_MIN( write->tg->nlines, in->Ysize - y );
+	if( im_wbuffer( write->tg, write_jpeg_block, write, NULL ) )
+		return( -1 );
 
-                if( im_prepare_thread( write->tg, write->reg, &area ) )
-                        return( -1 );
-
-		for( i = 0; i < area.height; i++ )
-			write->row_pointer[i] = (JSAMPROW) 
-				IM_REGION_ADDR( write->reg, 0, y + i );
-
-		jpeg_write_scanlines( &write->cinfo, 
-			write->row_pointer, area.height );
-	}
+	/* We have to reinstate the setjmp() before we jpeg_finish_compress().
+	 */
+	if( setjmp( write->eman.jmp ) ) 
+		return( -1 );
 
 	jpeg_finish_compress( &write->cinfo );
 
