@@ -91,6 +91,19 @@
  * 18/7/07 Andrey Kiselev
  * 	- remove "b" option on TIFFOpen()
  * 	- support TIFFTAG_PREDICTOR types for lzw and deflate compression
+ * 3/11/07
+ * 	- use im_wbuffer() for background writes
+ * 15/2/08
+ * 	- set TIFFTAG_JPEGQUALITY explicitly when we copy TIFF files, since 
+ * 	  libtiff doesn't keep this in the header (thanks Joe)
+ * 20/2/08
+ * 	- use tiff error handler from im_tiff2vips.c
+ * 27/2/08
+ * 	- don't try to copy icc profiles when building pyramids (thanks Joe)
+ * 9/4/08
+ * 	- use IM_META_RESOLUTION_UNIT to set default resunit
+ * 17/4/08
+ * 	- allow CMYKA (thanks Doron)
  */
 
 /*
@@ -232,14 +245,10 @@ typedef struct tiff_write {
 	char *icc_profile;		/* Profile to embed */
 } TiffWrite;
 
-/* Handle TIFF errors here. 
+/* Use these from im_tiff2vips().
  */
-static void 
-vhandle( char *module, char *fmt, va_list ap )
-{
-	im_error( "im_vips2tiff", _( "TIFF error in \"%s\": " ), module );
-	im_verror( "im_vips2tiff", fmt, ap );
-}
+void im__thandler_error( char *module, char *fmt, va_list ap );
+void im__thandler_warning( char *module, char *fmt, va_list ap );
 
 /* Open TIFF for output.
  */
@@ -364,10 +373,10 @@ pack2tiff( TiffWrite *tw, REGION *in, PEL *q, Rect *area )
 	}
 }
 
-/* Embed an ICC profile.
+/* Embed an ICC profile from a file.
  */
 static int
-embed_profile( TIFF *tif, const char *profile )
+embed_profile_file( TIFF *tif, const char *profile )
 {
 	char *buffer;
 	unsigned int length;
@@ -403,6 +412,18 @@ embed_profile_meta( TIFF *tif, IMAGE *im )
 	return( 0 );
 }
 
+static int
+embed_profile( TiffWrite *tw, TIFF *tif )
+{
+	if( tw->embed && embed_profile_file( tif, tw->icc_profile ) )
+		return( -1 );
+	if( !tw->embed && im_header_get_type( tw->im, IM_META_ICC_NAME ) &&
+		embed_profile_meta( tif, tw->im ) )
+		return( -1 );
+
+	return( 0 );
+}
+
 /* Write a TIFF header. width and height are the size of the IMAGE we are
  * writing (may have been shrunk!).
  */
@@ -427,20 +448,15 @@ write_tiff_header( TiffWrite *tw, TIFF *tif, int width, int height )
 	TIFFSetField( tif, TIFFTAG_YRESOLUTION, 
 		IM_CLIP( 0.01, tw->yres, 10000 ) );
 
-	if( tw->compression == COMPRESSION_JPEG ) {
+	if( tw->compression == COMPRESSION_JPEG ) 
 		TIFFSetField( tif, TIFFTAG_JPEGQUALITY, tw->jpqual );
-		TIFFSetField( tif, TIFFTAG_JPEGCOLORMODE, JPEGCOLORMODE_RGB );
-	}
 
 	if( tw->predictor != -1 ) 
 		TIFFSetField( tif, TIFFTAG_PREDICTOR, tw->predictor );
 
 	/* Attach ICC profile.
 	 */
-	if( tw->embed && embed_profile( tif, tw->icc_profile ) )
-		return( -1 );
-	if( !tw->embed && im_header_get_type( tw->im, IM_META_ICC_NAME ) &&
-		embed_profile_meta( tif, tw->im ) )
+	if( embed_profile( tw, tif ) )
 		return( -1 );
 
 	/* And colour fields.
@@ -492,6 +508,14 @@ write_tiff_header( TiffWrite *tw, TIFF *tif, int width, int height )
 			}
 			break;
 
+		case 5:
+			if( tw->im->Type == IM_TYPE_CMYK ) {
+				photometric = PHOTOMETRIC_SEPARATED;
+				TIFFSetField( tif, 
+					TIFFTAG_INKSET, INKSET_CMYK );
+			}
+			break;
+
 		default:
 			assert( 0 );
 		}
@@ -1037,57 +1061,52 @@ write_tif_tile( TiffWrite *tw )
 	return( 0 );
 }
 
+static int
+write_tif_block( REGION *region, Rect *area, void *a, void *b )
+{
+	TiffWrite *tw = (TiffWrite *) a;
+	IMAGE *im = tw->im;
+
+	int y;
+
+	for( y = 0; y < area->height; y++ ) {
+		PEL *p = (PEL *) IM_REGION_ADDR( region, 0, area->top + y );
+
+		/* Any repacking necessary.
+		 */
+		if( im->Coding == IM_CODING_LABQ ) {
+			LabQ2LabC( tw->tbuf, p, im->Xsize );
+			p = tw->tbuf;
+		}
+		else if( im->BandFmt == IM_BANDFMT_SHORT &&
+			im->Type == IM_TYPE_LABS ) {
+			LabS2Lab16( tw->tbuf, p, im->Xsize );
+			p = tw->tbuf;
+		}
+		else if( tw->onebit ) {
+			eightbit2onebit( tw->tbuf, p, im->Xsize );
+			p = tw->tbuf;
+		}
+
+		if( TIFFWriteScanline( tw->tif, p, area->top + y, 0 ) < 0 ) 
+			return( -1 );
+	}
+
+	return( 0 );
+}
+
 /* Write as scan-lines.
  */
 static int
 write_tif_strip( TiffWrite *tw )
 {
-	IMAGE *im = tw->im;
-	Rect area;
-	int y, y2;
+	g_assert( !tw->tbuf );
 
 	if( !(tw->tbuf = im_malloc( NULL, TIFFScanlineSize( tw->tif ) )) ) 
 		return( -1 );
 
-	/* Set up rect we ask for. 
-	 */
-	area.left = 0;
-	area.width = im->Xsize;
-
-	for( y = 0; y < im->Ysize; y += tw->tg->nlines ) {
-		area.top = y;
-		area.height = IM_MIN( tw->tg->nlines, im->Ysize - y );
-		if( im_prepare_thread( tw->tg, tw->reg, &area ) )
-			return( -1 );
-
-		for( y2 = 0; y2 < area.height; y2++ ) {
-			PEL *p = (PEL *) IM_REGION_ADDR( tw->reg, 0, y + y2 );
-
-			/* Any repacking necessary.
-			 */
-			if( im->Coding == IM_CODING_LABQ ) {
-				LabQ2LabC( tw->tbuf, p, im->Xsize );
-				p = tw->tbuf;
-			}
-			else if( im->BandFmt == IM_BANDFMT_SHORT &&
-				im->Type == IM_TYPE_LABS ) {
-				LabS2Lab16( tw->tbuf, p, im->Xsize );
-				p = tw->tbuf;
-			}
-			else if( tw->onebit ) {
-				eightbit2onebit( tw->tbuf, p, im->Xsize );
-				p = tw->tbuf;
-			}
-
-			/* Write to TIFF! easy.
-			 */
-			if( TIFFWriteScanline( tw->tif, p, y + y2, 0 ) < 0 ) {
-				im_error( "im_vips2tiff", 
-					_( "TIFF write failed" ) );
-				return( -1 );
-			}
-		}
-	}
+	if( im_wbuffer( tw->tg, write_tif_block, tw, NULL ) )
+		return( -1 );
 
 	return( 0 );
 }
@@ -1186,11 +1205,17 @@ make_tiff_write( IMAGE *im, const char *filename )
 	tw->embed = 0;
 	tw->icc_profile = NULL;
 
-	/* Output resolution settings default to VIPS-alike.
+	/* Output resolution settings ... default to VIPS-alike.
 	 */
 	tw->resunit = RESUNIT_CENTIMETER;
 	tw->xres = im->Xres * 10;
 	tw->yres = im->Yres * 10;
+	if( !im_meta_get_string( im, IM_META_RESOLUTION_UNIT, &p ) &&
+		strcmp( p, "in" ) == 0 ) {
+		tw->resunit = RESUNIT_INCH;
+		tw->xres *= 2.54;
+		tw->yres *= 2.54;
+	}
 
 	/* Parse mode string.
 	 */
@@ -1306,10 +1331,20 @@ make_tiff_write( IMAGE *im, const char *filename )
 		}
 	}
 	if( (q = im_getnextoption( &p )) ) {
-		if( im_isprefix( "res_cm", q ) ) 
+		if( im_isprefix( "res_cm", q ) ) {
+			if( tw->resunit == RESUNIT_INCH ) {
+				tw->xres /= 2.54;
+				tw->yres /= 2.54;
+			}
 			tw->resunit = RESUNIT_CENTIMETER;
-		else if( im_isprefix( "res_inch", q ) ) 
+		}
+		else if( im_isprefix( "res_inch", q ) ) {
+			if( tw->resunit == RESUNIT_CENTIMETER ) {
+				tw->xres *= 2.54;
+				tw->yres *= 2.54;
+			}
 			tw->resunit = RESUNIT_INCH;
+		}
 		else {
 			im_error( "im_vips2tiff", _( "unknown resolution unit "
 				"\"%s\"\nshould be one of \"res_cm\" or "
@@ -1399,11 +1434,10 @@ make_tiff_write( IMAGE *im, const char *filename )
  * we might have set.
  */
 static int
-tiff_copy( TIFF *out, TIFF *in )
+tiff_copy( TiffWrite *tw, TIFF *out, TIFF *in )
 {
-	uint32 i32, i32a;
+	uint32 i32;
 	uint16 i16;
-	int i;
 	float f;
 	tdata_t buf;
 	ttile_t tile;
@@ -1415,12 +1449,10 @@ tiff_copy( TIFF *out, TIFF *in )
 	CopyField( TIFFTAG_IMAGELENGTH, i32 );
 	CopyField( TIFFTAG_PLANARCONFIG, i16 );
 	CopyField( TIFFTAG_ORIENTATION, i16 );
-	CopyField( TIFFTAG_COMPRESSION, i16 );
 	CopyField( TIFFTAG_XRESOLUTION, f );
 	CopyField( TIFFTAG_YRESOLUTION, f );
 	CopyField( TIFFTAG_RESOLUTIONUNIT, i16 );
-	CopyField( TIFFTAG_JPEGQUALITY, i );
-        CopyField( TIFFTAG_PREDICTOR, i16 );
+	CopyField( TIFFTAG_COMPRESSION, i16 );
 	CopyField( TIFFTAG_SAMPLESPERPIXEL, i16 );
 	CopyField( TIFFTAG_BITSPERSAMPLE, i16 );
 	CopyField( TIFFTAG_PHOTOMETRIC, i16 );
@@ -1428,14 +1460,29 @@ tiff_copy( TIFF *out, TIFF *in )
 	CopyField( TIFFTAG_TILELENGTH, i32 );
 	CopyField( TIFFTAG_ROWSPERSTRIP, i32 );
 
-	if( TIFFGetField( in, TIFFTAG_ICCPROFILE, &i32, &i32a ) ) 
-		TIFFSetField( out, TIFFTAG_ICCPROFILE, i32, i32a );
+	if( tw->predictor != -1 ) 
+		TIFFSetField( out, TIFFTAG_PREDICTOR, tw->predictor );
+
+	/* TIFFTAG_JPEGQUALITY is a pesudo-tag, so we can't copy it.
+	 * Set explicitly from TiffWrite.
+	 */
+	if( tw->compression == COMPRESSION_JPEG ) 
+		TIFFSetField( out, TIFFTAG_JPEGQUALITY, tw->jpqual );
+
+	/* We can't copy profiles :( Set again from TiffWrite.
+	 */
+	if( embed_profile( tw, out ) )
+		return( -1 );
 
 	buf = im_malloc( NULL, TIFFTileSize( in ) );
 	n = TIFFNumberOfTiles( in );
 	for( tile = 0; tile < n; tile++ ) {
 		tsize_t len;
 
+		/* It'd be good to use TIFFReadRawTile()/TIFFWriteRawTile() 
+		 * here to save compression/decompression, but sadly it seems
+		 * not to work :-( investigate at some point.
+		 */
 		len = TIFFReadEncodedTile( in, tile, buf, (tsize_t) -1 );
 		if( len < 0 ||
 			TIFFWriteEncodedTile( out, tile, buf, len ) < 0 ) {
@@ -1451,14 +1498,14 @@ tiff_copy( TIFF *out, TIFF *in )
 /* Append a file to a TIFF file.
  */
 static int
-tiff_append( TIFF *out, const char *name )
+tiff_append( TiffWrite *tw, TIFF *out, const char *name )
 {
 	TIFF *in;
 
 	if( !(in = tiff_openin( name )) ) 
 		return( -1 );
 
-	if( tiff_copy( out, in ) ) {
+	if( tiff_copy( tw, out, in ) ) {
 		TIFFClose( in );
 		return( -1 );
 	}
@@ -1485,13 +1532,13 @@ gather_pyramid( TiffWrite *tw )
 	if( !(out = tiff_openout( tw->name )) ) 
 		return( -1 );
 
-	if( tiff_append( out, tw->bname ) ) {
+	if( tiff_append( tw, out, tw->bname ) ) {
 		TIFFClose( out );
 		return( -1 );
 	}
 
 	for( layer = tw->layer; layer; layer = layer->below ) 
-		if( tiff_append( out, layer->lname ) ) {
+		if( tiff_append( tw, out, layer->lname ) ) {
 			TIFFClose( out );
 			return( -1 );
 		}
@@ -1517,7 +1564,8 @@ im_vips2tiff( IMAGE *im, const char *filename )
 
 	/* Override the default TIFF error handler.
 	 */
-	TIFFSetErrorHandler( (TIFFErrorHandler) vhandle );
+	TIFFSetErrorHandler( (TIFFErrorHandler) im__thandler_error );
+	TIFFSetWarningHandler( (TIFFErrorHandler) im__thandler_warning );
 
 	/* Check input image.
 	 */
@@ -1537,8 +1585,8 @@ im_vips2tiff( IMAGE *im, const char *filename )
 		return( -1 );
 	}
 	if( im->Coding == IM_CODING_NONE ) {
-		if( im->Bands < 1 || im->Bands > 4 ) {
-			im_error( "im_vips2tiff", _( "1 to 4 bands only" ) );
+		if( im->Bands < 1 || im->Bands > 5 ) {
+			im_error( "im_vips2tiff", _( "1 to 5 bands only" ) );
 			return( -1 );
 		}
 	}
