@@ -152,6 +152,25 @@ static im_function contrast_surface_raw_desc = {
 	contrast_surface_raw_args 	/* Arg list */
 };
 
+/* Call im_phasecor_fft via arg vector.
+ */
+static int
+phasecor_fft_vec( im_object *argv )
+{
+	return( im_phasecor_fft( argv[0], argv[1], argv[2] ) );
+}
+
+/* Description of im_phasecor_fft.
+ */ 
+static im_function phasecor_fft_desc = {
+	"im_phasecor_fft",	 		/* Name */
+	"non-normalised correlation of gradient of in2 within in1",
+	IM_FN_TRANSFORM,	/* Flags */
+	phasecor_fft_vec, 			/* Dispatch function */
+	IM_NUMBER( two_in_one_out ), 	/* Size of arg list */
+	two_in_one_out 			/* Arg list */
+};
+
 /* Args to im_rank.
  */
 static im_arg_desc rank_args[] = {
@@ -702,7 +721,7 @@ grad_x_vec( im_object *argv )
  */ 
 static im_function grad_x_desc = {
 	"im_grad_x",	 		/* Name */
-	"x component of gradient of image",
+	"horizontal difference image",
 	IM_FN_PIO | IM_FN_TRANSFORM,	/* Flags */
 	grad_x_vec, 			/* Dispatch function */
 	IM_NUMBER( one_in_one_out ), 	/* Size of arg list */
@@ -721,7 +740,7 @@ grad_y_vec( im_object *argv )
  */ 
 static im_function grad_y_desc = {
 	"im_grad_y",	 		/* Name */
-	"y component of gradient of image",
+	"vertical difference image",
 	IM_FN_PIO | IM_FN_TRANSFORM,	/* Flags */
 	grad_y_vec, 			/* Dispatch function */
 	IM_NUMBER( one_in_one_out ), 	/* Size of arg list */
@@ -1007,44 +1026,6 @@ static im_function spcor_raw_desc = {
 	two_in_one_out 			/* Arg list */
 };
 
-/* Call im_spcor2 via arg vector.
- */
-static int
-spcor2_vec( im_object *argv )
-{
-	return( im_spcor2( argv[0], argv[1], argv[2] ) );
-}
-
-/* Description of im_spcor2.
- */ 
-static im_function spcor2_desc = {
-	"im_spcor2",	 		/* Name */
-	"normalised correlation of in2 within in1",
-	IM_FN_PIO | IM_FN_TRANSFORM,	/* Flags */
-	spcor2_vec, 			/* Dispatch function */
-	IM_NUMBER( two_in_one_out ), 	/* Size of arg list */
-	two_in_one_out 			/* Arg list */
-};
-
-/* Call im_spcor2_raw via arg vector.
- */
-static int
-spcor2_raw_vec( im_object *argv )
-{
-	return( im_spcor2_raw( argv[0], argv[1], argv[2] ) );
-}
-
-/* Description of im_spcor2_raw.
- */ 
-static im_function spcor2_raw_desc = {
-	"im_spcor2_raw", 		/* Name */
-	"normalised correlation of in2 within in1, no black padding",
-	IM_FN_PIO | IM_FN_TRANSFORM,	/* Flags */
-	spcor2_raw_vec, 		/* Dispatch function */
-	IM_NUMBER( two_in_one_out ), 	/* Size of arg list */
-	two_in_one_out 			/* Arg list */
-};
-
 /* Args for im_zerox.
  */
 static im_arg_desc zerox_args[] = {
@@ -1380,6 +1361,7 @@ static im_function *convol_list[] = {
 	&log_imask_desc,
 	&maxvalue_desc,
 	&mpercent_desc,
+	&phasecor_fft_desc,
 	&rank_desc,
 	&rank_raw_desc,
 	&read_dmask_desc,
@@ -1392,8 +1374,6 @@ static im_function *convol_list[] = {
 	&shrink_desc,
 	&spcor_desc,
 	&spcor_raw_desc,
-	&spcor2_desc,
-	&spcor2_raw_desc,
 	&stretch3_desc,
 	&zerox_desc
 };
