@@ -59,6 +59,11 @@
  *	- sets Xoffset / Yoffset
  * 11/11/05
  * 	- simpler inner loop avoids gcc4 bug 
+ * 7/11/07
+ * 	- new evalstart/end callbacks
+ * 12/5/08
+ * 	- int rounding was +1 too much, argh
+ * 	- only rebuild the buffer offsets if bpl changes
  */
 
 /*
@@ -119,19 +124,34 @@ typedef struct {
 } Conv;
 
 static int
-conv_destroy( Conv *conv )
+conv_close( Conv *conv )
+{
+	IM_FREEF( im_free_imask, conv->mask );
+
+        return( 0 );
+}
+
+static int
+conv_evalstart( Conv *conv )
+{
+	/* Reset underflow/overflow count.
+	 */
+	conv->overflow = 0;
+	conv->underflow = 0;
+
+        return( 0 );
+}
+
+static int
+conv_evalend( Conv *conv )
 {
 	/* Print underflow/overflow count.
 	 */
 	if( conv->overflow || conv->underflow )
-		im_warning( "im_conv: %d overflows and %d underflows detected",
+		im_warn( "im_conv", 
+			_( "%d overflows and %d underflows detected" ),
 			conv->overflow, conv->underflow );
 
-	if( conv->mask ) {
-		(void) im_free_imask( conv->mask );
-		conv->mask = NULL;
-	}
-
         return( 0 );
 }
 
@@ -154,7 +174,11 @@ conv_new( IMAGE *in, IMAGE *out, INTMASK *mask )
         conv->overflow = 0;
 
         if( im_add_close_callback( out, 
-		(im_callback_fn) conv_destroy, conv, NULL ) ||
+		(im_callback_fn) conv_close, conv, NULL ) ||
+		im_add_close_callback( out, 
+			(im_callback_fn) conv_evalstart, conv, NULL ) ||
+		im_add_close_callback( out, 
+			(im_callback_fn) conv_evalend, conv, NULL ) ||
         	!(conv->coeff = IM_ARRAY( out, ne, int )) ||
         	!(conv->mask = im_dup_imask( mask, "conv_mask" )) )
                 return( NULL );
@@ -179,24 +203,24 @@ typedef struct {
 
 	int underflow;		/* Underflow/overflow counts */
 	int overflow;
+
+	int last_bpl;		/* Avoid recalcing offsets, if we can */
 } ConvSequence;
 
 /* Free a sequence value.
  */
 static int
-stop_conv( ConvSequence *seq, IMAGE *in, Conv *conv )
+conv_stop( void *vseq, void *a, void *b )
 {
+	ConvSequence *seq = (ConvSequence *) vseq;
+	Conv *conv = (Conv *) b;
+
 	/* Add local under/over counts to global counts.
 	 */
 	conv->overflow += seq->overflow;
 	conv->underflow += seq->underflow;
 
-	/* Free attached objects.
-	 */
-	if( seq->ir ) {
-		im_region_free( seq->ir );
-		seq->ir = NULL;
-	}
+	IM_FREEF( im_region_free, seq->ir );
 
 	return( 0 );
 }
@@ -204,11 +228,13 @@ stop_conv( ConvSequence *seq, IMAGE *in, Conv *conv )
 /* Convolution start function.
  */
 static void *
-start_conv( IMAGE *out, IMAGE *in, Conv *conv )
+conv_start( IMAGE *out, void *a, void *b )
 {
-	ConvSequence *seq = IM_NEW( out, ConvSequence );
+	IMAGE *in = (IMAGE *) a;
+	Conv *conv = (Conv *) b;
+	ConvSequence *seq;
 
-	if( !seq ) 
+	if( !(seq = IM_NEW( out, ConvSequence )) )
 		return( NULL );
 
 	/* Init!
@@ -218,6 +244,7 @@ start_conv( IMAGE *out, IMAGE *in, Conv *conv )
 	seq->pts = NULL;
 	seq->underflow = 0;
 	seq->overflow = 0;
+	seq->last_bpl = -1;
 
 	/* Attach region and arrays.
 	 */
@@ -225,11 +252,11 @@ start_conv( IMAGE *out, IMAGE *in, Conv *conv )
 	seq->offsets = IM_ARRAY( out, conv->nnz, int );
 	seq->pts = IM_ARRAY( out, conv->nnz, PEL * );
 	if( !seq->ir || !seq->offsets || !seq->pts ) {
-		stop_conv( seq, in, conv );
+		conv_stop( seq, in, conv );
 		return( NULL );
 	}
 
-	return( (void *) seq );
+	return( seq );
 }
 
 #define INNER sum += *t++ * (*p++)[x]
@@ -275,11 +302,18 @@ start_conv( IMAGE *out, IMAGE *in, Conv *conv )
 /* Convolve!
  */
 static int
-gen_conv( REGION *or, ConvSequence *seq, IMAGE *in, Conv *conv )
+conv_gen( REGION *or, void *vseq, void *a, void *b )
 {
+	ConvSequence *seq = (ConvSequence *) vseq;
+	IMAGE *in = (IMAGE *) a;
+	Conv *conv = (Conv *) b;
 	REGION *ir = seq->ir;
 	INTMASK *mask = conv->mask;
-	int rounding = (mask->scale + 1)/2;
+
+	/* You might think this should be (scale+1)/2, but then we'd be adding
+	 * one for scale == 1.
+	 */
+	int rounding = mask->scale / 2;
 
 	Rect *r = &or->valid;
 	Rect s;
@@ -299,15 +333,21 @@ gen_conv( REGION *or, ConvSequence *seq, IMAGE *in, Conv *conv )
 	if( im_prepare( ir, &s ) )
 		return( -1 );
 
-        /* Fill offset array.
-         */
-	z = 0;
-        for( i = 0, y = 0; y < mask->ysize; y++ )
-                for( x = 0; x < mask->xsize; x++, i++ )
-                        if( mask->coeff[i] )
-                                seq->offsets[z++] = 
-					IM_REGION_ADDR( ir, x + le, y + to ) -
-                                        IM_REGION_ADDR( ir, le, to );
+        /* Fill offset array. Only do this if the bpl has changed since the 
+	 * previous im_prepare().
+	 */
+	if( seq->last_bpl != IM_REGION_LSKIP( ir ) ) {
+		seq->last_bpl = IM_REGION_LSKIP( ir );
+
+		z = 0;
+		for( i = 0, y = 0; y < mask->ysize; y++ )
+			for( x = 0; x < mask->xsize; x++, i++ )
+				if( mask->coeff[i] )
+					seq->offsets[z++] = 
+						IM_REGION_ADDR( ir, 
+							x + le, y + to ) -
+						IM_REGION_ADDR( ir, le, to );
+	}
 
 	for( y = to; y < bo; y++ ) { 
 		/* Init pts for this line of PELs.
@@ -397,7 +437,7 @@ im_conv_raw( IMAGE *in, IMAGE *out, INTMASK *mask )
 	if( im_demand_hint( out, IM_FATSTRIP, in, NULL ) )
 		return( -1 );
 
-	if( im_generate( out, start_conv, gen_conv, stop_conv, in, conv ) )
+	if( im_generate( out, conv_start, conv_gen, conv_stop, in, conv ) )
 		return( -1 );
 
 	out->Xoffset = -mask->xsize / 2;
