@@ -92,7 +92,7 @@ typedef struct {
 } Conv;
 
 static int
-conv_destroy( Conv *conv )
+conv_close( Conv *conv )
 {
 	if( conv->mask ) {
 		(void) im_free_dmask( conv->mask );
@@ -119,7 +119,7 @@ conv_new( IMAGE *in, IMAGE *out, DOUBLEMASK *mask )
         conv->coeff = NULL;
 
         if( im_add_close_callback( out, 
-		(im_callback_fn) conv_destroy, conv, NULL ) ||
+		(im_callback_fn) conv_close, conv, NULL ) ||
         	!(conv->coeff = IM_ARRAY( out, ne, double )) ||
         	!(conv->mask = im_dup_dmask( mask, "conv_mask" )) )
                 return( NULL );
@@ -146,14 +146,11 @@ typedef struct {
 /* Free a sequence value.
  */
 static int
-stop_conv( ConvSequence *seq, IMAGE *in, Conv *conv )
+conv_stop( void *vseq, void *a, void *b )
 {
-	/* Free attached objects.
-	 */
-	if( seq->ir ) {
-		im_region_free( seq->ir );
-		seq->ir = NULL;
-	}
+	ConvSequence *seq = (ConvSequence *) vseq;
+
+	IM_FREEF( im_region_free, seq->ir );
 
 	return( 0 );
 }
@@ -161,11 +158,13 @@ stop_conv( ConvSequence *seq, IMAGE *in, Conv *conv )
 /* Convolution start function.
  */
 static void *
-start_conv( IMAGE *out, IMAGE *in, Conv *conv )
+conv_start( IMAGE *out, void *a, void *b )
 {
-	ConvSequence *seq = IM_NEW( out, ConvSequence );
+	IMAGE *in = (IMAGE *) a;
+	Conv *conv = (Conv *) b;
+	ConvSequence *seq;
 
-	if( !seq ) 
+	if( !(seq = IM_NEW( out, ConvSequence )) )
 		return( NULL );
 
 	/* Init!
@@ -180,7 +179,7 @@ start_conv( IMAGE *out, IMAGE *in, Conv *conv )
 	seq->offsets = IM_ARRAY( out, conv->nnz, int );
 	seq->pts = IM_ARRAY( out, conv->nnz, PEL * );
 	if( !seq->ir || !seq->offsets || !seq->pts ) {
-		stop_conv( seq, in, conv );
+		conv_stop( seq, in, conv );
 		return( NULL );
 	}
 
@@ -209,8 +208,11 @@ start_conv( IMAGE *out, IMAGE *in, Conv *conv )
 /* Convolve!
  */
 static int
-gen_conv( REGION *or, ConvSequence *seq, IMAGE *in, Conv *conv )
+conv_gen( REGION *or, void *vseq, void *a, void *b )
 {
+	ConvSequence *seq = (ConvSequence *) vseq;
+	IMAGE *in = (IMAGE *) a;
+	Conv *conv = (Conv *) b;
 	REGION *ir = seq->ir;
 	DOUBLEMASK *mask = conv->mask;
 
@@ -283,13 +285,14 @@ im_convf_raw( IMAGE *in, IMAGE *out, DOUBLEMASK *mask )
 	/* Check parameters.
 	 */
 	if( !in || in->Coding != IM_CODING_NONE || im_iscomplex( in ) ) {
-		im_errormsg( "im_convf: input non-complex uncoded please!");
+		im_error( "im_convf", 
+			_( "non-complex uncoded only" ) );
 		return( -1 );
 	}
 	if( !mask || mask->xsize > 1000 || mask->ysize > 1000 || 
 		mask->xsize <= 0 || mask->ysize <= 0 || !mask->coeff ||
 		mask->scale == 0 ) {
-		im_errormsg( "im_convf: nonsense mask parameters" );
+		im_error( "im_convf", _( "nonsense mask parameters" ) );
 		return( -1 );
 	}
 	if( im_piocheck( in, out ) )
@@ -309,7 +312,7 @@ im_convf_raw( IMAGE *in, IMAGE *out, DOUBLEMASK *mask )
 	out->Xsize -= mask->xsize - 1;
 	out->Ysize -= mask->ysize - 1;
 	if( out->Xsize <= 0 || out->Ysize <= 0 ) {
-		im_errormsg( "im_convf: image too small for mask" );
+		im_error( "im_convf", _( "image too small for mask" ) );
 		return( -1 );
 	}
 
@@ -319,7 +322,7 @@ im_convf_raw( IMAGE *in, IMAGE *out, DOUBLEMASK *mask )
 	if( im_demand_hint( out, IM_FATSTRIP, in, NULL ) )
 		return( -1 );
 
-	if( im_generate( out, start_conv, gen_conv, stop_conv, in, conv ) )
+	if( im_generate( out, conv_start, conv_gen, conv_stop, in, conv ) )
 		return( -1 );
 
 	out->Xoffset = -mask->xsize / 2;
