@@ -438,6 +438,33 @@ static im_function maplut_desc = {
 	maplut_args 		/* Arg list */
 };
 
+/* Call im_project() via arg vector.
+ */
+static int
+project_vec( im_object *argv )
+{
+	return( im_project( argv[0], argv[1], argv[2] ) );
+}
+
+/* Args for im_project().
+ */
+static im_arg_desc project_args[] = {
+	IM_INPUT_IMAGE( "in" ),
+	IM_OUTPUT_IMAGE( "hout" ),
+	IM_OUTPUT_IMAGE( "vout" )
+};
+
+/* Description of im_project().
+ */ 
+static im_function project_desc = {
+	"im_project", 			/* Name */
+	"find horizontal and vertical projections of an image",	
+	IM_FN_TRANSFORM,		/* Flags */
+	project_vec, 			/* Dispatch function */
+	IM_NUMBER( project_args ), 	/* Size of arg list */
+	project_args 			/* Arg list */
+};
+
 /* Args for im_stdif.
  */
 static im_arg_desc stdif_args[] = {
@@ -503,6 +530,34 @@ static im_function stdif_raw_desc = {
 	stdif_args 		/* Arg list */
 };
 
+/* Args for im_buildlut.
+ */
+static im_arg_desc buildlut_args[] = {
+	IM_INPUT_DMASK( "xyes" ),
+	IM_OUTPUT_IMAGE( "lut" )
+};
+
+/* Call im_buildlut via arg vector.
+ */
+static int
+buildlut_vec( im_object *argv )
+{
+	im_mask_object *mi = argv[0];
+
+	return( im_buildlut( mi->mask, argv[1] ) );
+}
+
+/* Description of im_buildlut.
+ */ 
+static im_function buildlut_desc = {
+	"im_buildlut", 	/* Name */
+	"generate LUT table from set of x/y positions",
+	0,			/* Flags */
+	buildlut_vec, 		/* Dispatch function */
+	IM_NUMBER( buildlut_args ),/* Size of arg list */
+	buildlut_args 		/* Arg list */
+};
+
 /* Args for im_invertlut.
  */
 static im_arg_desc invertlut_args[] = {
@@ -736,7 +791,9 @@ static im_function *hist_list[] = {
 	&lhisteq_desc,
 	&lhisteq_raw_desc,
 	&invertlut_desc,
+	&buildlut_desc,
 	&maplut_desc,
+	&project_desc,
 	&stdif_desc,
 	&stdif_raw_desc,
 	&tone_analyse_desc,
