@@ -65,13 +65,10 @@ im_hist( IMAGE *in, IMAGE *out, int bandno )
 {
 	IMAGE *hist = im_open_local( out, "im_hist:#1", "p" );
 
-	if( !hist || im_iocheck( in, out ) )
-                return( -1 );
-
-	if( im_histgr( in, hist, bandno ) )
-		return( -1 );
-
-	if( im_histplot( hist, out ) )
+	if( !hist || 
+		im_iocheck( in, out ) ||
+		im_histgr( in, hist, bandno ) ||
+		im_histplot( hist, out ) )
 		return( -1 );
 
 	return( 0 );
