@@ -160,7 +160,7 @@ match( IMAGE *in, IMAGE *ref, IMAGE *out )
 		}
 	}
 
-        if( im_cp_desc( out, in ) )
+        if( im_cp_descv( out, in, ref, NULL ) )
 		return( -1 );
 	out->Xsize = px;
 	out->Ysize = 1;
