@@ -66,16 +66,11 @@ im_hsp( IMAGE *in, IMAGE *ref, IMAGE *out )
 	IMAGE *histref = im_open_local( out, "im_hsp:#2", "p" );
 	IMAGE *lut = im_open_local( out, "im_hsp:#3", "p" );
 
-	if( !histin || !histref || !lut )
-		return( -1 );
-
-	if( im_histgr( in, histin, 0 ) || im_histgr( ref, histref, 0 ) )
-		return( -1 );
-
-	if( im_histspec( histin, histref, lut ) )
-		return( -1 );
-
-	if( im_maplut( in, out, lut ) )
+	if( !histin || !histref || !lut ||
+		im_histgr( in, histin, -1 ) || 
+		im_histgr( ref, histref, -1 ) ||
+		im_histspec( histin, histref, lut ) ||
+		im_maplut( in, out, lut ) )
 		return( -1 );
 
 	return( 0 );
