@@ -16,6 +16,8 @@
  *	- im_incheck() call added
  * 16/8/94 JC
  *	- im_incheck() changed to im_makerw()
+ * 5/12/06
+ * 	- im_invalidate() after paint
  */
 
 /*
@@ -126,5 +128,7 @@ im_circle( IMAGE *im, int cx, int cy, int radius, int intensity)
 		*(start + offset - size * x + y) = (PEL)intensity;
 		}
 
+	im_invalidate( im );
+
         return(0);	
 }
