@@ -1,22 +1,23 @@
 /* int im_flood( IMAGE *im, int x, int y, PEL *ink, Rect *dout )
  *
  * Flood fill from point (x,y) with colour ink. Flood up to boundary == ink.
- * Any type, any number of bands, IM_CODING_LABQ too. Returns the bounding box of the
- * modified pixels in dout, whether it succeeds or not.
+ * Any type, any number of bands, IM_CODING_LABQ too. Returns the bounding box 
+ * of the modified pixels in dout, whether it succeeds or not.
  *
  * Currently a rather inefficient pixel-based algorithm, should put something
- * better in, really. Speed isn't likely to be a problem, except for *very*
+ * better in, really. Speed isn't likely to be a problem, except for very
  * large images.
  *
  * JC 30/8/97
  *	- VIPSified, cleaned up, from "John Robinson's prog to fill 
  *	  enclosed areas"
- *	- something Kirk gave me, so thanks John ... you should have used a
- *	  scan-line based filler though :)
+ *	- something Kirk gave me, so thanks John 
  * JC 1/10/97
  *	- swapped inner memcmp/cpy for a loop ... faster for small pixels
  * 13/7/02 JC
  *	- im_flood_blob() added
+ * 5/12/06
+ * 	- im_invalidate() after paint
  */
 
 /*
@@ -74,10 +75,10 @@ typedef struct {
 /* A buffer of points, and how many of them have been used. When full, alloc a
  * new buffer, and link it on.
  */
-typedef struct buffer_struct {
-	struct buffer_struct *next;
+typedef struct _Buffer {
+	struct _Buffer *next;
 	int n;
-	Point points[ PBUFSIZE ];
+	Point points[PBUFSIZE];
 } Buffer;
 
 /* Our state.
@@ -126,10 +127,7 @@ build_buffer( void )
 static void
 free_buffer( Buffer *buf )
 {
-	if( buf->next ) {
-		free_buffer( buf->next );
-		buf->next = NULL;
-	}
+	IM_FREE( buf->next );
 	im_free( buf );
 }
 
@@ -140,22 +138,14 @@ free_state( State *st )
 {
 	/* Write dirty back to caller.
 	 */
-	*st->dout = st->dirty;
+	if( st->dout )
+		*st->dout = st->dirty;
 
 	/* Free our stuff.
 	 */
-	if( st->ink ) {
-		im_free( st->ink );
-		st->ink = NULL;
-	}
-	if( st->buf1 ) {
-		free_buffer( st->buf1 );
-		st->buf1 = NULL;
-	}
-	if( st->buf2 ) {
-		free_buffer( st->buf2 );
-		st->buf2 = NULL;
-	}
+	IM_FREE( st->ink );
+	IM_FREEF( free_buffer, st->buf1 );
+	IM_FREEF( free_buffer, st->buf2 );
 	im_free( st );
 }
 
@@ -336,7 +326,7 @@ im_flood( IMAGE *im, int x, int y, PEL *ink, Rect *dout )
 	if( im_rwcheck( im ) )
 		return( -1 );
 	if( im->Coding != IM_CODING_NONE && im->Coding != IM_CODING_LABQ ) {
-		im_errormsg( "im_flood: uncoded or IM_CODING_LABQ only" );
+		im_error( "im_flood", _( "uncoded or IM_CODING_LABQ only" ) );
 		return( -1 );
 	}
 	if( !(st = build_state( im, x, y, ink, dout )) )
@@ -367,6 +357,8 @@ im_flood( IMAGE *im, int x, int y, PEL *ink, Rect *dout )
 	
 	free_state( st );
 
+	im_invalidate( im );
+
 	return( 0 );
 }
 
@@ -380,7 +372,7 @@ im_flood_blob( IMAGE *im, int x, int y, PEL *ink, Rect *dout )
 	if( im_rwcheck( im ) )
 		return( -1 );
 	if( im->Coding != IM_CODING_NONE && im->Coding != IM_CODING_LABQ ) {
-		im_errormsg( "im_flood: uncoded or IM_CODING_LABQ only" );
+		im_error( "im_flood", _( "uncoded or IM_CODING_LABQ only" ) );
 		return( -1 );
 	}
 	if( !(st = build_state( im, x, y, ink, dout )) )
@@ -404,5 +396,26 @@ im_flood_blob( IMAGE *im, int x, int y, PEL *ink, Rect *dout )
 	
 	free_state( st );
 
+	im_invalidate( im );
+
 	return( 0 );
 }
+
+/* A Flood blob we can call from nip. Grr! Should be a way to wrap these
+ * automatically. Maybe nip could do it if it seems a RW image argument?
+ */
+int
+im_flood_blob_copy( IMAGE *in, IMAGE *out, int x, int y, PEL *ink )
+{
+	IMAGE *t;
+
+	if( !(t = im_open_local( out, "im_flood_blob_copy", "t" )) ||
+		im_copy( in, t ) ||
+		im_flood_blob( t, x, y, ink, NULL ) ||
+		im_copy( t, out ) ) 
+		return( -1 );
+
+	return( 0 );
+}
+
+
