@@ -17,6 +17,8 @@
  * 1/9/04 JC
  *	- checks bands/types/etc match (thanks Matt)
  *	- smarter pixel size calculations
+ * 5/12/06
+ * 	- im_invalidate() after paint
  */
 
 /*
@@ -107,13 +109,15 @@ im_insertplace( IMAGE *big, IMAGE *small, int x, int y )
 
 	/* Loop, memcpying small to big.
 	 */
-	p = IM_IMAGE_ADDR( small, 0, 0 );
-	q = IM_IMAGE_ADDR( big, x, y );
+	p = (PEL *) IM_IMAGE_ADDR( small, 0, 0 );
+	q = (PEL *) IM_IMAGE_ADDR( big, x, y );
 	for( z = 0; z < small->Ysize; z++ ) {
 		memcpy( (char *) q, (char *) p, IM_IMAGE_SIZEOF_LINE( small ) );
 		p += IM_IMAGE_SIZEOF_LINE( small );
 		q += IM_IMAGE_SIZEOF_LINE( big );
 	}
 
+	im_invalidate( big );
+
 	return( 0 );
 }
