@@ -11,6 +11,8 @@
  *	- externs removed
  * 16/8/94 JC
  *	- im_incheck() changed to im_makerw()
+ * 5/12/06
+ * 	- im_invalidate() after paint
  */
 
 /*
@@ -146,5 +148,6 @@ else
 /* Draw point (x2, y2) */
 offset = x2 + y2 * image->Xsize;
 *(image->data + offset) = (PEL)pelval;
+	im_invalidate( image );
 return(0);
 }
