@@ -16,6 +16,8 @@
  *	- im_incheck() changed to im_makerw()
  * 24/10/03 JC
  *	- now blends with 0-255 mask
+ * 5/12/06
+ * 	- im_invalidate() after paint
  */
 
 /*
@@ -223,12 +225,15 @@ im_plotmask( IMAGE *im, int ix, int iy, PEL *ink, PEL *mask, Rect *r )
 				break;
 
 			default:
-				im_errormsg( "im_lrmerge: internal error" );
+				im_error( "im_plotmask", 
+					_( "internal error" ) );
 				return( -1 );
 			}
 		}
 	}
 
+	im_invalidate( im );
+
 	return( 0 );
 }
 
