@@ -38,6 +38,8 @@
  *	- im_incheck() added
  * 16/8/94 JC
  *	- im_incheck() changed to im_makerw()
+ * 5/12/06
+ * 	- im_invalidate() after paint
  */
 
 /*
@@ -244,6 +246,8 @@ im_fastline( IMAGE *im, int x1, int y1, int x2, int y2, PEL *pel )
 	else
 		error_exit( "internal error #9872659823475982375" );
 
+	im_invalidate( im );
+
 	return( 0 );
 }
 
@@ -390,5 +394,56 @@ im_fastlineuser( IMAGE *im,
 	else
 		error_exit( "internal error #9872659823475982375" );
 
+	im_invalidate( im );
+
+	return( 0 );
+}
+
+/* Draw a set of lines with an ink and a mask. A non-inplace operation, handy
+ * for nip2.
+ */
+int
+im_lineset( IMAGE *in, IMAGE *out, IMAGE *mask, IMAGE *ink,
+	int n, int *x1v, int *y1v, int *x2v, int *y2v )
+{
+	Rect mask_rect;
+	int i;
+
+	if( mask->Bands != 1 || mask->BandFmt != IM_BANDFMT_UCHAR ||
+		mask->Coding != IM_CODING_NONE ) {
+		im_error( "im_lineset", 
+			_( "mask image not 1 band 8 bit uncoded" ) );
+		return( -1 );
+	}
+	if( ink->Bands != in->Bands || ink->BandFmt != in->BandFmt ||
+		ink->Coding != in->Coding ) {
+		im_error( "im_lineset", 
+			_( "ink image does not match in image" ) );
+		return( -1 );
+	}
+	if( ink->Xsize != 1 || ink->Ysize != 1 ) {
+		im_error( "im_lineset", _( "ink image not 1x1 pixels" ) );
+		return( -1 );
+	}
+
+	/* Copy the image thenm fastline to it ... this will render to a "t"
+	 * usually.
+	 */
+	if( im_incheck( mask ) ||
+		im_incheck( ink ) ||
+		im_copy( in, out ) )
+		return( -1 );
+
+	mask_rect.left = mask->Xsize / 2;
+	mask_rect.top = mask->Ysize / 2;
+	mask_rect.width = mask->Xsize;
+	mask_rect.height = mask->Ysize;
+
+	for( i = 0; i < n; i++ ) {
+		if( im_fastlineuser( out, x1v[i], y1v[i], x2v[i], y2v[i], 
+			im_plotmask, ink->data, mask->data, &mask_rect ) )
+			return( -1 );
+	}
+
 	return( 0 );
 }
