@@ -14,6 +14,8 @@
  *	- im_incheck() added
  * 16/8/94 JC
  *	- im_incheck() changed to im_makerw()
+ * 5/12/06
+ * 	- im_invalidate() after paint
  */
 
 /*
@@ -111,5 +113,7 @@ im_plotpoint( IMAGE *im, int x, int y, PEL *pel )
 	for( b = 0; b < ps; b++ )
 		*to++ = *pel++;
 
+	im_invalidate( im );
+
 	return( 0 );
 }
