@@ -20,6 +20,8 @@
  *	- im_incheck() changed to im_makerw()
  * ? JC
  *	- im_makerw() changed to im_rwcheck()
+ * 5/12/06
+ * 	- im_invalidate() after paint
  */
 
 /*
@@ -117,7 +119,8 @@ im_smudge( IMAGE *im, int ix, int iy, Rect *r )
 			} \
  			\
 			for( a = 0; a < ba; a++ ) \
-				to[a] = (16 * (double) to[a] + total[a]) / 25.0; \
+				to[a] = (16 * (double) to[a] + total[a]) \
+					/ 25.0; \
 		}
 
 	/* Loop through the remaining pixels.
@@ -179,11 +182,13 @@ im_smudge( IMAGE *im, int ix, int iy, Rect *r )
 		break;
 
 	default:
-		im_errormsg( "im_smudge: unknown band format" );
+		im_error( "im_smudge", _( "unknown band format" ) );
 		return( -1 );
 
 	}
 
+	im_invalidate( im );
+
 	return( 0 );
 }
 
@@ -304,9 +309,11 @@ im_smear( IMAGE *im, int ix, int iy, Rect *r )
 		break;
 
 	default:
-		im_errormsg( "im_smear: unknown band format" );
+		im_error( "im_smear", _( "unknown band format" ) );
 		return( -1 );
 	}
 
+	im_invalidate( im );
+
 	return( 0 );
 }
