@@ -55,6 +55,11 @@
  * Modified on : 
  * 2/3/98 JC
  *	- IM_ANY added
+ * 19/5/06
+ * 	- minor change to rules: don't force ANY on no-input operations ...
+ * 	  fails for image import
+ * 1/12/06
+ * 	- build parent/child links as well
  */
 
 /*
@@ -97,6 +102,7 @@
 #include <stdarg.h>
 
 #include <vips/vips.h>
+#include <vips/internal.h>
 
 #ifdef WITH_DMALLOC
 #include <dmalloc.h>
@@ -104,7 +110,7 @@
 
 /* Max number of images we can handle.
  */
-#define IM_MAX_IMAGES (1000)
+#define MAX_IMAGES (1000)
 
 /* Given two im_demand_type, return the most restrictive.
  */
@@ -127,22 +133,24 @@ im_demand_hint_array( IMAGE *im, im_demand_type hint, IMAGE **in )
 		if( in[i]->dhint == IM_ANY )
 			nany++;
 
-	/* Special case: if all the inputs are IM_ANY, then output can be
-	 * IM_ANY regardless of what this function wants. Note that if there
-	 * are no input images, we also set ANY.
-	 */
-	if( nany == len ) {
-		im->dhint = IM_ANY;
-		return( 0 );
-	}
+	if( len == 0 ) 
+		/* No input images? Just set the requested hint. We don't 
+		 * force ANY, since the operation might be something like 
+		 * tiled read of an EXR image, where we certainly don't want 
+		 * ANY.
+		 */
+		;
+	else if( nany == len ) 
+		/* Special case: if all the inputs are IM_ANY, then output can 
+		 * be IM_ANY regardless of what this function wants. 
+		 */
+		hint = IM_ANY;
+	else
+		/* Find the most restrictive of all the hints available to us.
+		 */
+		for( i = 0; i < len; i++ )
+			hint = find_least( hint, in[i]->dhint );
 
-	/* Find the most restrictive of all the hints available to us.
-	 */
-	for( i = 0; i < len; i++ )
-		hint = find_least( hint, in[i]->dhint );
-	
-	/* And set us to that.
-	 */
 	im->dhint = hint;
 
 #ifdef DEBUG
@@ -150,6 +158,11 @@ im_demand_hint_array( IMAGE *im, im_demand_type hint, IMAGE **in )
 		im->filename, im_dhint2char( im->dhint ) );
 #endif /*DEBUG*/
 
+	/* im depends on all these ims.
+	 */
+	for( i = 0; i < len; i++ )
+		im__link_make( im, in[i] );
+
 	return( 0 );
 }
 
@@ -160,23 +173,16 @@ im_demand_hint( IMAGE *im, im_demand_type hint, ... )
 {
 	va_list ap;
 	int i;
-	IMAGE *ar[ IM_MAX_IMAGES ];
-	IMAGE *t;
+	IMAGE *ar[MAX_IMAGES];
 
-	/* Loop over args, collecting them in an array.
-	 */
 	va_start( ap, hint );
-	for( i = 0; (t = va_arg( ap, IMAGE * )); i++ ) 
-		if( i == IM_MAX_IMAGES - 1 ) {
-			im_errormsg( "im_demand_hint: too many images" );
-			return( -1 );
-		}
-		else
-			ar[i] = t;
+	for( i = 0; i < MAX_IMAGES && (ar[i] = va_arg( ap, IMAGE * )); i++ ) 
+		;
 	va_end( ap );
-	ar[i] = NULL;
+	if( i == MAX_IMAGES ) {
+		im_error( "im_demand_hint", _( "too many images" ) );
+		return( -1 );
+	}
 
-	/* Call the above!
-	 */
 	return( im_demand_hint_array( im, hint, ar ) );
 }
