@@ -22,6 +22,10 @@
  *	- frees old Hist correctly
  * 22/12/94
  *	- ANSIfied with stdarg
+ * 2/9/05
+ * 	- no more first line of Hist means something special nonsense
+ * 4/1/07
+ * 	- added im_history_get()
  */
 
 /*
@@ -66,59 +70,51 @@
 #include <time.h>
 
 #include <vips/vips.h>
+#include <vips/internal.h>
 
 #ifdef WITH_DMALLOC
 #include <dmalloc.h>
 #endif /*WITH_DMALLOC*/
 
 int 
-im_histlin( IMAGE *image, const char *fmt, ... )
+im_histlin( IMAGE *im, const char *fmt, ... )
 {
 	va_list args;
-	char line[ 4096 ];
-	char buf[ 4096 ];
+	char line[4096];
 	time_t timebuf;
-	char *newhist;
-	int newsize;
 
-	/* Format command.
+	/* Format command. -40, to leave 26 for the ctime, three for the # and
+	 * a bit.
 	 */
 	va_start( args, fmt );
-	(void) im_vsnprintf( line, 4096, fmt, args );
+	(void) im_vsnprintf( line, 4096 - 40, fmt, args );
 	va_end( args );
 	strcat( line, " # " );
 
-	/* Add the date. ctime always attaches a '\n'.
+	/* Add the date. ctime always attaches a '\n', gah.
 	 */
 	time( &timebuf );
 	strcat( line, ctime( &timebuf ) );
+	line[strlen( line ) - 1] = '\0';
 
 #ifdef DEBUG
-	printf( "im_histlin: adding:\n\t%sto history on image 0x%x\n",
-		line, (unsigned int) image );
+	printf( "im_histlin: adding:\n\t%s\nto history on image %p\n", 
+		line, im );
 #endif /*DEBUG*/
 
-	/* Make some history if there is none there now.
-	 */
-	if( !image->Hist ) {
-		if( !image->filename )
-			im_snprintf( buf, 4096, "# <no title>\n" );
-		else
-			im_snprintf( buf, 4096, "%s\n", image->filename );
-		image->Hist = im_strdup( NULL, buf );
-	}
-
-	/* Malloc space for new history, +1 for NULL.
-	 */
-	newsize = strlen( image->Hist ) + strlen( line ) + 1;
-	if( !(newhist = im_malloc( NULL, newsize )) ) 
-                return( -1 );
-	im_snprintf( newhist, newsize, "%s%s", image->Hist, line );
-
-	/* Free old history, attach new history.
-	 */
-	im_free( image->Hist );
-	image->Hist = newhist;
+	im->history_list = g_slist_append( im->history_list, 
+		im__gvalue_ref_string_new( line ) );
 
 	return( 0 );
 }
+
+/* Read an image's history.
+ */
+const char *
+im_history_get( IMAGE *im )
+{
+	if( !im->Hist )
+		im->Hist = im__gslist_gvalue_get( im->history_list );
+
+	return( im->Hist ? im->Hist : "" );
+}
