@@ -14,6 +14,8 @@
  * Modified on : 3/6/92 Kirk Martinez 
  * 23/2/94 JC
  *	- ANSIfied
+ * 2/6/07
+ * 	- ignore bandbits
  */
 
 /*
@@ -66,7 +68,9 @@ im_initdesc( IMAGE *image,
 	image->Xsize = xsize;
 	image->Ysize = ysize;
 	image->Bands = bands;
-	image->Bbits = bandbits;
+	/* bandbits is deprecated ... set to whatever the format requires.
+	 */
+	image->Bbits = im_bits_of_fmt( bandfmt );
 	image->BandFmt = bandfmt;
 	image->Coding = coding;
 	image->Type = type;
