@@ -89,6 +89,7 @@
 #endif /*HAVE_SYS_FILE_H*/
 
 #include <vips/vips.h>
+#include <vips/internal.h>
 
 #ifdef WITH_DMALLOC
 #include <dmalloc.h>
@@ -135,7 +136,7 @@ convert_otom( IMAGE *im )
 	 */
 	if( im_mapfile( im ) ) 
 		return( -1 );
-	im->data = im->baseaddr + 64;
+	im->data = im->baseaddr + im->sizeof_header;
 	im->dtype = IM_MMAPIN;
 
 	return( 0 );
