@@ -37,8 +37,10 @@
  *	- _INTEL and _SPARC auto byte-swap added
  * 6/8/02 JC
  *	- redone for mmap() window stuff
+ * 13/3/06 JC
+ * 	- don't abort load if we can't get the XML
  * 16/8/06
- * 	- more O_BINARY nonsense to help cygwin
+ *      - more O_BINARY nonsense to help cygwin
  */
 
 /*
@@ -102,12 +104,14 @@
 #endif /*BINARY_OPEN*/
 
 #include <vips/vips.h> 
+#include <vips/internal.h>
 
 #ifdef WITH_DMALLOC
 #include <dmalloc.h>
 #endif /*WITH_DMALLOC*/
 
-/* mmap() whole vs. window threshold ... an int, so we can tune easily.
+/* mmap() whole vs. window threshold ... an int, so we can tune easily from a
+ * debugger.
  */
 #ifdef DEBUG
 int im__mmap_limit = 1;
@@ -138,8 +142,8 @@ im__open_image_file( const char *filename )
 #else /*BINARY_OPEN*/
 		if( (fd = open( filename, O_RDONLY )) == -1 ) {
 #endif /*BINARY_OPEN*/
-			im_errormsg( "im__open_image_file: unable to "
-				"open \"%s\", %s",
+			im_error( "im__open_image_file", 
+				_( "unable to open \"%s\", %s" ),
 				filename, strerror( errno ) );
 			return( -1 );
 		}
@@ -148,91 +152,77 @@ im__open_image_file( const char *filename )
 	return( fd );
 }
 
+/* Predict the size of the header plus pixel data. Don't use off_t,
+ * it's sometimes only 32 bits (eg. on many windows build environments) and we
+ * want to always be 64 bit.
+ */
+gint64
+im__image_pixel_length( IMAGE *im )
+{
+	gint64 psize;
+
+	switch( im->Coding ) {
+	case IM_CODING_LABQ:
+	case IM_CODING_NONE:
+		psize = (gint64) IM_IMAGE_SIZEOF_LINE( im ) * im->Ysize;
+		break;
+
+	default:
+		psize = im->Length;
+		break;
+	}
+
+	return( psize + im->sizeof_header );
+}
+
 /* Open the filename, read the header, some sanity checking.
  */
 int
 im__read_header( IMAGE *image )
-{	
-	VAS_HD hd;
+{
+	/* We don't use im->sizeof_header here, but we know we're reading a
+	 * VIPS image anyway.
+	 */
+	unsigned char header[IM_SIZEOF_HEADER];
+
 	gint64 length;
 	gint64 psize;
 
 	image->dtype = IM_OPENIN;
 	if( (image->fd = im__open_image_file( image->filename )) == -1 ) 
 		return( -1 );
-	if( read( image->fd, &hd, sizeof( VAS_HD ) ) != sizeof( VAS_HD ) ||
-		im_desc_hd( image, &hd ) ) {
-		im_errormsg( "im_openin: unable to read header for \"%s\", %s",
+	if( read( image->fd, header, IM_SIZEOF_HEADER ) != IM_SIZEOF_HEADER ||
+		im__read_header_bytes( image, header ) ) {
+		im_error( "im_openin", 
+			_( "unable to read header for \"%s\", %s" ),
 			image->filename, strerror( errno ) );
 		return( -1 );
 	}
 
-	/* Extra checks. 
-	
-		FIXME ... this is v bad :-( do something about it.
-
-	 */
-	assert( sizeof( VAS_HD ) == 64 );
-
 	/* Predict and check the file size.
 	 */
-	switch( image->Coding ) {
-	case IM_CODING_LABQ:
-	case IM_CODING_NONE:
-		psize = (gint64) IM_IMAGE_SIZEOF_LINE( image ) * 
-			image->Ysize + 64;
-
-		/* Read the file length again, and check against what we think 
-		 * the size should be.
-		 */
-		if( (length = im_file_length( image->fd )) == -1 ) 
-			return( -1 );
-
-		/* Very common, so special message.
-		 */
-		if( psize > length ) {
-			im_errormsg( "im_openin: unable to open \"%s\": "
-				"file has been truncated", image->filename );
-			return( -1 );
-		}
-
-		/* Just wierd. Only print a warning for this, since we should
-		 * still be able to process it without coredumps.
-		 */
-		if( psize < length )
-			im_warning( "im_openin: \"%s\" is longer than expected",
-				image->filename );
-
-		break;
-	
-	default:
-		/* Check the Length field in the header against the actual
-		 * file size.
-		 */
-		if( (length = im_file_length( image->fd )) == -1 ) 
-			return( -1 );
-		if( image->Length > length ) {
-			im_errormsg( "im_openin: unable to open "
-				"compressed image file \"%s\": "
-				"file has been truncated", image->filename );
-			return( -1 );
-		}
-		if( image->Length < length )
-			im_warning( "im_openin: compressed image \"%s\" is "
-				"longer than expected",
-				image->filename );
-
-		break;
+	psize = im__image_pixel_length( image );
+	if( (length = im_file_length( image->fd )) == -1 ) 
+		return( -1 );
+	if( psize > length ) {
+		im_error( "im_openin", _( "unable to open \"%s\", %s" ),
+			image->filename, _( "file has been truncated" ) );
+		return( -1 );
 	}
 
 	/* Set demand style. Allow the most permissive sort.
 	 */
 	image->dhint = IM_THINSTRIP;
 
-	/* Set the history part of im descriptor.
+	/* Set the history part of im descriptor. Don't return an error if this
+	 * fails (due to eg. corrupted XML) because it's probably mostly
+	 * harmless.
 	 */
-	if( im_readhist( image ) ) 
-		return( -1 );
+	if( im__readhist( image ) ) {
+		im_warn( "im_openin", _( "error reading XML: %s" ),
+			im_error_buffer() );
+		im_error_clear();
+	}
 
 	return( 0 );
 }
@@ -254,14 +244,16 @@ im_openin( IMAGE *image )
 			im__mmap_limit );
 	}
 #endif /*DEBUG*/
+
 	if( im__read_header( image ) )
 		return( -1 );
 
-	size = (gint64) IM_IMAGE_SIZEOF_LINE( image ) * image->Ysize + 64;
+	size = (gint64) IM_IMAGE_SIZEOF_LINE( image ) * image->Ysize + 
+		image->sizeof_header;
 	if( size < im__mmap_limit ) {
 		if( im_mapfile( image ) )
 			return( -1 );
-		image->data = image->baseaddr + 64;
+		image->data = image->baseaddr + image->sizeof_header;
 		image->dtype = IM_MMAPIN;
 
 #ifdef DEBUG
@@ -289,7 +281,7 @@ im_openinrw( IMAGE *image )
 
 	if( im_mapfilerw( image ) ) 
 		return( -1 );
-	image->data = image->baseaddr + 64;
+	image->data = image->baseaddr + image->sizeof_header;
 	image->dtype = IM_MMAPINRW;
 
 #ifdef DEBUG
