@@ -19,6 +19,8 @@
  * 9/8/02 JC
  *	- im_prepare_inplace() broke with mmap() windows ... im_prepare_to()
  *	  replaces it and is nicer
+ * 30/9/05
+ * 	- hmm, did not stop if a start function failed
  */
 
 /*
@@ -76,7 +78,8 @@ fill_region( REGION *reg )
 
         /* Start new sequence, if necessary.
          */
-        im__call_start( reg );
+        if( im__call_start( reg ) )
+		return( -1 );
 
 	/* Ask for evaluation.
 	 */
@@ -92,7 +95,7 @@ im__test_kill( IMAGE *im )
 	/* Has kill been set for this image? If yes, abort evaluation.
 	 */
 	if( im->kill ) {
-		im_errormsg( "im__test_kill: killed for image \"%s\"",
+		im_error( "im__test_kill", _( "killed for image \"%s\"" ),
 			im->filename );
 		return( -1 );
 	}
@@ -108,9 +111,30 @@ im_prepare( REGION *reg, Rect *r )
 {	
 	IMAGE *im = reg->im;
 
+	Rect save = *r;
+
+	im__region_check_ownership( reg );
+
 	if( im__test_kill( im ) )
 		return( -1 );
 
+	/* We use save for sanity checking valid: we test at the end that the
+	 * pixels we have generated are indeed all the ones that were asked
+	 * for.
+	 *
+	 * However, r may be clipped by the image size, so we need to clip
+	 * save as well to make sure we don't fail the assert due to that.
+	 */
+{	
+	Rect image;
+
+	image.left = 0;
+	image.top = 0;
+	image.width = reg->im->Xsize;
+	image.height = reg->im->Ysize;
+	im_rect_intersectrect( &save, &image, &save );
+}
+
 #ifdef DEBUG
         printf( "im_prepare: left = %d, top = %d, width = %d, height = %d\n",
 		r->left, r->top, r->width, r->height );
@@ -118,22 +142,8 @@ im_prepare( REGION *reg, Rect *r )
 
 	switch( im->dtype ) {
 	case IM_PARTIAL:
-		/* Sanity check.
-		 */
-		if( !im->generate ) {
-			im_errormsg( "im_prepare: incomplete header" );
-			return( -1 );
-		}
-
-		/* Make sure there is some local storage on reg before we 
-		 * start the user function.
-		 */
-		if( im_region_local( reg, r ) )
-			return( -1 );
-
-		/* Run sequence into reg.
-		 */
-		if( fill_region( reg ) )
+		if( im_region_fill( reg, r, 
+			(im_region_fill_fn) fill_region, NULL ) )
 			return( -1 );
 
 		break;
@@ -142,6 +152,7 @@ im_prepare( REGION *reg, Rect *r )
 	case IM_SETBUF_FOREIGN:
 	case IM_MMAPIN:
 	case IM_MMAPINRW:
+	case IM_OPENIN:
 		/* Attach to existing buffer.
 		 */
 		if( im_region_image( reg, r ) )
@@ -149,20 +160,16 @@ im_prepare( REGION *reg, Rect *r )
 
 		break;
 
-	case IM_OPENIN:
-		/* Should be a mmap() window on im.
-		 */
-		if( im_region_mmap_window( reg, r ) )
-			return( -1 );
-
-		break;
-
 	default:
-		im_errormsg( "im_prepare: unable to input from a %s image",
+		im_error( "im_prepare", _( "unable to input from a %s image" ),
 			im_dtype2char( im->dtype ) );
 		return( -1 );
 	}
 
+	/* valid should now include all the pixels that were asked for.
+	 */
+	assert( im_rect_includesrect( &reg->valid, &save ) );
+
 	return( 0 );
 }
 
@@ -218,7 +225,7 @@ im_prepare_to_generate( REGION *reg, REGION *dest, Rect *r, int x, int y )
 	char *p;
 
 	if( !im->generate ) {
-		im_errormsg( "im_prepare_to: incomplete header" );
+		im_error( "im_prepare_to", _( "incomplete header" ) );
 		return( -1 );
 	}
 
@@ -266,12 +273,13 @@ im_prepare_to( REGION *reg, REGION *dest, Rect *r, int x, int y )
 	 */
 	if( !dest->data || dest->im->BandFmt != reg->im->BandFmt ||
 		dest->im->Bands != reg->im->Bands ) {
-		im_errormsg( "im_prepare_to: inappropriate region type" );
+		im_error( "im_prepare_to", _( "inappropriate region type" ) );
 		return( -1 );
 	}
 
-	/* clip r first against the size of reg->im, then against the memory we
-	 * have available to write to on dest. Just like im_region_region()
+	/* clip r first against the size of reg->im, then again against the 
+	 * memory we have available to write to on dest. Just like 
+	 * im_region_region()
 	 */
 	image.top = 0;
 	image.left = 0;
@@ -279,12 +287,25 @@ im_prepare_to( REGION *reg, REGION *dest, Rect *r, int x, int y )
 	image.height = reg->im->Ysize;
 	im_rect_intersectrect( r, &image, &clipped );
 
+	assert( clipped.left == r->left );
+	assert( clipped.top == r->top );
+
 	wanted.left = x + (clipped.left - r->left);
 	wanted.top = y + (clipped.top - r->top);
 	wanted.width = clipped.width;
 	wanted.height = clipped.height;
+
+	/* Test that dest->valid is large enough.
+	 */
+	if( !im_rect_includesrect( &dest->valid, &wanted ) ) {
+		im_error( "im_prepare_to", _( "dest too small" ) );
+		return( -1 );
+	}
+
 	im_rect_intersectrect( &wanted, &dest->valid, &clipped2 );
 
+	/* Translate back to reg's coordinate space and set as valid.
+	 */
 	final.left = r->left + (clipped2.left - wanted.left);
 	final.top = r->top + (clipped2.top - wanted.top);
 	final.width = clipped2.width;
@@ -294,7 +315,7 @@ im_prepare_to( REGION *reg, REGION *dest, Rect *r, int x, int y )
 	y = clipped2.top;
 
 	if( im_rect_isempty( &final ) ) {
-		im_errormsg( "im_prepare_to: valid clipped to nothing" );
+		im_error( "im_prepare_to", _( "valid clipped to nothing" ) );
 		return( -1 );
 	}
 
@@ -317,6 +338,7 @@ im_prepare_to( REGION *reg, REGION *dest, Rect *r, int x, int y )
 
 	case IM_MMAPIN:
 	case IM_MMAPINRW:
+	case IM_OPENIN:
 		/* Attach to existing buffer and copy to dest.
 		 */
 		if( im_region_image( reg, &final ) )
@@ -325,15 +347,6 @@ im_prepare_to( REGION *reg, REGION *dest, Rect *r, int x, int y )
 
 		break;
 
-	case IM_OPENIN:
-		/* Should be a mmap() window on im, then copy.
-		 */
-		if( im_region_mmap_window( reg, &final ) )
-			return( -1 );
-		im__copy_region( reg, dest, &final, x, y );
-
-		break;
-
 	case IM_SETBUF:
 	case IM_SETBUF_FOREIGN:
 		/* Could be either input or output. If there is a generate
@@ -352,10 +365,20 @@ im_prepare_to( REGION *reg, REGION *dest, Rect *r, int x, int y )
 		break;
 
 	default:
-		im_errormsg( "im_prepare_to: unable to input from a "
-			"%s image", im_dtype2char( im->dtype ) );
+		im_error( "im_prepare_to", _( "unable to input from a "
+			"%s image" ), im_dtype2char( im->dtype ) );
 		return( -1 );
 	}
 
 	return( 0 );
 }
+
+int
+im_prepare_many( REGION **reg, Rect *r )
+{
+	for( ; *reg; ++reg )
+		if( im_prepare( *reg, r ) )
+			return( -1 );
+
+	return( 0 );
+}
