@@ -22,6 +22,17 @@
  *	- fix for mmap a local region
  * 28/2/05
  *	- shrink local region memory if required much-greater-than allocated
+ * 3/6/05
+ *	- im_region_region() allows Bands and BandFmt to differ, provided
+ *	  sizeof( pel ) is the same ... makes im_copy_morph() work
+ * 30/10/06
+ * 	- switch to im_window_t for mmap window stuff
+ * 29/11/06
+ * 	- switch to im_buffer_t for local mem buffer stuff
+ * 19/1/07
+ * 	- im_region_image() only sets r, not whole image
+ * 1'2'07
+ * 	- gah, im_region_image() could still break (thanks Mikkel)
  */
 
 /*
@@ -51,10 +62,10 @@
  */
 
 /*
-#define DEBUG
 #define DEBUG_MOVE 
 #define DEBUG_ENVIRONMENT 1
 #define DEBUG_CREATE
+#define DEBUG
  */
 
 #ifdef HAVE_CONFIG_H
@@ -75,6 +86,7 @@
 #include <assert.h>
 
 #include <vips/vips.h>
+#include <vips/internal.h>
 #include <vips/thread.h>
 
 #ifdef OS_WIN32
@@ -85,26 +97,10 @@
 #include <dmalloc.h>
 #endif /*WITH_DMALLOC*/
 
-/* Add this many lines above and below the mmap() window.
- */
-int im__mmap_window_margin = IM__MMAP_WINDOW_MARGIN;
-
-/* Sanity checking ... write to this during read tests to make sure we don't
- * get optimised out.
- */
-int im__read_test;
-
-#ifdef DEBUG
-static int total_mmap_usage = 0;
-static int max_mmap_usage = 0;
-static GMutex *mmap_mutex = NULL;
-#endif /*DEBUG*/
-
 #ifdef DEBUG
 /* Track all regions here for debugging.
  */
 static GSList *im__regions_all = NULL;
-static GMutex *regions_mutex = NULL;
 #endif /*DEBUG*/
 
 /* Call a start function if no sequence is running on this REGION.
@@ -122,7 +118,8 @@ im__call_start( REGION *reg )
                 g_mutex_unlock( im->sslock );
  
                 if( !reg->seq ) {
-                        im_errormsg( "user start function failed for image %s",
+                        im_error( "im__call_start", 
+				_( "start function failed for image %s" ),
                                 im->filename );
                         return( -1 );
                 }
@@ -155,6 +152,59 @@ im__call_stop( REGION *reg )
         }
 }
 
+/* If a region is being created in one thread (eg. the main thread) and then
+ * used in another (eg. a worker thread), the new thread needs to tell VIPS
+ * to stop sanity assert() fails. The previous owner needs to
+ * im__region_no_ownership() before we can call this.
+ */
+void
+im__region_take_ownership( REGION *reg )
+{
+	/* Lock so that there's a memory barrier with the thread doing the
+	 * im__region_no_ownership() before us.
+	 */
+	g_mutex_lock( reg->im->sslock );
+
+	assert( reg->thread == NULL );
+
+	/* We don't want to move shared buffers: the other region using this
+	 * buffer will still be on the other thread. Not sure if this will
+	 * ever happen: if it does, we'll need to dup the buffer.
+	 */
+	assert( !reg->buffer || reg->buffer->ref_count == 1 );
+
+	reg->thread = g_thread_self();
+
+	g_mutex_unlock( reg->im->sslock );
+}
+
+void
+im__region_check_ownership( REGION *reg )
+{
+	if( reg->thread ) {
+		assert( reg->thread == g_thread_self() );
+		if( reg->buffer && reg->buffer->cache )
+			assert( reg->thread == reg->buffer->cache->thread );
+	}
+}
+
+/* Call this from the relinquishing thread. Removes the buffer (if any) from
+ * this thread's buffer cache.
+ */
+void
+im__region_no_ownership( REGION *reg )
+{
+	g_mutex_lock( reg->im->sslock );
+
+	im__region_check_ownership( reg );
+
+	reg->thread = NULL;
+	if( reg->buffer )
+		im_buffer_undone( reg->buffer );
+
+	g_mutex_unlock( reg->im->sslock );
+}
+
 /* Create a region. Set no attachments. Either im_prepare() or im_generate()
  * are responsible for getting regions ready for user functions to read
  * from/write to.
@@ -163,16 +213,6 @@ REGION *
 im_region_create( IMAGE *im )
 {	
 	REGION *reg;
-#ifdef DEBUG_ENVIRONMENT
-	char *str;
-#endif /*DEBUG_ENVIRONMENT*/
-
-#ifdef DEBUG
-	if( !mmap_mutex )
-		mmap_mutex = g_mutex_new();
-	if( !regions_mutex )
-		regions_mutex = g_mutex_new();
-#endif /*DEBUG*/
 
 	im_image_sanity( im );
 
@@ -188,93 +228,45 @@ im_region_create( IMAGE *im )
 	reg->data = NULL;
 	reg->bpl = 0;
 	reg->seq = NULL;
-	reg->buf = NULL;
-	reg->bsize = 0;
-	reg->baseaddr = NULL;
-	reg->length = 0;
-	im->regions = g_slist_prepend( im->regions, reg );
+	reg->thread = NULL;
+	reg->window = NULL;
+	reg->buffer = NULL;
 
-#ifdef DEBUG_ENVIRONMENT
-	printf( "iofuncs/region.c: compiled with DEBUG_ENVIRONMENT enabled\n" );
-	if( (str = g_getenv( "IM_MMAP_WINDOW_MARGIN" )) ) {
-		im__mmap_window_margin = atoi( str );
-		printf( "im_region_create: setting window margin to %d "
-			"from environment\n", im__mmap_window_margin );
-	}
-#endif /*DEBUG_ENVIRONMENT*/
+	im__region_take_ownership( reg );
+
+	/* We're usually inside the ss lock anyway. But be safe ...
+	 */
+	g_mutex_lock( im->sslock );
+	im->regions = g_slist_prepend( im->regions, reg );
+	g_mutex_unlock( im->sslock );
 
 #ifdef DEBUG
-	g_mutex_lock( regions_mutex );
+	g_mutex_lock( im__global_lock );
 	im__regions_all = g_slist_prepend( im__regions_all, reg );
 	printf( "%d regions in vips\n", g_slist_length( im__regions_all ) );
-	g_mutex_unlock( regions_mutex );
+	g_mutex_unlock( im__global_lock );
 #endif /*DEBUG*/
 
 	return( reg );
 }
 
-/* Free local memory.
+/* Free any resources we have.
  */
 static void
 im_region_reset( REGION *reg )
 {
-	if( reg->buf ) {
-		im_free( reg->buf );
-		reg->buf = NULL;
-		reg->bsize = 0;
-	}
+	IM_FREEF( im_window_unref, reg->window );
+	IM_FREEF( im_buffer_unref, reg->buffer );
 }
 
-static int
-im_region_unmap_window( REGION *reg )
-{
-	/* unmap the old window
-	 */
-	if( reg->baseaddr ) {
-		if( im__munmap( reg->baseaddr, reg->length ) )
-			return( -1 );
-
-#ifdef DEBUG
-		g_mutex_lock( mmap_mutex );
-		total_mmap_usage -= reg->length;
-		assert( total_mmap_usage >= 0 );
-		g_mutex_unlock( mmap_mutex );
-#endif /*DEBUG*/
-
-		reg->type = IM_REGION_NONE;
-		reg->data = NULL;
-		reg->baseaddr = NULL;
-		reg->length = 0;
-	}
-
-	return( 0 );
-}
-
-#ifdef DEBUG
-static void
-trace_mmap_usage( void )
-{
-	g_mutex_lock( mmap_mutex );
-	{
-		static int last_total = 0;
-		int total = total_mmap_usage / (1024 * 1024);
-		int max = max_mmap_usage / (1024 * 1024);
-
-		if( total != last_total ) {
-			printf( "im_region_mmap_window: current mmap "
-				"usage of ~%dMB (high water mark %dMB)\n", 
-				total, max );
-			last_total = total;
-		}
-	}
-	g_mutex_unlock( mmap_mutex );
-}
-#endif /*DEBUG*/
-
 void 
 im_region_free( REGION *reg )
 {	
-	IMAGE *im = reg->im;
+	IMAGE *im;
+
+        if( !reg )
+		return;
+        im = reg->im;
 
         /* Stop this sequence.
          */
@@ -284,18 +276,11 @@ im_region_free( REGION *reg )
 	 */
 	im_region_reset( reg );
 
-#ifdef DEBUG
-	if( reg->baseaddr ) {
-		(void) im_region_unmap_window( reg );
-		trace_mmap_usage();
-	}
-#else
-	(void) im_region_unmap_window( reg );
-#endif /*DEBUG*/
-
 	/* Detach from image. 
 	 */
+	g_mutex_lock( im->sslock );
 	im->regions = g_slist_remove( im->regions, reg );
+	g_mutex_unlock( im->sslock );
 	reg->im = NULL;
 
 	/* Was this the last region on an image with close_pending? If yes,
@@ -315,66 +300,64 @@ im_region_free( REGION *reg )
 	im_free( reg );
 
 #ifdef DEBUG
-	g_mutex_lock( regions_mutex );
+	g_mutex_lock( im__global_lock );
 	assert( g_slist_find( im__regions_all, reg ) );
 	im__regions_all = g_slist_remove( im__regions_all, reg );
 	printf( "%d regions in vips\n", g_slist_length( im__regions_all ) );
-	g_mutex_unlock( regions_mutex );
+	g_mutex_unlock( im__global_lock );
 #endif /*DEBUG*/
 }
 
-/* Move reg to position r and make sure it has enough local memory to be able
- * to write to all of r. If there is too little space or we have much to much
- * space, free the old local buffer and reallocate. r is clipped against the 
- * size of the image.
+/* Region should be a pixel buffer. On return, check
+ * reg->buffer->done to see if there are pixels there already. Otherwise, you
+ * need to calculate.
  */
 int
-im_region_local( REGION *reg, Rect *r )
+im_region_buffer( REGION *reg, Rect *r )
 {
-	int sz;
+	IMAGE *im = reg->im;
+
 	Rect image;
 	Rect clipped;
 
+	im__region_check_ownership( reg );
+
 	/* Clip against image.
 	 */
 	image.top = 0;
 	image.left = 0;
-	image.width = reg->im->Xsize;
-	image.height = reg->im->Ysize;
+	image.width = im->Xsize;
+	image.height = im->Ysize;
 	im_rect_intersectrect( r, &image, &clipped );
 
 	/* Test for empty.
 	 */
 	if( im_rect_isempty( &clipped ) ) {
-		im_errormsg( "im_region_local: valid clipped to nothing" );
+		im_error( "im_region_buffer", _( "valid clipped to nothing" ) );
 		return( -1 );
 	}
 
+	/* Already have stuff?
+	 */
+	if( reg->type == IM_REGION_BUFFER &&
+		im_rect_includesrect( &reg->valid, &clipped ) &&
+		reg->buffer &&
+		!reg->buffer->invalid ) 
+		return( 0 );
+
+	/* Don't call im_region_reset() ... we combine buffer unref and new
+	 * buffer ref in one call to reduce malloc/free cycling.
+	 */
+	IM_FREEF( im_window_unref, reg->window );
+	if( !(reg->buffer = im_buffer_unref_ref( reg->buffer, im, &clipped )) )
+		return( -1 );
+
 	/* Init new stuff.
 	 */
-	reg->valid = clipped;
-	reg->bpl = IM_REGION_SIZEOF_LINE( reg );
-
-	/* Need new memory? We have too little space, or the amount of space
-	 * we need is a lot less than the space we have.
-	 */
-	sz = reg->bpl * reg->valid.height;
-	if( sz > reg->bsize || reg->bsize - sz > 10000 ) {
-		/* Free old memory.
-		 */
-		im_region_reset( reg );
-
-		/* Allocate new memory.
-		 */
-		if( !(reg->buf = im_malloc( NULL, sz )) ) 
-			return( -1 );
-		reg->bsize = sz;
-	}
-
-	/* Reset us to local.
-	 */
-	reg->type = IM_REGION_LOCAL;
-	reg->data = reg->buf;
+	reg->valid = reg->buffer->area;
+	reg->bpl = IM_IMAGE_SIZEOF_PEL( im ) * reg->buffer->area.width;
+	reg->type = IM_REGION_BUFFER;
+	reg->data = reg->buffer->buf;
 
 	return( 0 );
 }
@@ -391,10 +374,7 @@ im_region_image( REGION *reg, Rect *r )
 
 	/* Sanity check.
 	 */
-	if( !reg->im->data ) {
-		im_errormsg( "im_region_image: inappropriate image type" );
-		return( -1 );
-	}
+	im__region_check_ownership( reg );
 
 	/* Clip against image.
 	 */
@@ -407,177 +387,54 @@ im_region_image( REGION *reg, Rect *r )
 	/* Test for empty.
 	 */
 	if( im_rect_isempty( &clipped ) ) {
-		im_errormsg( "im_region_image: valid clipped to nothing" );
+		im_error( "im_region_image", _( "valid clipped to nothing" ) );
 		return( -1 );
 	}
 
-	/* Init new stuff.
-	 */
-	reg->valid = clipped;
-	reg->bpl = IM_IMAGE_SIZEOF_LINE( reg->im );
-	reg->data = reg->im->data + 
-		clipped.top * IM_IMAGE_SIZEOF_LINE( reg->im ) +
-		clipped.left * IM_IMAGE_SIZEOF_PEL( reg->im );;
-	reg->type = IM_REGION_OTHER_IMAGE;
-
-	return( 0 );
-}
-
-static int
-im_getpagesize()
-{
-	int pagesize;
-
-#ifdef OS_WIN32
-	SYSTEM_INFO si;
-
-	GetSystemInfo( &si );
-
-	pagesize = si.dwAllocationGranularity;
-#else /*OS_WIN32*/
-	pagesize = getpagesize();
-#endif /*OS_WIN32*/
-
-#ifdef DEBUG
-	printf( "im_getpagesize: 0x%x\n", pagesize );
-#endif /*DEBUG*/
-
-	return( pagesize );
-}
-
-/* Map a window into a file.
- */
-static int
-im_region_set_window( REGION *reg, int window_top, int window_height )
-{
-	static int pagesize = -1;
-
-	void *baseaddr;
-	gint64 start, end, pagestart;
-	size_t length, pagelength;
-
-	/* Calculate start and length for our window.
-	 */
-	start = 64 + (gint64) IM_IMAGE_SIZEOF_LINE( reg->im ) * window_top;
-	length = (size_t) IM_IMAGE_SIZEOF_LINE( reg->im ) * window_height;
-
-	/* We can only mmap multiples of the page size ... round down and up.
-	 */
-	if( pagesize == -1 )
-		pagesize = im_getpagesize();
-
-	pagestart = start - start % pagesize;
-	end = start + length;
-	pagelength = end - pagestart;
-
-	if( !(baseaddr = im__mmap( reg->im->fd, 0, pagelength, pagestart )) )
-		return( -1 ); 
-
-	reg->baseaddr = baseaddr;
-	reg->length = pagelength;
-	reg->data = (char *) baseaddr + (start - pagestart);
-	reg->valid.left = 0;
-	reg->valid.top = window_top;
-	reg->valid.width = reg->im->Xsize;
-	reg->valid.height = window_height;
-	reg->bpl = IM_IMAGE_SIZEOF_LINE( reg->im );
-
-	/* Sanity check ... make sure the data pointer is readable.
-	 */
-	im__read_test = reg->data[0] & reg->baseaddr[0];
-
-#ifdef DEBUG
-	g_mutex_lock( mmap_mutex );
-	total_mmap_usage += reg->length;
-	if( total_mmap_usage > max_mmap_usage )
-		max_mmap_usage = total_mmap_usage;
-	g_mutex_unlock( mmap_mutex );
-#endif /*DEBUG*/
-
-	return( 0 );
-}
-
-/* reg is defined on an OPENIN image ... do the rolling mmap() window stuff.
- * The Rect is clipped against the image size.
- */
-int
-im_region_mmap_window( REGION *reg, Rect *r )
-{
-	Rect image;
-	Rect clipped;
-	Rect window;
-
-	/* Sanity check.
-	 */
-	if( reg->im->dtype != IM_OPENIN ) {
-		im_errormsg( "im_region_mmap_window: bad image type" );
-		return( -1 );
-	}
-
-#ifdef DEBUG_CREATE
-	if( reg->type == IM_REGION_NONE ) {
-		int height = r->height + 2 * im__mmap_window_margin;
-		float size = ((float) height * 
-			IM_IMAGE_SIZEOF_LINE( reg->im )) / (1024 * 1024);
-
-		printf( "im_region_mmap_window: new window 0x%x on file "
-			"\"%s\" of ~%.2fMB\n",
-			(unsigned int) reg, reg->im->filename, size );
-		printf( "im_region_mmap_window: \"%s\" now has %d regions\n",
-			reg->im->filename, 
-			g_slist_length( reg->im->regions ) );
-	}
-#endif /*DEBUG_CREATE*/
-
-	/* Clip against image.
-	 */
-	image.top = 0;
-	image.left = 0;
-	image.width = reg->im->Xsize;
-	image.height = reg->im->Ysize;
-	im_rect_intersectrect( r, &image, &clipped );
-	if( im_rect_isempty( &clipped ) ) {
-		im_errormsg( "im_region_mmap_window: clipped to nothing" );
-		return( -1 );
-	}
-
-	/* Does the area fit inside the current valid area? Move the window if
-	 * it does not.
-	 */
-	if( reg->type != IM_REGION_MMAP_WINDOW ||
-		!im_rect_includesrect( &reg->valid, &clipped ) ) {
-		if( im_region_unmap_window( reg ) )
-			return( -1 );
-
-		/* Calculate the new window size ... the full width of the
-		 * image, the height of the requested area, plus some margins.
+	if( reg->im->data ) {
+		/* We have the whole image available ... easy!
 		 */
-		window.left = 0;
-		window.top = r->top - im__mmap_window_margin;
-		window.width = reg->im->Xsize;
-		window.height = r->height + 2 * im__mmap_window_margin;
-		im_rect_intersectrect( &window, &image, &clipped );
+		im_region_reset( reg );
 
-#ifdef DEBUG_MOVE
-		printf( "im_region_mmap_window: moving window 0x%x on file "
-			"\"%s\"\n"
-			"    from top=%d, height=%d to top=%d, height=%d\n",
-			(unsigned int) reg, reg->im->filename,
-			reg->valid.top, reg->valid.height, 
-			clipped.top, clipped.height );
-#endif /*DEBUG_MOVE*/
-
-		/* Map new window.
+		/* We can't just set valid = clipped, since this may be an
+		 * incompletely calculated memory buffer. Just set valid to r.
 		 */
-		if( im_region_set_window( reg, clipped.top, clipped.height ) ) 
-			return( -1 );
-
-		reg->type = IM_REGION_MMAP_WINDOW;
+		reg->valid = clipped;
+		reg->bpl = IM_IMAGE_SIZEOF_LINE( reg->im );
+		reg->data = reg->im->data +
+			(gint64) clipped.top * IM_IMAGE_SIZEOF_LINE( reg->im ) +
+			clipped.left * IM_IMAGE_SIZEOF_PEL( reg->im );
+		reg->type = IM_REGION_OTHER_IMAGE;
 	}
+	else if( reg->im->dtype == IM_OPENIN ) {
+		/* No complete image data ... but we can use a rolling window.
+		 */
+		if( reg->type != IM_REGION_WINDOW || !reg->window ||
+			reg->window->top > clipped.top ||
+			reg->window->top + reg->window->height < 
+				clipped.top + clipped.height ) {
+			im_region_reset( reg );
 
-#ifdef DEBUG
-	trace_mmap_usage();
-#endif /*DEBUG*/
+			if( !(reg->window = im_window_ref( reg->im, 
+				clipped.top, clipped.height )) )
+				return( -1 );
+
+			reg->type = IM_REGION_WINDOW;
+		}
+
+		/* Note the area the window actually represents.
+		 */
+		reg->valid.left = 0;
+		reg->valid.top = reg->window->top;
+		reg->valid.width = reg->im->Xsize;
+		reg->valid.height = reg->window->height;
+		reg->bpl = IM_IMAGE_SIZEOF_LINE( reg->im );
+		reg->data = reg->window->data;
+	}
+	else {
+		im_error( "im_region_image", _( "bad image type" ) );
+		return( -1 );
+	}
 
 	return( 0 );
 }
@@ -593,7 +450,7 @@ im_region_mmap_window( REGION *reg, Rect *r )
  *
  * If the region we attach to is modified, we are left with dangling pointers!
  * If the region we attach to is on another image, the two images must have 
- * the same BandFmt and number of Bands.
+ * the same sizeof( pel ).
  */
 int
 im_region_region( REGION *reg, REGION *dest, Rect *r, int x, int y )
@@ -606,13 +463,24 @@ im_region_region( REGION *reg, REGION *dest, Rect *r, int x, int y )
 
 	/* Sanity check.
 	 */
-	if( !dest->data || dest->im->BandFmt != reg->im->BandFmt ||
-		dest->im->Bands != reg->im->Bands ) {
-		im_errormsg( "im_region_region: inappropriate region type" );
+	if( !dest->data || 
+		IM_IMAGE_SIZEOF_PEL( dest->im ) != 
+			IM_IMAGE_SIZEOF_PEL( reg->im ) ) {
+		im_error( "im_region_region", 
+			_( "inappropriate region type" ) );
 		return( -1 );
 	}
+	im__region_check_ownership( reg );
 
-	/* Clip r against size of reg.
+	/* We can't test
+
+		assert( dest->thread == g_thread_self() );
+
+	 * since we can have several threads writing to the same region in
+	 * threadgroup.
+	 */
+
+	/* Clip r against size of the image.
 	 */
 	image.top = 0;
 	image.left = 0;
@@ -627,6 +495,16 @@ im_region_region( REGION *reg, REGION *dest, Rect *r, int x, int y )
 	wanted.top = y + (clipped.top - r->top);
 	wanted.width = clipped.width;
 	wanted.height = clipped.height;
+
+	/* Test that dest->valid is large enough.
+	 */
+	if( !im_rect_includesrect( &dest->valid, &wanted ) ) {
+		im_error( "im_region_region", _( "dest too small" ) );
+		return( -1 );
+	}
+
+	/* Clip against the available pixels.
+	 */
 	im_rect_intersectrect( &wanted, &dest->valid, &clipped2 );
 
 	/* Translate back to reg's coordinate space and set as valid.
@@ -639,12 +517,13 @@ im_region_region( REGION *reg, REGION *dest, Rect *r, int x, int y )
 	/* Test for empty.
 	 */
 	if( im_rect_isempty( &final ) ) {
-		im_errormsg( "im_region_region: valid clipped to nothing" );
+		im_error( "im_region_region", _( "valid clipped to nothing" ) );
 		return( -1 );
 	}
 
 	/* Init new stuff.
 	 */
+	im_region_reset( reg );
 	reg->valid = final;
 	reg->bpl = dest->bpl;
 	reg->data = IM_REGION_ADDR( dest, clipped2.left, clipped2.top );
@@ -655,7 +534,8 @@ im_region_region( REGION *reg, REGION *dest, Rect *r, int x, int y )
 
 /* Do two regions point to the same piece of image? ie. 
  * 	IM_REGION_ADDR( reg1, x, y ) == IM_REGION_ADDR( reg2, x, y ) &&
- * 	*IM_REGION_ADDR( reg1, x, y ) == *IM_REGION_ADDR( reg2, x, y ) for all x, y, reg1, reg2.
+ * 	*IM_REGION_ADDR( reg1, x, y ) == 
+ * 		*IM_REGION_ADDR( reg2, x, y ) for all x, y, reg1, reg2.
  */
 int
 im_region_equalsregion( REGION *reg1, REGION *reg2 )
@@ -687,10 +567,37 @@ im_region_position( REGION *reg, int x, int y )
 	req.height = reg->valid.height;
 	im_rect_intersectrect( &image, &req, &clipped );
 	if( x < 0 || y < 0 || im_rect_isempty( &clipped ) ) {
-		im_errormsg( "im_region_position: bad position" );
+		im_error( "im_region_position", _( "bad position" ) );
 		return( -1 );
 	}
 
 	reg->valid = clipped;
+
+	return( 0 );
+}
+
+int
+im_region_fill( REGION *reg, Rect *r, im_region_fill_fn fn, void *a )
+{
+	assert( reg->im->dtype == IM_PARTIAL );
+	assert( reg->im->generate );
+
+	/* Should have local memory.
+	 */
+	if( im_region_buffer( reg, r ) )
+		return( -1 );
+
+	/* Evaluate into or, if we've not got calculated pixels.
+	 */
+	if( !reg->buffer->done ) {
+		if( fn( reg, a ) )
+			return( -1 );
+
+		/* Publish our results.
+		 */
+		if( reg->buffer )
+			im_buffer_done( reg->buffer );
+	}
+
 	return( 0 );
 }
