@@ -97,6 +97,24 @@ im_slist_map2( GSList *list, VSListMap2Fn fn, void *a, void *b )
 	return( result );
 }
 
+void *
+im_slist_map4( GSList *list, 
+	VSListMap4Fn fn, void *a, void *b, void *c, void *d )
+{
+	GSList *copy;
+	GSList *i;
+	void *result;
+
+	copy = g_slist_copy( list );
+	result = NULL;
+	for( i = copy; 
+		i && !(result = fn( i->data, a, b, c, d )); i = i->next ) 
+		;
+	g_slist_free( copy );
+
+	return( result );
+}
+
 /* Map backwards. We _reverse() rather than recurse and unwind to save stack.
  */
 void *
@@ -269,11 +287,37 @@ im_isprefix( const char *a, const char *b )
 	return( TRUE );
 }
 
-/* Like strtok(). Give a string and a list of break characters;
- * write a '\0' into the string over the first break character and return a
- * pointer to the next non-break character. If there are no break characters,
- * then return a pointer to the end of the string. If passed a pointer to an
- * empty string or NULL, then return NULL.
+/* Like strtok(). Give a string and a list of break characters. Then:
+ * - skip initial break characters
+ * - EOS? return NULL
+ * - skip a series of non-break characters
+ * - write a '\0' over the next break character and return a pointer to the
+ *   char after that
+ *
+ * The idea is that this can be used in loops as the iterator. Example:
+ *
+ * char *p = " 1 2 3   "; // mutable 
+ * char *q;
+ * int i;
+ * int v[...];
+ *
+ * for( i = 0; (q = im_break_token( p, " " )); i++, p = q )
+ * 	v[i] = atoi( p );
+ *
+ * will set
+ * 	v[0] = 1;
+ * 	v[1] = 2;
+ * 	v[2] = 3;
+ *
+ * or with just one pointer, provided your atoi() is OK with trailing chars
+ * and you know there is at least one item there
+ *
+ * char *p = " 1 2 3   "; // mutable
+ * int i;
+ * int v[...];
+ *
+ * for( i = 0; p; p = im_break_token( p, " " ) )
+ *   v[i] = atoi( p );
  */
 char *
 im_break_token( char *str, const char *brk )
@@ -289,7 +333,13 @@ im_break_token( char *str, const char *brk )
          */
         p = str + strspn( str, brk );
 
-        /* Search for the first break character after the token.
+	/* No item?
+	 */
+        if( !*p ) 
+		return( NULL );
+
+        /* We have a token ... search for the first break character after the 
+	 * token.
          */
         p += strcspn( p, brk );
 
@@ -397,10 +447,12 @@ im_filename_split( const char *path, char *name, char *mode )
 const char *
 im_skip_dir( const char *path )
 {
-	char name[FILENAME_MAX], mode[FILENAME_MAX];
-        const char *p, *q;
+	char name[FILENAME_MAX];
+	char mode[FILENAME_MAX];
+        const char *p;
+        const char *q;
 
-	const char native_dir_sep = IM_DIR_SEP;
+	const char native_dir_sep = G_DIR_SEPARATOR;
 	const char non_native_dir_sep = native_dir_sep == '/' ? '\\' : '/';
 
 	/* Remove any trailing save modifiers: we don't want '/' or '\' in the
@@ -459,7 +511,7 @@ im_filename_suffix_match( const char *path, const char *suffixes[] )
 
 /* p points to the start of a buffer ... move it on through the buffer (ready
  * for the next call), and return the current option (or NULL for option
- * missing).
+ * missing). ',' characters inside options can be escaped with a '\'.
  */
 char *
 im_getnextoption( char **in )
@@ -470,7 +522,12 @@ im_getnextoption( char **in )
         if( !p || !*p )
                 return( NULL );
 
-        if( (p = strchr( p, ',' )) ) {
+	/* Find the next ',' not prefixed with a '\'.
+	 */
+	while( (p = strchr( p, ',' )) && p[-1] == '\\' )
+		p += 1;
+
+        if( p ) {
                 /* Another option follows this one .. set up to pick that out
                  * next time.
                  */
@@ -494,12 +551,25 @@ im_getnextoption( char **in )
 char *
 im_getsuboption( const char *buf )
 {
-        char *p;
+        char *p, *q, *r;
 
-        if( (p = strchr( buf, ':' )) )
-                return( p + 1 );
+        if( !(p = strchr( buf, ':' )) ) 
+		/* No suboption.
+		 */
+		return( NULL );
 
-        return( NULL );
+	/* Step over the ':'.
+	 */
+	p += 1;
+
+	/* Need to unescape any \, pairs. Shift stuff down one if we find one.
+	 */
+	for( q = p; *q; q++ ) 
+		if( q[0] == '\\' && q[1] == ',' )
+			for( r = q; *r; r++ )
+				r[0] = r[1];
+
+        return( p );
 }
 
 /* Make something local to an image descriptor ... pass in a constructor
@@ -558,16 +628,37 @@ im_file_length( int fd )
 #endif /*OS_WIN32*/
 		im_error_system( errno, "im_file_length", 
 			_( "unable to get file stats" ) );
-		return( 0 );
+		return( -1 );
 	}
 
 	return( st.st_size );
 }
 
+/* Wrap write() up
+ */
+int
+im__write( int fd, const void *buf, size_t count )
+{
+	do {
+		size_t nwritten = write( fd, buf, count );
+
+		if( nwritten == (size_t) -1 ) {
+                        im_error_system( errno, "im__write", 
+				_( "write failed" ) );
+                        return( -1 ); 
+		}
+
+		buf = (void *) ((char *) buf + nwritten);
+		count -= nwritten;
+	} while( count > 0 );
+
+	return( 0 );
+}
+
 /* Load up a file as a string.
  */
 char *
-im__file_read( FILE *fp, const char *name, int *length_out )
+im__file_read( FILE *fp, const char *name, unsigned int *length_out )
 {
         long len;
 	size_t read;
@@ -577,26 +668,60 @@ im__file_read( FILE *fp, const char *name, int *length_out )
          */
         fseek( fp, 0L, 2 );
         len = ftell( fp );
-        rewind( fp );
-        if( len < 0 || len > 20 * 1024 * 1024 ) {
+	if( len > 20 * 1024 * 1024 ) {
 		/* Seems crazy!
 		 */
-                im_error( "im__file_read", _( "file \"%s\" too long" ), name );
+                im_error( "im__file_read", _( "\"%s\" too long" ), name );
                 return( NULL );
         }
 
-        /* Allocate memory and fill.    
-         */
-        if( !(str = im_malloc( NULL, len + 1 )) )
-                return( NULL );
-	read = fread( str, sizeof( char ), (size_t) len, fp );
-        if( read != (size_t) len ) {
-                im_free( str );
-                im_error( "im__file_read", 
-			_( "error reading from file \"%s\"" ), name );
-                return( NULL );
-        }
-        if( length_out )
+	if( len == -1 ) {
+		int size;
+
+		/* Can't get length: read in chunks and realloc() to end of
+		 * file.
+		 */
+		str = NULL;
+		len = 0;
+		size = 0;
+		do {
+			size += 1024;
+			if( !(str = realloc( str, size )) ) {
+				im_error( "im__file_read", 
+					_( "out of memory" ) );
+				return( NULL );
+			}
+
+			/* -1 to allow space for an extra NULL we add later.
+			 */
+			read = fread( str + len, sizeof( char ), 
+				(size - len - 1) / sizeof( char ),
+				fp );
+			len += read;
+		} while( !feof( fp ) );
+
+#ifdef DEBUG
+		printf( "read %d bytes from unseekable stream\n", len );
+#endif /*DEBUG*/
+	}
+	else {
+		/* Allocate memory and fill.    
+		 */
+		if( !(str = im_malloc( NULL, len + 1 )) )
+			return( NULL );
+		rewind( fp );
+		read = fread( str, sizeof( char ), (size_t) len, fp );
+		if( read != (size_t) len ) {
+			im_free( str );
+			im_error( "im__file_read", 
+				_( "error reading from file \"%s\"" ), name );
+			return( NULL );
+		}
+	}
+
+	str[len] = '\0';
+
+	if( length_out )
 		*length_out = len;
 
         return( str );
@@ -605,7 +730,7 @@ im__file_read( FILE *fp, const char *name, int *length_out )
 /* Load from a filename as a string.
  */
 char *
-im__file_read_name( const char *name, int *length_out )
+im__file_read_name( const char *name, unsigned int *length_out )
 {
 	FILE *fp;
 	char *buffer;
@@ -627,3 +752,164 @@ im__file_read_name( const char *name, int *length_out )
 
 	return( buffer );
 }
+
+/* Alloc/free a GValue.
+ */
+static GValue *
+im__gvalue_new( GType type )
+{
+	GValue *value;
+
+	value = g_new0( GValue, 1 );
+	g_value_init( value, type );
+
+	return( value );
+}
+
+static GValue *
+im__gvalue_copy( GValue *value )
+{
+	GValue *value_copy;
+
+	value_copy = im__gvalue_new( G_VALUE_TYPE( value ) );
+	g_value_copy( value, value_copy );
+
+	return( value_copy );
+}
+
+static void
+im__gvalue_free( GValue *value )
+{
+	g_value_unset( value );
+	g_free( value );
+}
+
+GValue *
+im__gvalue_ref_string_new( const char *text )
+{
+	GValue *value;
+
+	value = im__gvalue_new( IM_TYPE_REF_STRING );
+	im_ref_string_set( value, text );
+
+	return( value );
+}
+
+/* Free a GSList of GValue.
+ */
+void
+im__gslist_gvalue_free( GSList *list )
+{
+	g_slist_foreach( list, (GFunc) im__gvalue_free, NULL );
+	g_slist_free( list );
+}
+
+/* Copy a GSList of GValue.
+ */
+GSList *
+im__gslist_gvalue_copy( const GSList *list )
+{
+	GSList *copy;
+	const GSList *p;
+
+	copy = NULL;
+
+	for( p = list; p; p = p->next ) 
+		copy = g_slist_prepend( copy, 
+			im__gvalue_copy( (GValue *) p->data ) );
+
+	copy = g_slist_reverse( copy );
+
+	return( copy );
+}
+
+/* Merge two GSList of GValue ... append to a all elements in b which are not 
+ * in a. Return the new value of a. Works for any vips refcounted type 
+ * (string, blob, etc.).
+ */
+GSList *
+im__gslist_gvalue_merge( GSList *a, const GSList *b )
+{
+	const GSList *i, *j;
+	GSList *tail;
+
+	tail = NULL;
+
+	for( i = b; i; i = i->next ) {
+		GValue *value = (GValue *) i->data;
+
+		assert( G_VALUE_TYPE( value ) == IM_TYPE_REF_STRING );
+
+		for( j = a; j; j = j->next ) {
+			GValue *value2 = (GValue *) j->data;
+
+			assert( G_VALUE_TYPE( value2 ) == IM_TYPE_REF_STRING );
+
+			/* Just do a pointer compare ... good enough 99.9% of 
+			 * the time.
+			 */
+			if( im_ref_string_get( value ) ==
+				im_ref_string_get( value2 ) )
+				break;
+		}
+
+		if( !j )
+			tail = g_slist_prepend( tail, 
+				im__gvalue_copy( value ) );
+	}
+
+	a = g_slist_concat( a, g_slist_reverse( tail ) );
+
+	return( a );
+}
+
+/* Make a char* from GSList of GValue. Each GValue should be a ref_string.
+ * free the result. Empty list -> "", not NULL. Join strings with '\n'.
+ */
+char *
+im__gslist_gvalue_get( const GSList *list )
+{
+	const GSList *p;
+	size_t length;
+	char *all;
+	char *q;
+
+	/* Need to estimate length first.
+	 */
+	length = 0;
+	for( p = list; p; p = p->next ) {
+		GValue *value = (GValue *) p->data;
+
+		assert( G_VALUE_TYPE( value ) == IM_TYPE_REF_STRING );
+
+		/* +1 for the newline we will add for each item.
+		 */
+		length += im_ref_string_get_length( value ) + 1;
+	}
+
+	if( length == 0 )
+		return( NULL );
+
+	/* More than 10MB of history? Madness!
+	 */
+	assert( length < 10 * 1024 * 1024 );
+
+	/* +1 for '\0'.
+	 */
+	if( !(all = im_malloc( NULL, length + 1 )) )
+		return( NULL );
+
+	q = all;
+	for( p = list; p; p = p->next ) {
+		GValue *value = (GValue *) p->data;
+
+		strcpy( q, im_ref_string_get( value ) );
+		q += im_ref_string_get_length( value );
+		strcpy( q, "\n" );
+		q += 1;
+	}
+
+	assert( q - all == length );
+
+	return( all );
+}
