@@ -209,6 +209,19 @@ im_buf_appendc( VBuf *buf, char ch )
 	return( im_buf_appendns( buf, tiny, 1 ) );
 }
 
+/* Append a double, non-localised. Useful for config files etc.
+ */
+gboolean
+im_buf_appendg( VBuf *buf, double g )
+{
+	char text[G_ASCII_DTOSTR_BUF_SIZE];
+
+	g_ascii_dtostr( text, sizeof( text ), g );
+
+	return( im_buf_appends( buf, text ) );
+}
+
+
 /* Swap the rightmost occurence of old for new.
  */
 gboolean
