@@ -159,7 +159,7 @@ find_top( REGION *ir, int *pos, int x, int y, int h )
 	case IM_BANDFMT_DPCOMPLEX:tsearch( double ); break;
 
 	default:
-		im_errormsg( "im_tbmerge: internal error" );
+		im_error( "im_tbmerge", _( "internal error" ) );
 		return( -1 );
 	}
 
@@ -213,7 +213,7 @@ find_bot( REGION *ir, int *pos, int x, int y, int h )
 	case IM_BANDFMT_DPCOMPLEX:rsearch( double ); break;
 
 	default:
-		im_errormsg( "im_tbmerge: internal error" );
+		im_error( "im_tbmerge", _( "internal error" ) );
 		return( -1 );
 	}
 
@@ -531,7 +531,7 @@ tb_blend( REGION *or, MergeInfo *inf, Overlapping *ovlap, Rect *oreg )
 			fblend( double, im->Bands*2, pr, ps, q ); break;
 
 		default:
-			im_errormsg( "im_tbmerge: internal error" );
+			im_error( "im_tbmerge", _( "internal error" ) );
 			return( -1 );
 		}
 	}
@@ -629,7 +629,7 @@ build_tbstate( IMAGE *ref, IMAGE *sec, IMAGE *out, int dx, int dy, int mwidth )
 		break;
 
 	default:
-		im_errormsg( "im_tbmerge: unknown coding type" );
+		im_error( "im_tbmerge", _( "unknown coding type" ) );
 		return( NULL );
 	}
 
@@ -647,7 +647,7 @@ build_tbstate( IMAGE *ref, IMAGE *sec, IMAGE *out, int dx, int dy, int mwidth )
 	 */
 	if( IM_RECT_BOTTOM( &ovlap->rarea ) > IM_RECT_BOTTOM( &ovlap->sarea ) ||
 		ovlap->rarea.top > ovlap->sarea.top ) {
-		im_errormsg( "im_tbmerge: too much overlap" );
+		im_error( "im_tbmerge", _( "too much overlap" ) );
 		return( NULL );
 	}
 
@@ -668,11 +668,12 @@ im__tbmerge( IMAGE *ref, IMAGE *sec, IMAGE *out, int dx, int dy, int mwidth )
 	if( ref->Bands != sec->Bands || ref->Bbits != sec->Bbits ||
 		ref->BandFmt != sec->BandFmt ||
 		ref->Coding != sec->Coding ) {
-		im_errormsg( "im_tbmerge: input images incompatible" );
+		im_error( "im_tbmerge", _( "input images incompatible" ) );
 		return( -1 );
 	}
 	if( ref->Coding != IM_CODING_NONE && ref->Coding != IM_CODING_LABQ ) {
-		im_errormsg( "im_tbmerge: inputs not uncoded or IM_CODING_LABQ" );
+		im_error( "im_tbmerge", 
+			_( "inputs not uncoded or IM_CODING_LABQ" ) );
 		return( -1 );
 	}
 	if( dy > 0 || dy < 1 - ref->Ysize ) {
@@ -695,9 +696,7 @@ im__tbmerge( IMAGE *ref, IMAGE *sec, IMAGE *out, int dx, int dy, int mwidth )
 
 	/* Prepare the output IMAGE.
 	 */ 
-	if( im_cp_desc( out, ref ) )
-		return( -1 );
-	if( im_append_Hist( out, sec ) )
+	if( im_cp_descv( out, ref, sec, NULL ) )
 		return( -1 );
 	out->Xsize = ovlap->oarea.width;
 	out->Ysize = ovlap->oarea.height;
