@@ -49,6 +49,7 @@
 #include <math.h>
 
 #include <vips/vips.h>
+#include <vips/vbuf.h>
 
 #include "mosaic.h"
 #include "merge.h"
@@ -104,6 +105,8 @@ im__lrmerge1( IMAGE *ref, IMAGE *sec, IMAGE *out,
 {
 	Transformation trn;
 	IMAGE *t1 = im_open_local( out, "im_lrmosaic1:1", "p" );
+	VBuf buf;
+	char text[1024];
 
 	/* Scale, rotate and displace sec.
 	 */
@@ -119,10 +122,18 @@ im__lrmerge1( IMAGE *ref, IMAGE *sec, IMAGE *out,
 	/* Note parameters in history file ... for global balance to pick up
 	 * later.
 	 */
-	if( im_histlin( out, "#LRROTSCALE <%s> <%s> <%s> "
-		"<%g> <%g> <%g> <%g> <%d>", 
-		ref->filename, sec->filename, out->filename, 
-		a, b, dx, dy, mwidth ) )
+	im_buf_init_static( &buf, text, 1024 );
+	im_buf_appendf( &buf, "#LRROTSCALE <%s> <%s> <%s> <",
+		ref->filename, sec->filename, out->filename ); 
+	im_buf_appendg( &buf, a );
+	im_buf_appendf( &buf, "> <" );
+	im_buf_appendg( &buf, b );
+	im_buf_appendf( &buf, "> <" );
+	im_buf_appendg( &buf, dx );
+	im_buf_appendf( &buf, "> <" );
+	im_buf_appendg( &buf, dy );
+	im_buf_appendf( &buf, "> <%d>", mwidth );
+	if( im_histlin( out, im_buf_all( &buf ) ) )
 		return( -1 );
 
 	return( 0 );
@@ -136,6 +147,8 @@ im__tbmerge1( IMAGE *ref, IMAGE *sec, IMAGE *out,
 {
 	Transformation trn;
 	IMAGE *t1 = im_open_local( out, "im_lrmosaic1:2", "p" );
+	VBuf buf;
+	char text[1024];
 
 	/* Scale, rotate and displace sec.
 	 */
@@ -151,10 +164,18 @@ im__tbmerge1( IMAGE *ref, IMAGE *sec, IMAGE *out,
 	/* Note parameters in history file ... for global balance to pick up
 	 * later.
 	 */
-	if( im_histlin( out, "#TBROTSCALE <%s> <%s> <%s> "
-		"<%g> <%g> <%g> <%g> <%d>", 
-		ref->filename, sec->filename, out->filename, 
-		a, b, dx, dy, mwidth ) )
+	im_buf_init_static( &buf, text, 1024 );
+	im_buf_appendf( &buf, "#TBROTSCALE <%s> <%s> <%s> <",
+		ref->filename, sec->filename, out->filename ); 
+	im_buf_appendg( &buf, a );
+	im_buf_appendf( &buf, "> <" );
+	im_buf_appendg( &buf, b );
+	im_buf_appendf( &buf, "> <" );
+	im_buf_appendg( &buf, dx );
+	im_buf_appendf( &buf, "> <" );
+	im_buf_appendg( &buf, dy );
+	im_buf_appendf( &buf, "> <%d>", mwidth );
+	if( im_histlin( out, im_buf_all( &buf ) ) )
 		return( -1 );
 
 	return( 0 );
@@ -401,10 +422,10 @@ old_lrmosaic1( IMAGE *ref, IMAGE *sec, IMAGE *out,
 
 	/* Now combine the two transformations to get a corrected transform.
 	 */
-	af = a1*a - b1*b;
-	bf = a1*b + b1*a;
-	dxf = a1*dx - b1*dy + dx1;
-	dyf = b1*dx + a1*dy + dy1;
+	af = a1 * a - b1 * b;
+	bf = a1 * b + b1 * a;
+	dxf = a1 * dx - b1 * dy + dx1;
+	dyf = b1 * dx + a1 * dy + dy1;
 
 	printf( "transform was: a = %g, b = %g, dx = %g, dy = %g\n",
 		a, b, dx, dy );
