@@ -151,7 +151,7 @@ relational_process( char *name, IMAGE **in, IMAGE *out,
 
 	/* Prepare the output image.
 	 */
-	if( im_cp_desc( out, in[0] ) )
+	if( im_cp_desc_array( out, in ) )
 		return( -1 );
 	out->BandFmt = IM_BANDFMT_UCHAR;
 	out->Bbits = IM_BBITS_BYTE;
