@@ -76,7 +76,33 @@ static im_function video_v4l1_desc = {
         video_v4l1_arg_types            /* Arg list */
 };
 
+static int
+video_test_vec( im_object *argv )
+{
+        IMAGE *out = argv[0];
+        int brightness = *((int*)argv[1]);
+        int error = *((int*)argv[2]);
+
+        return( im_video_test( out, brightness, error ) );
+}
+
+static im_arg_desc video_test_arg_types[] = {
+        IM_OUTPUT_IMAGE( "out" ),
+        IM_INPUT_INT( "brightness" ),
+        IM_INPUT_INT( "error" )
+};
+
+static im_function video_test_desc = {
+        "im_video_test",                /* Name */
+        "test video grabber",		/* Description */
+        IM_FN_NOCACHE,                  /* Flags */
+        video_test_vec,                 /* Dispatch function */
+        IM_NUMBER( video_test_arg_types ), /* Size of arg list */
+        video_test_arg_types            /* Arg list */
+};
+
 static im_function *video_list[] = {
+        &video_test_desc,
         &video_v4l1_desc
 };
 
