@@ -31,8 +31,8 @@
 #endif /*HAVE_CONFIG_H*/
 #include <vips/intl.h>
 
-#include <stdio.h>
-#include <stdlib.h>
+#include <cstdio>
+#include <cstdlib>
 
 #include <iostream>
 
@@ -43,16 +43,18 @@
 #include <dmalloc.h>
 #endif /*WITH_DMALLOC*/
 
+VIPS_NAMESPACE_START
+
 void VError::perror() 
 { 
-	std::cerr << note; 
-	exit( 1 ); 
+	std::cerr << _what; 
+	exit( 1 );
 }
 
 void VError::perror( const char *name ) 
 { 
-	std::cerr << name << ": " << note; 
-	exit( 1 ); 
+	std::cerr << name << ": " << _what; 
+	exit( 1 );
 }
 
 // Add a new bit to the end of the error buffer
@@ -61,43 +63,36 @@ VError &VError::app( const int i )
 	char buf[ 256 ];
 
 	sprintf( buf, "%d", i );
-	note += buf;
+	_what += buf;
 
 	return( *this );
 }
 
-VError &VError::app( const char *txt ) 
+VError &VError::app( std::string txt ) 
 { 
-	note += txt; 
+	_what += txt; 
 
 	return( *this );
 }; 
 
-// Error from VImage wrapper
-void verror( const char *str )
+void VError::ostream_print( std::ostream &file ) const
+{
+	file << _what;
+}
+
+void verror( std::string str ) throw( VError )
 {
 	VError err;
 
-	err.app( "VImage error: " ).app( str ).app( "\n" );
+	err.app( "VIPS error: " );
+	if( str == "" ) {
+		err.app( im_error_buffer() );
+		im_error_clear();
+	}
+	else 
+		err.app( str ).app( "\n" );
 
 	throw( err );
 }
 
-// Error from VIPS function
-void verror()
-{
-	VError err;
-
-	err.app( "VIPS error: " ).app( im_error_buffer() );
-	im_error_clear();
-
-	throw( err );
-}
-
-// Output accumulated string
-std::ostream &operator<<( std::ostream &file, const VError &err )
-{ 
-	file << err.note;
-
-	return file; 
-}
+VIPS_NAMESPACE_END
