@@ -31,8 +31,8 @@
 #endif /*HAVE_CONFIG_H*/
 #include <vips/intl.h>
 
-#include <stdlib.h>
-#include <math.h>
+#include <cstdlib>
+#include <cmath>
 
 #include <vips/vips.h>
 #include <vips/vipscpp.h>
@@ -41,6 +41,8 @@
 #include <dmalloc.h>
 #endif /*WITH_DMALLOC*/
 
+VIPS_NAMESPACE_START
+
 /* Functions for VMask - refcounting layer over VPMask.
  */
 
@@ -84,12 +86,17 @@ void VMask::make_private()
 	}
 }
 
+void VMask::ostream_print( std::ostream &file ) const
+{
+	file << *(ref->pmask);
+}
+
 // Embed INTMASK in VIMask
-void VIMask::embed( void *i )
+void VIMask::embed( INTMASK *i ) throw( VError )
 {
 	if( ref->pmask )
 		verror( "embed: VIMask not empty" );
-	ref->pmask = new VPIMask( i );
+	ref->pmask = new _private_detail::VPIMask( i );
 }
 
 // Type conversions: implicit INTMASK to DOUBLEMASK 
@@ -97,8 +104,8 @@ VIMask::operator VDMask()
 {
 	VDMask out( xsize(), ysize() );
 
-	((DOUBLEMASK *) out.mask())->scale = scale();
-	((DOUBLEMASK *) out.mask())->offset = offset();
+	out.mask().dptr->scale = scale();
+	out.mask().dptr->offset = offset();
 
 	for( int i = 0; i < size(); i++ )
 		out[i] = (*this)[i];
@@ -115,8 +122,8 @@ VDMask::operator VIMask()
 {
 	VIMask out( xsize(), ysize() );
 
-	((INTMASK *) out.mask())->scale = int( scale() );
-	((INTMASK *) out.mask())->offset = int( offset() );
+	out.mask().iptr->scale = int( scale() );
+	out.mask().iptr->offset = int( offset() );
 
 	for( int i = 0; i < size(); i++ )
 		out[i] = (int) rint( (*this)[i] );
@@ -125,11 +132,11 @@ VDMask::operator VIMask()
 }
 
 // Type conversions: implicit DOUBLEMASK to VImage
-VDMask::operator VImage()
+VDMask::operator VImage() throw( VError )
 {
 	VImage out;
 
-	if( im_mask2vips( (DOUBLEMASK *) mask(), (IMAGE *) out.image() ) )
+	if( im_mask2vips( mask().dptr, out.image() ) )
 		verror();
 
 	return( out );
@@ -139,76 +146,78 @@ VDMask::operator VImage()
 VIMask::operator VImage() { return( VImage( VDMask( *this ) ) ); }
 
 // Embed DOUBLEMASK in VDMask
-void VDMask::embed( void *i )
+void VDMask::embed( DOUBLEMASK *i ) throw( VError )
 {
 	if( ref->pmask )
 		verror( "embed: VDMask not empty" );
-	ref->pmask = new VPDMask( i );
+	ref->pmask = new _private_detail::VPDMask( i );
 }
 
 /* Functions for P*Mask - layer over im_*_*mask() functions.
  */
 
 // Create empty imask
-VPIMask::VPIMask( int xsize, int ysize )
+_private_detail::VPIMask::VPIMask( int xsize, int ysize ) throw( VError )
 {
-	if( !(data = im_create_imask( "VPIMask::VPIMask", xsize, ysize )) )
+	if( !(data.iptr = im_create_imask( "VPIMask::VPIMask", xsize, ysize )) )
 		verror();
-	type = VPMask::INT;
+	type = _private_detail::VPMask::INT;
 }
 
 // Init from data
-VPIMask::VPIMask( int xsize, int ysize, int scale, int offset, va_list ap )
+_private_detail::VPIMask::VPIMask( int xsize, int ysize, 
+	int scale, int offset, va_list ap )
+	throw( VError )
 {
-	if( !(data = im_create_imask( "VPIMask::VPIMask", xsize, ysize )) )
+	if( !(data.iptr = im_create_imask( "VPIMask::VPIMask", xsize, ysize )) )
 		verror();
-	type = VPMask::INT;
+	type = _private_detail::VPMask::INT;
 
-	((INTMASK *) data)->scale = scale;
-	((INTMASK *) data)->offset = offset;
+	data.iptr->scale = scale;
+	data.iptr->offset = offset;
 	for( int i = 0; i < xsize * ysize; i++ )
-		((INTMASK *) data)->coeff[i] = va_arg( ap, int );
+		data.iptr->coeff[i] = va_arg( ap, int );
 }
 
 // Create from filename
-VPIMask::VPIMask( const char *name )
+_private_detail::VPIMask::VPIMask( const char *name ) throw( VError )
 {
-	if( !(data = im_read_imask( (char *) name )) )
+	if( !(data.iptr = im_read_imask( (char *) name )) )
 		verror();
-	type = VPMask::INT;
+	type = _private_detail::VPMask::INT;
 }
 
 // Create from existing INTMASK
-VPIMask::VPIMask( void *imask )
+_private_detail::VPIMask::VPIMask( INTMASK *imask )
 {
-	data = imask;
-	type = VPMask::INT;
+	data.iptr = imask;
+	type = _private_detail::VPMask::INT;
 }
 
 // Create empty
-VPIMask::VPIMask()
+_private_detail::VPIMask::VPIMask()
 {
-	data = 0;
-	type = VPMask::UNASSIGNED;
+	data.iptr = 0;
+	type = _private_detail::VPMask::UNASSIGNED;
 }
 
-VPIMask::~VPIMask()
+_private_detail::VPIMask::~VPIMask()
 {
-	if( data ) {
-		im_free_imask( (INTMASK *) data );
-		data = 0;
-		type = VPMask::UNASSIGNED;
+	if( data.iptr ) {
+		im_free_imask( data.iptr );
+		data.iptr = 0;
+		type = _private_detail::VPMask::UNASSIGNED;
 	}
 }
 
 // Duplicate -- we are a VPIMask, return a new VPIMask which is a copy of us.
 // Return as a VPMask tho'.
-VPMask *VPIMask::dup() const
+_private_detail::VPMask *_private_detail::VPIMask::dup() const throw( VError )
 {
-	VPIMask *out = new VPIMask();
+	_private_detail::VPIMask *out = new _private_detail::VPIMask();
 
 	INTMASK *msk;
-	if( !(msk = im_dup_imask( (INTMASK *) data, "VPIMask::dup" )) ) {
+	if( !(msk = im_dup_imask( data.iptr, "VPIMask::dup" )) ) {
 		delete out;
 		verror();
 	}
@@ -218,63 +227,63 @@ VPMask *VPIMask::dup() const
 }
 
 // Insert INTMASK pointer
-void VPIMask::embed( void *msk )
+void _private_detail::VPIMask::embed( INTMASK *msk ) throw( VError )
 {
-	if( type != VPMask::UNASSIGNED )
+	if( type != _private_detail::VPMask::UNASSIGNED )
 		verror( "VPIMask::embed: VPIMask not empty" );
 
-	data = msk;
-	type = VPMask::INT;
+	data.iptr = msk;
+	type = _private_detail::VPMask::INT;
 }
 
-int VPIMask::xsize() const
+int _private_detail::VPIMask::xsize() const throw( VError )
 {
-	if( !data ) 
+	if( !data.iptr ) 
 		verror( "xsize: mask not set" );
 
-	return( ((INTMASK *) data)->xsize );
+	return( data.iptr->xsize );
 }
 
-int VPIMask::ysize() const
+int _private_detail::VPIMask::ysize() const throw( VError )
 {
-	if( !data ) 
+	if( !data.iptr ) 
 		verror( "ysize: mask not set" );
 
-	return( ((INTMASK *) data)->ysize );
+	return( data.iptr->ysize );
 }
 
-int VPIMask::scale() const
+int _private_detail::VPIMask::scale() const throw( VError )
 {
-	if( !data ) 
+	if( !data.iptr ) 
 		verror( "scale: mask not set" );
 
-	return( ((INTMASK *) data)->scale );
+	return( data.iptr->scale );
 }
 
-int VPIMask::offset() const
+int _private_detail::VPIMask::offset() const throw( VError )
 {
-	if( !data ) 
+	if( !data.iptr ) 
 		verror( "offset: mask not set" );
 
-	return( ((INTMASK *) data)->offset );
+	return( data.iptr->offset );
 }
 
-const char *VPIMask::filename() const
+const char *_private_detail::VPIMask::filename() const throw( VError )
 {
-	if( !data ) 
+	if( !data.iptr ) 
 		verror( "filename: mask not set" );
 
-	return( ((INTMASK *) data)->filename );
+	return( data.iptr->filename );
 }
 
-void VPIMask::print( std::ostream &file ) const
+void _private_detail::VPIMask::ostream_print( std::ostream &file ) const 
+	throw( VError )
 {
-	if( !data )
+	if( !data.iptr )
 		verror( "internal error #7447234" );
 	
-	INTMASK *dmsk = (INTMASK *) data;
 	int i, j;
-	int *p = dmsk->coeff;
+	int *p = data.iptr->coeff;
 
 	file << this->xsize() << "\t" << this->ysize() << "\t";
 	file << this->scale() << "\t" << this->offset() << "\n";
@@ -288,72 +297,72 @@ void VPIMask::print( std::ostream &file ) const
 }
 
 // Extract start of int array
-int *VPIMask::array() const 
+int *_private_detail::VPIMask::array() const 
 { 
-	return( ((INTMASK *)data)->coeff ); 
+	return( data.iptr->coeff ); 
 }
 
 // Create empty dmask
-VPDMask::VPDMask( int xsize, int ysize )
+_private_detail::VPDMask::VPDMask( int xsize, int ysize ) throw( VError )
 {
-	if( !(data = im_create_dmask( "VPDMask::VPDMask", xsize, ysize )) )
+	if( !(data.dptr = im_create_dmask( "VPDMask::VPDMask", xsize, ysize )) )
 		verror();
-	type = VPMask::DOUBLE;
+	type = _private_detail::VPMask::DOUBLE;
 }
 
 // Create from args
-VPDMask::VPDMask( int xsize, int ysize, 
-	double scale, double offset, va_list ap )
+_private_detail::VPDMask::VPDMask( int xsize, int ysize, 
+	double scale, double offset, va_list ap ) throw( VError )
 {
-	if( !(data = im_create_dmask( "VPDMask::VPDMask", xsize, ysize )) )
+	if( !(data.dptr = im_create_dmask( "VPDMask::VPDMask", xsize, ysize )) )
 		verror();
-	type = VPMask::DOUBLE;
+	type = _private_detail::VPMask::DOUBLE;
 
-	((DOUBLEMASK *) data)->scale = scale;
-	((DOUBLEMASK *) data)->offset = offset;
+	data.dptr->scale = scale;
+	data.dptr->offset = offset;
 	for( int i = 0; i < xsize * ysize; i++ )
-		((DOUBLEMASK *) data)->coeff[i] = va_arg( ap, double );
+		data.dptr->coeff[i] = va_arg( ap, double );
 }
 
 // Create from filename
-VPDMask::VPDMask( const char *name )
+_private_detail::VPDMask::VPDMask( const char *name ) throw( VError )
 {
-	if( !(data = im_read_dmask( (char *) name )) )
+	if( !(data.dptr = im_read_dmask( (char *) name )) )
 		verror();
-	type = VPMask::DOUBLE;
+	type = _private_detail::VPMask::DOUBLE;
 }
 
 // Create empty
-VPDMask::VPDMask()
+_private_detail::VPDMask::VPDMask()
 {
-	data = 0;
-	type = VPMask::UNASSIGNED;
+	data.dptr = 0;
+	type = _private_detail::VPMask::UNASSIGNED;
 }
 
 // Create from existing DOUBLEMASK
-VPDMask::VPDMask( void *dmask )
+_private_detail::VPDMask::VPDMask( DOUBLEMASK *dmask )
 {
-	data = dmask;
-	type = VPMask::DOUBLE;
+	data.dptr = dmask;
+	type = _private_detail::VPMask::DOUBLE;
 }
 
-VPDMask::~VPDMask()
+_private_detail::VPDMask::~VPDMask()
 {
-	if( data ) {
-		im_free_dmask( (DOUBLEMASK *) data );
-		data = 0;
-		type = VPMask::UNASSIGNED;
+	if( data.dptr ) {
+		im_free_dmask( data.dptr );
+		data.dptr = 0;
+		type = _private_detail::VPMask::UNASSIGNED;
 	}
 }
 
 // Duplicate -- we are a VPIMask, return a new VPIMask which is a copy of us.
 // Return as a VPMask tho'.
-VPMask *VPDMask::dup() const
+_private_detail::VPMask *_private_detail::VPDMask::dup() const throw( VError )
 {
-	VPDMask *out = new VPDMask();
+	_private_detail::VPDMask *out = new _private_detail::VPDMask();
 
 	DOUBLEMASK *msk;
-	if( !(msk = im_dup_dmask( (DOUBLEMASK *) data, "VPDMask::dup" )) ) {
+	if( !(msk = im_dup_dmask( data.dptr, "VPDMask::dup" )) ) {
 		delete out;
 		verror();
 	}
@@ -363,63 +372,63 @@ VPMask *VPDMask::dup() const
 }
 
 // Insert DOUBLEMASK pointer
-void VPDMask::embed( void *msk )
+void _private_detail::VPDMask::embed( DOUBLEMASK *msk ) throw( VError )
 {
-	if( type != VPMask::UNASSIGNED )
+	if( type != _private_detail::VPMask::UNASSIGNED )
 		verror( "VPDMask::embed: VPDMask not empty" );
 
-	data = msk;
-	type = VPMask::DOUBLE;
+	data.dptr = msk;
+	type = _private_detail::VPMask::DOUBLE;
 }
 
-int VPDMask::xsize() const
+int _private_detail::VPDMask::xsize() const throw( VError )
 {
-	if( !data ) 
+	if( !data.dptr ) 
 		verror( "xsize: mask not set" );
 
-	return( ((DOUBLEMASK *) data)->xsize );
+	return( data.dptr->xsize );
 }
 
-int VPDMask::ysize() const
+int _private_detail::VPDMask::ysize() const throw( VError )
 {
-	if( !data ) 
+	if( !data.dptr ) 
 		verror( "ysize: mask not set" );
 
-	return( ((DOUBLEMASK *) data)->ysize );
+	return( data.dptr->ysize );
 }
 
-double VPDMask::scale() const
+double _private_detail::VPDMask::scale() const throw( VError )
 {
-	if( !data ) 
+	if( !data.dptr ) 
 		verror( "scale: mask not set" );
 
-	return( ((DOUBLEMASK *) data)->scale );
+	return( data.dptr->scale );
 }
 
-double VPDMask::offset() const
+double _private_detail::VPDMask::offset() const throw( VError )
 {
-	if( !data ) 
+	if( !data.dptr ) 
 		verror( "offset: mask not set" );
 
-	return( ((DOUBLEMASK *) data)->offset );
+	return( data.dptr->offset );
 }
 
-const char *VPDMask::filename() const
+const char *_private_detail::VPDMask::filename() const throw( VError )
 {
-	if( !data ) 
+	if( !data.dptr ) 
 		verror( "filename: mask not set" );
 
-	return( ((DOUBLEMASK *) data)->filename );
+	return( data.dptr->filename );
 }
 
-void VPDMask::print( std::ostream &file ) const
+void _private_detail::VPDMask::ostream_print( std::ostream &file ) const 
+	throw( VError )
 {
-	if( !data )
+	if( !data.dptr )
 		verror( "internal error #7447234" );
 	
-	DOUBLEMASK *dmsk = (DOUBLEMASK *) data;
 	int i, j;
-	double *p = dmsk->coeff;
+	double *p = data.dptr->coeff;
 
 	file << this->xsize() << "\t" << this->ysize() << "\t";
 	file << this->scale() << "\t" << this->offset() << "\n";
@@ -433,13 +442,13 @@ void VPDMask::print( std::ostream &file ) const
 }
 
 // Extract data pointer
-double *VPDMask::array() const 
+double *_private_detail::VPDMask::array() const 
 { 
-	return( ((DOUBLEMASK *)data)->coeff ); 
+	return( data.dptr->coeff ); 
 }
 
 // Build functions
-VIMask VIMask::gauss( double sig, double minamp )
+VIMask VIMask::gauss( double sig, double minamp ) throw( VError )
 {
 	VIMask out;
 	INTMASK *msk;
@@ -451,7 +460,7 @@ VIMask VIMask::gauss( double sig, double minamp )
 	return( out );
 }
 
-VDMask VDMask::gauss( double sig, double minamp )
+VDMask VDMask::gauss( double sig, double minamp ) throw( VError )
 {
 	VDMask out;
 	DOUBLEMASK *msk;
@@ -463,7 +472,7 @@ VDMask VDMask::gauss( double sig, double minamp )
 	return( out );
 }
 
-VIMask VIMask::log( double sig, double minamp )
+VIMask VIMask::log( double sig, double minamp ) throw( VError )
 {
 	VIMask out;
 	INTMASK *msk;
@@ -475,7 +484,7 @@ VIMask VIMask::log( double sig, double minamp )
 	return( out );
 }
 
-VDMask VDMask::log( double sig, double minamp )
+VDMask VDMask::log( double sig, double minamp ) throw( VError )
 {
 	VDMask out;
 	DOUBLEMASK *msk;
@@ -488,114 +497,108 @@ VDMask VDMask::log( double sig, double minamp )
 }
 
 // Manipulation functions
-VIMask VIMask::rotate45()
+VIMask VIMask::rotate45() throw( VError )
 {
 	VIMask out;
 	INTMASK *msk;
 
-	if( !(msk = im_rotate_imask45( (INTMASK *) mask(), 
-		"VIMask::rotate45" )) )
+	if( !(msk = im_rotate_imask45( mask().iptr, "VIMask::rotate45" )) )
 		verror();
 	out.embed( msk );
 
 	return( out );
 }
 
-VIMask VIMask::rotate90()
+VIMask VIMask::rotate90() throw( VError )
 {
 	VIMask out;
 	INTMASK *msk;
 
-	if( !(msk = im_rotate_imask90( (INTMASK *) mask(), 
-		"VIMask::rotate90" )) )
+	if( !(msk = im_rotate_imask90( mask().iptr, "VIMask::rotate90" )) )
 		verror();
 	out.embed( msk );
 
 	return( out );
 }
 
-VDMask VDMask::rotate45()
+VDMask VDMask::rotate45() throw( VError )
 {
 	VDMask out;
 	DOUBLEMASK *msk;
 
-	if( !(msk = im_rotate_dmask45( (DOUBLEMASK *) mask(), 
-		"VDMask::rotate45" )) )
+	if( !(msk = im_rotate_dmask45( mask().dptr, "VDMask::rotate45" )) )
 		verror();
 	out.embed( msk );
 
 	return( out );
 }
 
-VDMask VDMask::rotate90()
+VDMask VDMask::rotate90() throw( VError )
 {
 	VDMask out;
 	DOUBLEMASK *msk;
 
-	if( !(msk = im_rotate_dmask90( (DOUBLEMASK *) mask(), 
-		"VDMask::rotate90" )) )
+	if( !(msk = im_rotate_dmask90( mask().dptr, "VDMask::rotate90" )) )
 		verror();
 	out.embed( msk );
 
 	return( out );
 }
 
-VDMask VDMask::trn()
+VDMask VDMask::trn() throw( VError )
 {
 	VDMask out;
 	DOUBLEMASK *msk;
 
-	if( !(msk = im_mattrn( (DOUBLEMASK *) mask(), "VDMask::trn" )) )
+	if( !(msk = im_mattrn( mask().dptr, "VDMask::trn" )) )
 		verror();
 	out.embed( msk );
 
 	return( out );
 }
 
-VDMask VDMask::inv()
+VDMask VDMask::inv() throw( VError )
 {
 	VDMask out;
 	DOUBLEMASK *msk;
 
-	if( !(msk = im_matinv( (DOUBLEMASK *) mask(), "VDMask::inv" )) )
+	if( !(msk = im_matinv( mask().dptr, "VDMask::inv" )) )
 		verror();
 	out.embed( msk );
 
 	return( out );
 }
 
-VDMask VDMask::mul( VDMask m )
+VDMask VDMask::mul( VDMask m ) throw( VError )
 {
 	VDMask out;
 	DOUBLEMASK *msk;
 
-	if( !(msk = im_matmul( (DOUBLEMASK *) mask(), (DOUBLEMASK *) m.mask(), 
-		"VDMask::mul" )) )
+	if( !(msk = im_matmul( mask().dptr, m.mask().dptr, "VDMask::mul" )) )
 		verror();
 	out.embed( msk );
 
 	return( out );
 }
 
-VDMask VDMask::cat( VDMask m )
+VDMask VDMask::cat( VDMask m ) throw( VError )
 {
 	VDMask out;
 	DOUBLEMASK *msk;
 
-	if( !(msk = im_matcat( (DOUBLEMASK *) mask(), (DOUBLEMASK *) m.mask(), 
-		"VDMask::cat" )) )
+	if( !(msk = im_matcat( mask().dptr, m.mask().dptr, "VDMask::cat" )) )
 		verror();
 	out.embed( msk );
 
 	return( out );
 }
 
-VIMask VDMask::scalei()
+VIMask VDMask::scalei() throw( VError )
 {
-	VDMask out;
+	VIMask out;
 	INTMASK *msk;
 
-	if( !(msk = im_scale_dmask( (DOUBLEMASK *) mask(), "VDMask::scalei" )) )
+	if( !(msk = im_scale_dmask( mask().dptr, "VDMask::scalei" )) )
 		verror();
 	out.embed( msk );
 
@@ -603,16 +606,20 @@ VIMask VDMask::scalei()
 }
 
 // Arithmetic on a VIMask ... just cast and use VDMask
-VDMask VIMask::trn() { return( ((VDMask)*this).trn() ); }
-VDMask VIMask::inv() { return( ((VDMask)*this).inv() ); }
-VDMask VIMask::cat( VDMask a ) { return( ((VDMask)*this).cat( a ) ); }
-VDMask VIMask::mul( VDMask a ) { return( ((VDMask)*this).mul( a ) ); }
+VDMask VIMask::trn() throw( VError ) 
+	{ return( ((VDMask)*this).trn() ); }
+VDMask VIMask::inv() throw( VError ) 
+	{ return( ((VDMask)*this).inv() ); }
+VDMask VIMask::cat( VDMask a ) throw( VError ) 
+	{ return( ((VDMask)*this).cat( a ) ); }
+VDMask VIMask::mul( VDMask a ) throw( VError ) 
+	{ return( ((VDMask)*this).mul( a ) ); }
 
 // Overload [] to get linear array subscript.
 // Our caller may write to the result, so make sure we have a private
 // copy.
 // Involves function call, slow anyway, so do range checking
-int &VIMask::operator[]( int x )
+int &VIMask::operator[]( int x ) throw( VError )
 { 
 	if( ref->nrefs != 1 )
 		make_private();
@@ -620,10 +627,10 @@ int &VIMask::operator[]( int x )
 	if( x > size() )
 		verror( "VIMask::operator[]: subscript out of range" );
 
-	return( ((VPIMask *)ref->pmask)->array()[x] ); 
+	return( ((_private_detail::VPIMask *)ref->pmask)->array()[x] ); 
 }
 
-double &VDMask::operator[]( int x )
+double &VDMask::operator[]( int x ) throw( VError )
 { 
 	if( ref->nrefs != 1 )
 		make_private();
@@ -631,5 +638,7 @@ double &VDMask::operator[]( int x )
 	if( x > size() )
 		verror( "VDMask::operator[]: subscript out of range" );
 
-	return( ((VPDMask *)ref->pmask)->array()[x] ); 
+	return( ((_private_detail::VPDMask *)ref->pmask)->array()[x] ); 
 }
+
+VIPS_NAMESPACE_END
