@@ -68,15 +68,14 @@ main( int argc, char **argv )
 		error_exit( "unable to start VIPS" );
 
 	if( !(out = im_open( argv[2], "w" )) )
-		error_exit( "binfile: unable to open %s for output", argv[2] );
+		error_exit( "unable to open %s for output", argv[2] );
 	if( !(bin = im_binfile( argv[1], xs, ys, bands, offset )) )
-		error_exit( "binfile: unable to im_binfile" );
+		error_exit( "unable to im_binfile" );
 	if( im_copy( bin, out ) )
-		error_exit( "binfile: unable to copy to %s", argv[2] );
+		error_exit( "unable to copy to %s", argv[2] );
 
-	if( im_close( out ) || im_close( bin ) )
-		error_exit( "binfile: unable to close %s or %s", 
-			argv[1], argv[2] );
+	im_close( out );
+	im_close( bin );
 
 	return( 0 );
 }
