@@ -52,19 +52,18 @@ main( int argc, char **argv )
 	IMAGE  *in;
 
 	if( argc != 2 )
-		error_exit( "Usage: %s infile", argv[0] );
+		error_exit( "usage: %s infile", argv[0] );
 
 	if( im_init_world( argv[0] ) )
 	        error_exit( "unable to start VIPS" );
 
 	if( !(in = im_open( argv[1], "r" )) )
-		error_exit( "debugim: Unable to open %s for input", argv[1]);
+		error_exit( "unable to open %s for input", argv[1]);
 
 	if( im_debugim( in ) )
-		error_exit( "debugim: unable to im_debugim");
+		error_exit( "unable to im_debugim");
 
-	if( im_close( in ) )
-		error_exit( "debugim: Unable to close %s", argv[1]);
+	im_close( in );
 
 	return( 0 );
 }
