@@ -1,6 +1,29 @@
 /* VIPS universal main program.
  *
  * J. Cupitt, 8/4/93.
+ * 12/5/06
+ * 	- use GOption. g_*_prgname()
+ * 16/7/06
+ * 	- hmm, was broken for function name as argv1 case
+ * 11/7/06
+ * 	- add "all" option to -l
+ * 14/7/06
+ * 	- ignore "--" arguments.
+ * 2/9/06
+ * 	- do less init ... im_init_world() does more now
+ * 18/8/06
+ * 	- use IM_EXEEXT
+ * 16/10/06
+ * 	- add --version
+ * 17/10/06
+ * 	- add --swig
+ * 	- cleanups
+ * 	- remove --swig again, sigh
+ * 	- add throw() decls to C++ to help SWIG
+ * 14/1/07
+ * 	- add --list packages
+ * 26/2/07
+ * 	- add input *VEC arg types to C++ binding
  */
 
 /*
@@ -45,38 +68,114 @@
 #define strcasecmp(a,b) _stricmp(a,b)
 #endif
 
-/* Name we were run as.
- */
-static char *our_name;
+static char *main_option_list = NULL;
+static char *main_option_usage = NULL;
+static char *main_option_plugin = NULL;
+static gboolean main_option_links;
+static char *main_option_cpph = NULL;
+static char *main_option_cppc = NULL;
+static gboolean *main_option_version;
 
-/* Print a package's name.
+static GOptionEntry main_option[] = {
+	{ "list", 'l', 0, G_OPTION_ARG_STRING, &main_option_list, 
+		N_( "list operations in PACKAGE (or \"all\", \"packages\")" ),
+		"PACKAGE" },
+	{ "usage", 'u', 0, G_OPTION_ARG_STRING, &main_option_usage, 
+		N_( "show usage message for OPERATION" ), 
+		"OPERATION" },
+	{ "plugin", 'p', 0, G_OPTION_ARG_FILENAME, &main_option_plugin, 
+		N_( "load PLUGIN" ), 
+		"PLUGIN" },
+	{ "links", 'k', 0, G_OPTION_ARG_NONE, &main_option_links, 
+		N_( "print link lines for all operations" ), NULL },
+	{ "cpph", 'h', 0, G_OPTION_ARG_STRING, &main_option_cpph, 
+		N_( "print C++ decls for PACKAGE (or \"all\")" ), 
+		"PACKAGE" },
+	{ "cppc", 'c', 0, G_OPTION_ARG_STRING, &main_option_cppc, 
+		N_( "print C++ binding for PACKAGE (or \"all\")" ), 
+		"PACKAGE" },
+	{ "version", 'v', 0, G_OPTION_ARG_NONE, &main_option_version, 
+		N_( "print im_version_string" ), NULL },
+	{ NULL }
+};
+
+typedef void *(*map_name_fn)( im_function * );
+
+/* Loop over a package.
  */
 static void *
-print_package_name( im_package *pack )
+map_package( im_package *pack, map_name_fn fn )
 {
-	printf( "\t%s (%d functions)\n", pack->name, pack->nfuncs );
+	int i;
+	void *result;
+
+	for( i = 0; i < pack->nfuncs; i++ ) 
+		if( (result = fn( pack->table[i] )) )
+			return( result );
+
 	return( NULL );
 }
 
-/* List all functions in a package, with their description.
+/* Apply a function to a vips operation, or map over a package of operations.
  */
 static void *
-list_functions( im_package *pack )
+map_name( const char *name, map_name_fn fn )
 {
-	int i;
+	im_package *pack;
+	im_function *func;
 
-	printf( "functions available in package \"%s\":\n", pack->name );
-	for( i = 0; i < pack->nfuncs; i++ ) 
-		printf( "%-18s - %s\n", 
-			pack->table[i]->name, _( pack->table[i]->desc ) );
+	if( strcmp( name, "all" ) == 0 ) 
+		/* Do all packages.
+		 */
+		im_map_packages( (VSListMap2Fn) map_package, fn );
+	else if( (pack = im_find_package( name )) )
+		/* Do one package.
+		 */
+		map_package( pack, fn );
+	else if( (func = im_find_function( name )) )
+		/* Do a single function.
+		 */
+		fn( func );
+	else {
+		im_error( "map_name", 
+			_( "no package or function \"%s\"" ), name );
+		return( fn );
+	}
+
+	return( NULL );
+}
+
+static void *
+list_package( im_package *pack )
+{
+	printf( "%-18s - %d operations\n", pack->name, pack->nfuncs );
 	
 	return( NULL );
 }
 
+static void *
+list_function( im_function *func )
+{
+	printf( "%-18s - %s\n", func->name, _( func->desc ) );
+	
+	return( NULL );
+}
+
+static void
+print_list( const char *name )
+{
+	if( strcmp( name, "packages" ) == 0 ) 
+		im_map_packages( (VSListMap2Fn) list_package, NULL );
+	else {
+		if( map_name( name, list_function ) )
+			error_exit( "unknown package \"%s\"", name ); 
+	}
+}
+
 /* Is s1 a prefix of s2?
  */
 static int
-isprefix( char *s1, char *s2 )
+isprefix( const char *s1, const char *s2 )
 {
 	while( *s1 && *s1 == *s2 ) {
 		s1++;
@@ -89,7 +188,7 @@ isprefix( char *s1, char *s2 )
 /* Is s1 a postfix of s2?
  */
 static int
-ispostfix( char *s1, char *s2 )
+ispostfix( const char *s1, const char *s2 )
 {
 	int l1 = strlen( s1 );
 	int l2 = strlen( s2 );
@@ -137,7 +236,12 @@ usage( im_function *fn )
 	int i;
 	im_package *pack = im_package_of_function( fn->name );
 
-	fprintf( stderr, "%s: args: ", our_name ); 
+	/* Don't print the prgname if we're being run as a symlink.
+	 */
+	fprintf( stderr, "usage: " );
+	if( im_isprefix( "vips", g_get_prgname() ) ) 
+		fprintf( stderr, "%s ", g_get_prgname() );
+	fprintf( stderr, "%s ", fn->name ); 
 
 	/* Print args requiring command-line input.
 	 */
@@ -194,213 +298,6 @@ usage( im_function *fn )
 	fprintf( stderr, "\n" );
 }
 
-/* Help system.
- */
-static void
-print_help( char *cat )
-{
-	im_function *fn;
-	im_package *pack;
-
-	if( !cat ) {
-		/* Plain "-help". 
-		 */
-		printf( 
-"\"-help flags\" for a list of flags available\n"
-"\"-help <package name>\" for a list of commands in package\n"
-"where <package name> is one of:\n" );
-		im_map_packages( 
-			(VSListMap2Fn) print_package_name, NULL );
-		printf( 
-"\"-help all\" for a list of all commands in all packages\n" 
-"\"-help <command>\" for an explanation of a command\n" ); 
-	}
-	else if( strcmp( cat, "flags" ) == 0 )
-		printf(
-"flags supported are:\n"
-"-help\t\t\tsimple help system\n"
-"-plugin <file name>\tadd commands in plugin to those available\n"
-"-links\t\t\tprint link lines for all functions\n"
-"-ip <package name>\tprint sample ip \".def\" file for package\n"
-"-c++ <package name>\tprint C++ bindings for package\n" 
-"-c++.h <package name>\tprint just C++ member decls\n"
-"-c++.cc <package name>\tprint just C++ function bodies\n" );
-	else if( strcmp( cat, "all" ) == 0 )
-		im_map_packages( (VSListMap2Fn) list_functions, NULL );
-	else {
-		/* Search for package or function of this name.
-		 */
-		if( (fn = im_find_function( cat )) )
-			/* Found a matching function. Print usage.
-			 */
-			usage( fn );
-		else if( (pack = im_find_package( cat )) ) 
-			(void) list_functions( pack );
-		else if( !isprefix( "im_", cat ) ) {
-			char buf[ 256 ];
-
-			im_snprintf( buf, 256, "im_%s", cat );
-			if( (fn = im_find_function( buf )) )
-				usage( fn );
-		}
-		else
-			error_exit( "%s: -help: no such function or "
-				"package \"%s\"", our_name, cat );
-	}
-}
-
-/* Print a sample .def file for package.
- */
-static void
-print_defs( char *pname )
-{
-	im_package *pack = im_find_package( pname );
-	int i, j, k;
-
-	/* VIPS types.
-	 */
-	static char *vtypes[] = {
-		IM_TYPE_DOUBLE,
-		IM_TYPE_INT,  
-		IM_TYPE_COMPLEX,
-		IM_TYPE_STRING,
-		IM_TYPE_IMAGE         
-	};
-
-	/* Corresponding ip type predicates.
-	 */
-	static char *itypes[] = {
-		"is_number",
-		"is_number",
-		"is_complex",
-		"is_string",
-		"is_imreg"
-	};
-
-	/* Corresponding ip names.
-	 */
-	static char *inames[] = {
-		"number",
-		"number",
-		"complex",
-		"string",
-		"image"
-	};
-
-	if( !pack )
-		error_exit( "%s: -ip: no such package \"%s\"", 
-			our_name, pname );
-
-	for( i = 0; i < pack->nfuncs; i++ ) {
-		im_function *fn = pack->table[i];
-		char name[ 4096 ];
-
-		/* Make name for function: chop off "im_" prefix.
-		 */
-		if( isprefix( "im_", fn->name ) )
-			strcpy( name, fn->name + 3 );
-		/* Nope --- try capitalising the initial letter.
-		 */
-		else if( islower( fn->name[0] ) ) {
-			strcpy( name, fn->name );
-			name[0] = toupper( name[0] );
-		}
-		/* Nope --- add an ip_ prefix.
-		 */
-		else {
-			strcpy( name, "ip_" );
-			strcat( name, fn->name );
-		}
-		printf( "/* %s", name );
-
-		/* Print names for each input arg. Ignore DISPLAY arguments.
-		 */
-		for( j = 0; j < fn->argc; j++ ) {
-			im_type_desc *vt = fn->argv[j].desc;
-
-			if( !(vt->flags & IM_TYPE_OUTPUT) &&
-				strcmp( vt->type, IM_TYPE_DISPLAY ) != 0 )
-				printf( " %s", fn->argv[j].name );
-		}
-		
-		/* Print description.
-		 */
-		printf( ": %s\n */\n\n", _( fn->desc ) );
-
-		/* Print definition!
-		 */
-		printf( "%s", name );
-		for( j = 0; j < fn->argc; j++ ) {
-			im_type_desc *vt = fn->argv[j].desc;
-
-			if( !(vt->flags & IM_TYPE_OUTPUT) &&
-				strcmp( vt->type, IM_TYPE_DISPLAY ) != 0 )
-				printf( " %s", fn->argv[j].name );
-		}
-		printf( "\n" );
-
-		/* Print a typecheck for each arg.
-		 */
-		for( j = 0; j < fn->argc; j++ ) {
-			im_type_desc *ty = fn->argv[j].desc;
-
-			/* Skip output args.
-			 */
-			if( ty->flags & IM_TYPE_OUTPUT )
-				continue;
-			
-			/* Skip DISPLAY args.
-			 */
-			if( strcmp( ty->type, IM_TYPE_DISPLAY ) == 0 )
-				continue;
-
-			/* Find type.
-			 */
-			for( k = 0; k < IM_NUMBER( vtypes ); k++ )
-				if( strcmp( ty->type, vtypes[k] ) == 0 ) {
-					printf( 				
-"      =\terror \"argument %d is not %s\", not %s %s;\n",
-						j + 1, inames[k], itypes[k],
-						fn->argv[j].name );
-					break;
-				}
-
-			/* Help!
-			 */
-			if( k == IM_NUMBER( vtypes ) )
-				printf( 
-"      =\terror \"unknown type %s\", true;\n", ty->type );
-		}
-
-		/* Call the function itself.
-		 */
-		printf( "      =\t%s", fn->name );
-		for( j = 0; j < fn->argc; j++ ) {
-			im_type_desc *ty = fn->argv[j].desc;
-
-			/* Skip output args.
-			 */
-			if( ty->flags & IM_TYPE_OUTPUT )
-				continue;
-			
-			/* Skip DISPLAY args.
-			 */
-			if( strcmp( ty->type, IM_TYPE_DISPLAY ) == 0 )
-				continue;
-
-			/* If its an IMAGE, then forceim it.
-			 */
-			if( strcmp( ty->type, IM_TYPE_IMAGE ) == 0 )
-				printf( " (forceim %s)", 
-					fn->argv[j].name );
-			else
-				printf( " %s", 
-					fn->argv[j].name );
-		}
-		printf( ";\n\n" );
-	}
-}
-
 /* Convert VIPS type name to C++ type name. NULL for type unsupported by C++
  * layer.
  */
@@ -419,7 +316,10 @@ vips2cpp( im_type_desc *ty )
 		IM_TYPE_IMAGE,
 		IM_TYPE_IMASK,
 		IM_TYPE_DMASK,
-		IM_TYPE_DISPLAY
+		IM_TYPE_DISPLAY,
+		IM_TYPE_IMAGEVEC,
+		IM_TYPE_DOUBLEVEC,
+		IM_TYPE_INTVEC
 	};
 
 	/* Corresponding C++ types.
@@ -432,7 +332,10 @@ vips2cpp( im_type_desc *ty )
 		"VImage",
 		"VIMask",
 		"VDMask",
-		"VDisplay"
+		"VDisplay",
+		"std::vector<VImage>",
+		"std::vector<double>",
+		"std::vector<int>"
 	};
 
 	for( k = 0; k < IM_NUMBER( vtypes ); k++ )
@@ -458,6 +361,18 @@ is_cppable( im_function *fn )
 			return( 0 );
 	}
 
+	/* We dont wrap output IMAGEVEC/DOUBLEVEC/INTVEC.
+	 */
+	for( j = 0; j < fn->argc; j++ ) {
+		im_type_desc *ty = fn->argv[j].desc;
+
+		if( ty->flags & IM_TYPE_OUTPUT ) 
+			if( strcmp( ty->type, IM_TYPE_IMAGEVEC ) == 0 ||
+				strcmp( ty->type, IM_TYPE_DOUBLEVEC ) == 0 ||
+				strcmp( ty->type, IM_TYPE_INTVEC ) == 0 )
+			return( 0 );
+	}
+
 	/* Must be at least one image argument (input or output) ... since we 
 	 * get inserted in the VImage class. Other funcs get wrapped by hand.
 	 */
@@ -505,51 +420,60 @@ find_ioargs( im_function *fn, int *ia, int *oa )
 	}
 }
 
-/* Print decl for a function. If decl == 1, then print member part, else print
- * full declaration. Print "&" for reference args for other than first
- * output.
- *
- * decl == 1:
- * 	VImage cos();
- *	VImage lin( double, double );
- *
- * decl == 0:
- * 	VImage VImage::cos()
- *	VImage VImage::lin( double a, double b )
+/* Turn a VIPS name into a C++ name. Eg. im_lintra_vec becomes lin.
  */
 static void
-print_cppdecl( im_function *fn, int decl )
+c2cpp_name( const char *in, char *out )
+{
+	/* chop off "im_" prefix.
+	 */
+	if( isprefix( "im_", in ) )
+		strcpy( out, in + 3 );
+	else
+		strcpy( out, in );
+
+	/* Drop "_vec" postfix (eg. so im_lintra_vec becomes lintra). We rely
+	 * on overloading to distinguish conflicts.
+	 */
+	if( ispostfix( "_vec", out ) )
+		out[strlen( out ) - 4] = '\0';
+
+	/* Drop "const" postfix (eg. so im_eorimageconst becomes eorimage). 
+	 */
+	if( ispostfix( "const", out ) )
+		out[strlen( out ) - 5] = '\0';
+
+	/* Drop "tra" postfix (eg. so im_costra becomes cos).
+	 */
+	if( ispostfix( "tra", out ) )
+		out[strlen( out ) - 3] = '\0';
+}
+
+/* Print prototype for a function (ie. will be followed by code). 
+ *
+ * Eg.:
+ *	VImage VImage::lin( double a, double b ) throw( VError )
+ */
+static void *
+print_cppproto( im_function *fn )
 {
 	int j;
-	char name[ 4096 ];
+	char name[4096];
 	int oa, ia;
 	int flg;
 
 	/* If it's not cppable, do nothing.
 	 */
 	if( !is_cppable( fn ) )
-		return;
+		return( NULL );
 
-	/* Make name for function: chop off "im_" prefix.
+	/* Make C++ name.
 	 */
-	if( isprefix( "im_", fn->name ) )
-		strcpy( name, fn->name + 3 );
-	else
-		strcpy( name, fn->name );
-
-	/* And drop "tra" postfix (eg. so im_costra becomes cos).
-	 */
-	if( ispostfix( "tra", name ) )
-		name[strlen( name ) - 3] = '\0';
+	c2cpp_name( fn->name, name );
 
 	/* Find input and output args. 
 	 */
 	find_ioargs( fn, &ia, &oa );
-	if( ia == -1 && decl ) 
-		/* No input image, so make it a static in the class
-		 * declaration.
-		 */
-		printf( "static " );
 
 	/* Print output type.
 	 */
@@ -558,12 +482,7 @@ print_cppdecl( im_function *fn, int decl )
 	else 
 		printf( "%s ", vips2cpp( fn->argv[oa].desc ) );
 
-	/* Print function name and start arg list.
-	 */
-	if( decl )
-		printf( "%s(", name );
-	else
-		printf( "VImage::%s(", name );
+	printf( "VImage::%s(", name );
 
 	/* Print arg list.
 	 */
@@ -593,23 +512,114 @@ print_cppdecl( im_function *fn, int decl )
 
 		/* Print arg name.
 		 */
-		if( !decl )
-			printf( " %s", fn->argv[j].name );
+		printf( " %s", fn->argv[j].name );
 	}
 
 	/* End of arg list!
 	 */
 	if( flg )
 		printf( " " );
-	printf( ")" );
-	if( decl )
-		printf( ";" );
-	printf( "\n" );
+	printf( ") throw( VError )\n" );
+
+	return( NULL );
+}
+
+/* Print cpp decl for a function. 
+ *
+ * Eg.
+ *	VImage lin( double, double ) throw( VError );
+ */
+static void *
+print_cppdecl( im_function *fn )
+{
+	int j;
+	char name[4096];
+	int oa, ia;
+	int flg;
+
+	/* If it's not cppable, do nothing.
+	 */
+	if( !is_cppable( fn ) )
+		return( NULL );
+
+	/* Make C++ name.
+	 */
+	c2cpp_name( fn->name, name );
+
+	/* Find input and output args. 
+	 */
+	find_ioargs( fn, &ia, &oa );
+	if( ia == -1 ) 
+		/* No input image, so make it a static in the class
+		 * declaration.
+		 */
+		printf( "static " );
+
+	/* Print output type.
+	 */
+	if( oa == -1 )
+		printf( "void " );
+	else 
+		printf( "%s ", vips2cpp( fn->argv[oa].desc ) );
+
+	/* Print function name and start arg list.
+	 */
+	printf( "%s(", name );
+
+	/* Print arg list.
+	 */
+	flg = 0;
+	for( j = 0; j < fn->argc; j++ ) {
+		im_type_desc *ty = fn->argv[j].desc;
+
+		/* Skip ia and oa.
+		 */
+		if( j == ia || j == oa )
+			continue;
+
+		/* Print arg type.
+		 */
+		if( flg )
+			printf( ", %s", vips2cpp( ty ) );
+		else {
+			printf( " %s", vips2cpp( ty ) );
+			flg = 1;
+		}
+
+		/* If it's an putput arg, print a "&" to make a reference
+		 * argument.
+		 */
+		if( ty->flags & IM_TYPE_OUTPUT )
+			printf( "&" );
+	}
+
+	/* End of arg list!
+	 */
+	if( flg )
+		printf( " " );
+
+	printf( ") throw( VError );\n" );
+
+	return( NULL );
+}
+
+static void
+print_invec( int j, const char *arg, 
+	const char *vips_name, const char *c_name, const char *extract )
+{
+	printf( "\t((%s*) _vec.data(%d))->n = %s.size();\n",
+		vips_name, j, arg );
+	printf( "\t((%s*) _vec.data(%d))->vec = new %s[%s.size()];\n",
+		vips_name, j, c_name, arg );
+	printf( "\tfor( unsigned int i = 0; i < %s.size(); i++ )\n",
+		arg );
+	printf( "\t\t((%s*) _vec.data(%d))->vec[i] = %s[i]%s;\n",
+		vips_name, j, arg, extract );
 }
 
 /* Print the definition for a function.
  */
-static void
+static void *
 print_cppdef( im_function *fn )
 {
 	int j;
@@ -618,12 +628,12 @@ print_cppdef( im_function *fn )
 	/* If it's not cppable, do nothing.
 	 */
 	if( !is_cppable( fn ) )
-		return;
+		return( NULL );
 
 	find_ioargs( fn, &ia, &oa );
 
 	printf( "// %s: %s\n", fn->name, _( fn->desc ) );
-	print_cppdecl( fn, 0 );
+	print_cppproto( fn );
 	printf( "{\n" );
 
 	/* Declare the implicit input image.
@@ -672,13 +682,16 @@ print_cppdef( im_function *fn )
 		if( ty->flags & IM_TYPE_OUTPUT )
 			continue;
 
-		if( strcmp( ty->type, IM_TYPE_IMASK ) == 0 ||
-			strcmp( ty->type, IM_TYPE_DMASK ) == 0 ) 
+		if( strcmp( ty->type, IM_TYPE_IMASK ) == 0 )
 			/* Mask types are different - have to use
 			 * im_mask_object.
 			 */
 			printf( "\t((im_mask_object*) "
-				"_vec.data(%d))->mask = %s.mask();\n",
+				"_vec.data(%d))->mask = %s.mask().iptr;\n",
+				j, fn->argv[j].name );
+		else if( strcmp( ty->type, IM_TYPE_DMASK ) == 0 ) 
+			printf( "\t((im_mask_object*) "
+				"_vec.data(%d))->mask = %s.mask().dptr;\n",
 				j, fn->argv[j].name );
 		else if( strcmp( ty->type, IM_TYPE_DISPLAY ) == 0 )
 			/* Display have to use VDisplay.
@@ -690,6 +703,15 @@ print_cppdef( im_function *fn )
 			 */
 			printf( "\t_vec.data(%d) = (im_object) %s;\n",
 				j, fn->argv[j].name );
+		else if( strcmp( ty->type, IM_TYPE_IMAGEVEC ) == 0 ) 
+			print_invec( j, fn->argv[j].name, 
+				"im_imagevec_object", "IMAGE *", ".image()" );
+		else if( strcmp( ty->type, IM_TYPE_DOUBLEVEC ) == 0 ) 
+			print_invec( j, fn->argv[j].name, 
+				"im_doublevec_object", "double", "" );
+		else if( strcmp( ty->type, IM_TYPE_INTVEC ) == 0 ) 
+			print_invec( j, fn->argv[j].name, 
+				"im_intvec_object", "int", "" );
 		else
 			/* Just use vips2cpp().
 			 */
@@ -721,7 +743,7 @@ print_cppdef( im_function *fn )
 			/* Mask types are different - have to use
 			 * im_mask_object.
 			 */
-			printf( "\t%s.embed( (void *)((im_mask_object*)"
+			printf( "\t%s.embed( (DOUBLEMASK *)((im_mask_object*)"
 				"_vec.data(%d))->mask );\n",
 				fn->argv[j].name, j );
 		else if( strcmp( ty->type, IM_TYPE_STRING ) == 0 )
@@ -754,17 +776,26 @@ print_cppdef( im_function *fn )
 				if( ty2->flags & IM_TYPE_OUTPUT )
 					continue;
 
-				/* Skip non-image args.
+				/* Input image.
 				 */
-				if( strcmp( ty2->type, 
-					IM_TYPE_IMAGE ) != 0 )
-					continue;
-
-				/* Found one!
-				 */
-				printf( "\t%s._ref->addref( %s._ref );\n",
-					fn->argv[oa].name,
-					fn->argv[j].name );
+				if( strcmp( ty2->type, IM_TYPE_IMAGE ) == 0 ) 
+					printf( "\t%s._ref->addref( "
+						"%s._ref );\n",
+						fn->argv[oa].name,
+						fn->argv[j].name );
+				else if( strcmp( ty2->type, IM_TYPE_IMAGEVEC ) 
+					== 0 ) {
+					/* The out depends on every image in
+					 * the input vector.
+					 */
+					printf( "\tfor( unsigned int i = 0; "
+						"i < %s.size(); i++ )\n",
+						fn->argv[j].name );
+					printf( "\t\t%s._ref->addref( "
+						"%s[i]._ref );\n",
+						fn->argv[oa].name,
+						fn->argv[j].name );
+				}
 			}
 		}
 	}
@@ -774,41 +805,7 @@ print_cppdef( im_function *fn )
 	if( oa != -1 )
 		printf( "\n\treturn( %s );\n", fn->argv[oa].name );
 
-	printf( "}\n" );
-}
-
-/* Generate member function decls for package.
- */
-static void *
-print_cpppackdelcs( im_package *pack )
-{
-	int i;
-
-	printf( "// Member function decls for package \"%s\"\n", pack->name );
-	for( i = 0; i < pack->nfuncs; i++ ) 
-		print_cppdecl( pack->table[i], 1 );
-	printf( "\n" );
-	
-	return( NULL );
-}
-
-/* Generate function bodies for package.
- */
-static void *
-print_cpppackdefs( im_package *pack )
-{
-	int i;
-
-	printf( "// Member function defs for package \"%s\"\n\n", 
-		pack->name );
-	for( i = 0; i < pack->nfuncs; i++ ) {
-		im_function *fn = pack->table[i];
-
-		if( is_cppable( fn ) ) {
-			print_cppdef( fn );
-			printf( "\n" );
-		}
-	}
+	printf( "}\n\n" );
 
 	return( NULL );
 }
@@ -818,27 +815,11 @@ print_cpppackdefs( im_package *pack )
 static void
 print_cppdecls( char *name )
 {
-	im_package *pack = im_find_package( name );
-	im_function *fn = im_find_function( name );
-
 	printf( "// this file automatically generated from\n"
 		"// VIPS library %s\n", im_version_string() );
 
-	if( strcmp( name, "all" ) == 0 ) 
-		/* Do all packages.
-		 */
-		im_map_packages( (VSListMap2Fn) print_cpppackdelcs, NULL );
-	else if( pack ) 
-		/* Do one package.
-		 */
-		print_cpppackdelcs( pack );
-	else if( fn ) 
-		/* Do a single function.
-		 */
-		print_cppdecl( fn, 1 );
-	else
-		error_exit( "%s: -c++: no package/function \"%s\"", 
-			our_name, name );
+	if( map_name( name, print_cppdecl ) )
+		error_exit( "unknown package \"%s\"", name ); 
 }
 
 /* Print C++ bindings for function, package or all.
@@ -846,27 +827,11 @@ print_cppdecls( char *name )
 static void
 print_cppdefs( char *name )
 {
-	im_package *pack = im_find_package( name );
-	im_function *fn = im_find_function( name );
-
 	printf( "// this file automatically generated from\n"
 		"// VIPS library %s\n", im_version_string() );
 
-	if( strcmp( name, "all" ) == 0 ) 
-		/* Do all packages.
-		 */
-		im_map_packages( (VSListMap2Fn) print_cpppackdefs, NULL );
-	else if( pack ) 
-		/* Do one package.
-		 */
-		print_cpppackdefs( pack );
-	else if( fn ) 
-		/* Do a single function.
-		 */
-		print_cppdef( fn );
-	else
-		error_exit( "%s: -c++: no package/function \"%s\"", 
-			our_name, name );
+	if( map_name( name, print_cppdef ) )
+		error_exit( "unknown package \"%s\"", name ); 
 }
 
 /* VIPS universal main program. 
@@ -874,127 +839,106 @@ print_cppdefs( char *name )
 int
 main( int argc, char **argv )
 {
-	const char *vipshome = im_guess_prefix( argv[0], "VIPSHOME" );
+	GOptionContext *context;
+	GError *error = NULL;
 	im_function *fn;
-	int i;
-
-	/* Want numeric conversions to be non-locale, for eg. matrix read.
-	 */
-	setlocale( LC_NUMERIC, "C" );
+	int i, j;
 
 	if( im_init_world( argv[0] ) )
 	        error_exit( "unable to start VIPS" );
 
-	/* Load up any standard plugins.
-	 */
-	if( vipshome )
-		if( im_load_plugins( "%s/lib", vipshome ) )
-			error_exit( "error loading plugins" );
+        context = g_option_context_new( _( "- VIPS driver program" ) );
 
-	/* Remove any leading pathname components from our name, drop ".exe"
-	 * suffix.
-	 */
-	if( (our_name = strrchr( argv[0], IM_DIR_SEP )) )
-		our_name++;
-	else
-		our_name = argv[0];
-	if( (i = strlen( IM_EXEEXT )) > 0 ) {
-		if( ispostfix( IM_EXEEXT, our_name ) )
-			our_name[strlen( our_name ) - i] = '\0';
+	g_option_context_add_main_entries( context,
+		main_option, GETTEXT_PACKAGE );
+	g_option_context_add_group( context, im_get_option_group() );
+
+	if( !g_option_context_parse( context, &argc, &argv, &error ) ) {
+		if( error ) {
+			fprintf( stderr, "%s\n", error->message );
+			g_error_free( error );
+		}
+
+		error_exit( "try \"%s --help\"", g_get_prgname() );
 	}
 
+	g_option_context_free( context );
+
+	if( main_option_plugin ) {
+		if( !im_load_plugin( main_option_plugin ) )
+			error_exit( "unable to load plugin %s",
+				main_option_plugin );
+	}
+	if( main_option_cpph ) 
+		print_cppdecls( main_option_cpph );
+	if( main_option_cppc ) 
+		print_cppdefs( main_option_cppc );
+	if( main_option_links )
+		im_map_packages( (VSListMap2Fn) print_links, NULL );
+	if( main_option_list ) 
+		print_list( main_option_list );
+	if( main_option_usage ) {
+		if( !(fn = im_find_function( main_option_usage )) )
+			error_exit( "unknown operation %s", main_option_usage );
+		usage( fn );
+	}
+	if( main_option_version ) 
+		printf( "vips-%s\n", im_version_string() );
+
+	/* Remove any "--" argument. If one of our arguments is a negative
+	 * number, the user will need to have added the "--" flag to stop
+	 * GOption parsing. But "--" is still passed down to us and we need to
+	 * ignore it.
+	 */
+	for( i = 1; i < argc - 1; i++ )
+		if( strcmp( argv[i], "--" ) == 0 ) {
+			for( j = i; j < argc; j++ )
+				argv[j] = argv[j + 1];
+
+			argc -= 1;
+		}
+
 	/* Should we try to run the thing we are named as?
 	 */
-	if( strcasecmp( our_name, "vips" ) != 0 ) {
-		/* We are not called vips - try to look up ourselves.
-		 */
-		char name2[ 1024 ];
+	if( !im_isprefix( "vips", g_get_prgname() ) ) {
+		char name[256];
 
-		/* If our_name is unknown, try with "im_" prepended.
+		/* Drop any .exe suffix.
 		 */
-		if( !(fn = im_find_function( our_name )) ) {
-			im_snprintf( name2, 1024, "im_%s", our_name );
+		im_strncpy( name, g_get_prgname(), 256 );
+		if( ispostfix( ".exe", name ) )
+			name[strlen( name ) - 4] = '\0';
 
-			if( (fn = im_find_function( name2 )) )
-				our_name = name2;
-			else
+		/* If unknown, try with "im_" prepended.
+		 */
+		if( !(fn = im_find_function( name )) ) {
+			im_snprintf( name, 256, "im_%s", g_get_prgname() );
+			if( ispostfix( ".exe", name ) )
+				name[strlen( name ) - 4] = '\0';
+
+			if( !(fn = im_find_function( name )) )
 				error_exit( "unknown function" );
 		}
 
 		/* Execute it!
 		 */
-		if( im_run_command( our_name, argc-1, argv+1 ) ) {
+		if( im_run_command( name, argc - 1, argv + 1 ) ) {
 			usage( fn );
-			error_exit( "%s: error calling function", our_name );
+			error_exit( "error calling function" );
 		}
-
-		return( 0 );
 	}
-
-	/* We are called "vips". No more args?
-	 */
-	if( argc == 1 )
-		error_exit( "usage: %s [flags] [command]", our_name );
-
-	/* Look for flags.
-	 */
-	for( i = 1; argv[i] && argv[i][0] == '-'; i++ )
-		if( strcmp( argv[i], "-help" ) == 0 ) {
-			print_help( argv[i+1] );
-			return( 0 );
-		}
-		else if( strcmp( argv[i], "-plugin" ) == 0 ) {
-			if( !argv[i+1] )
-				error_exit( "%s: missing argument to "
-					"-plugin", argv[0] );
-			if( !im_load_plugin( argv[i+1] ) )
-				error_exit( "%s: unable to load plugin",
-					argv[0] );
-			i++;
-		}
-		else if( strcmp( argv[i], "-ip" ) == 0 ) {
-			if( !argv[i+1] )
-				error_exit( "%s: missing argument to "
-					"-ip", argv[0] );
-			print_defs( argv[++i] );
-		}
-		else if( strcmp( argv[i], "-c++.h" ) == 0 ) {
-			if( !argv[i+1] )
-				error_exit( "%s: missing argument to "
-					"-c++.h", argv[0] );
-			print_cppdecls( argv[++i] );
-		}
-		else if( strcmp( argv[i], "-c++.cc" ) == 0 ) {
-			if( !argv[i+1] )
-				error_exit( "%s: missing argument to "
-					"-c++.cc", argv[0] );
-			print_cppdefs( argv[++i] );
-		}
-		else if( strcmp( argv[i], "-c++" ) == 0 ) {
-			if( !argv[i+1] )
-				error_exit( "%s: missing argument to "
-					"-c++", argv[0] );
-			print_cppdecls( argv[i+1] );
-			print_cppdefs( argv[i+1] );
-			i++;
-		}
-		else if( strcmp( argv[i], "-links" ) == 0 ) {
-			im_map_packages( (VSListMap2Fn) print_links, NULL );
-		}
-		else 
-			error_exit( "%s: unknown flag \"%s\" - try \"-help\"", 
-				argv[0], argv[i] );
-
-	/* All flags processed. Any args left?
-	 */
-	if( i < argc )
-		if( im_run_command( argv[i], argc-i-1, argv+i+1 ) ) {
-			if( !(fn = im_find_function( argv[i] )) )
+	else if( argc > 1 ) {
+		/* Nope ... run the first arg instead.
+		 */
+		if( im_run_command( argv[1], argc - 2, argv + 2 ) ) {
+			if( !(fn = im_find_function( argv[1] )) )
 				error_exit( "unknown function" );
 			usage( fn );
-			error_exit( "%s: error calling function", argv[0] );
+			error_exit( "error calling function" );
 		}
+	}
 
 	im_close_plugins();
+
 	return( 0 );
 }
