@@ -69,7 +69,6 @@ int xoverlap;
 int yoverlap;
 
 extern int im_lrmerge();
-extern int im_updatehist();
 extern int im_merge_analysis();
 extern int im__find_lroverlap();
 extern int im__find_tboverlap();
@@ -220,6 +219,7 @@ double scale1, angle1, dx1, dy1;
 	curr_im = -1;
 	curr_disp_x = -1;
 	curr_disp_y = -1;
+	dy = -1;
 	for(i=0; i<=height; i++){
 		for(j=0; j<=width; j++){
 		++curr_im;
@@ -263,7 +263,7 @@ double scale1, angle1, dx1, dy1;
 				(int)(xoff/2), (int)(inp[curr_im+1]->Ysize /2),
                 		halfcorsize, halfareasize , &fx, &fy,
 				&scale1, &angle1, &dx1, &dy1 ) == -1 )
-                		error_exit("find_mosaic: Unable to im__find_lroverlap");
+                		error_exit("Unable to im__find_lroverlap");
 
 			++curr_disp_x;
 			hxdisp[curr_disp_x] = inp[curr_im]->Xsize - xoff + fx;
@@ -310,7 +310,7 @@ double scale1, angle1, dx1, dy1;
 			 	(int)(inp[curr_row]->Xsize/2 ), (int)(yoff/2),
                 	 	halfcorsize, halfareasize, &fx, &fy,
 				&scale1, &angle1, &dx1, &dy1 ) == -1 )
-                		error_exit("find_mosaic: Unable to im__find_tboverlap");
+                		error_exit("Unable to im__find_tboverlap");
 
 
 		     ++curr_disp_y;
@@ -391,7 +391,7 @@ char **argv;
 	/* Make output name. and store them in an array.
 	 */
         if( !(out = im_open( "tmp.v", "t" )) )
-            error_exit("find_mosaic: unable to open file for output");
+            error_exit("unable to open file for output");
 
 	file_ptr =0;
         for(i=height; i>=0; i--)
@@ -399,7 +399,7 @@ char **argv;
                 im_snprintf( name, 1024, "%s.%dx%d.v", file_root,j,i );
                 output_file = strdup( name );
                 if( !(in[file_ptr] = im_open( output_file, "r" )) )
-                    error_exit("find_mosaic: unable to open %s for input",output_file);
+                    error_exit("unable to open %s for input",output_file);
                 ++file_ptr;
 
             }
@@ -420,9 +420,9 @@ char **argv;
 
 	for(i=0; i<file_ptr; i++)
 		if( im_close(in[i]) == -1)
-			error_exit("find_mosaic: unable to close partial file");  
+			error_exit("unable to close partial file");  
 		if( im_close(out) == -1)
-			error_exit("find_mosaic: unable to close\n ");
+			error_exit("unable to close\n ");
 	fclose( fp );
 
 
