@@ -518,7 +518,7 @@ char **argv;
 	 */
 	im_snprintf( name, 1000, "%s/paint.hr.v", argv[4] );
         if( !(out = im_open( name, "w" )) )
-            error_exit("mergeup: unable to open file for output");
+            error_exit("unable to open file for output");
 
 	file_ptr =0;
         for(i=height; i>=0; i--)
@@ -526,7 +526,7 @@ char **argv;
                 im_snprintf( name, 1000, "%s.%dx%d.v", file_root,j,i );
                 output_file = strdup( name );
                 if( !(in[file_ptr] = im_open( output_file, "r" )) )
-                    error_exit("mergeup: unable to open %s for input",output_file);
+                    error_exit("unable to open %s for input",output_file);
                 ++file_ptr;
 
             }
@@ -548,9 +548,9 @@ char **argv;
 
 	for(i=0; i<file_ptr; i++)
 		if( im_close(in[i]) == -1)
-			error_exit("mergeup: unable to close partial file");  
+			error_exit("unable to close partial file");  
 		if( im_close(out) == -1)
-			error_exit("mergeup: unable to close\n ");
+			error_exit("unable to close\n ");
 
 	return( 0 );
 }
