@@ -67,20 +67,20 @@ argv[0]);
 	dy = atoi(argv[8]);
 
 	if ( (image = im_open(argv[1],"r")) == NULL )
-		error_exit("glds: Unable to open %s for input", argv[1]);
+		error_exit("Unable to open %s for input", argv[1]);
 
 	if ( (matrix = im_open(argv[2],"w")) == NULL )
-		error_exit("glds: Unable to open %s for output", argv[2]);
+		error_exit("Unable to open %s for output", argv[2]);
 
 	if ( im_glds_matrix(image, matrix, xpos, ypos, xsize, ysize,
 		dx, dy) == -1 )
-		error_exit("glds: Unable to im_glds_matrix");
+		error_exit("Unable to im_glds_matrix");
 
-	if ( im_updatehist(matrix, argc, argv) == -1)
-		error_exit("glds: Unable to update history");
+	if ( im_updatehist(image, argv[0], argc - 1, argv + 1) == -1)
+		error_exit("Unable to update history");
 
 	if ( ( im_close( image ) == -1 )||( im_close( matrix ) == -1 ) )
-		error_exit("glds: Unable to close %s or %s", argv[1], argv[2]);
+		error_exit("Unable to close %s or %s", argv[1], argv[2]);
 
 	return(0);
 }
