@@ -63,16 +63,16 @@ argv[0]);
 
 
 	if ( (image = im_openout(argv[1])) == NULL )
-		error_exit("simcontr: Unable to open %s for output", argv[1]);
+		error_exit("Unable to open %s for output", argv[1]);
 
 	if ( im_simcontr(image, xsize, ysize) == -1 )
-		error_exit("simcontr: Unable to im_simcontr");
+		error_exit("Unable to im_simcontr");
 
-	if ( im_updatehist(image, argc, argv) == -1)
-		error_exit("simcontr: Unable to update history");
+	if ( im_updatehist(image, argv[0], argc - 1, argv + 1) == -1)
+		error_exit("Unable to update history");
 
 	if ( im_close( image ) == -1 )
-		error_exit("simcontr: Unable to close %s", argv[1]);
+		error_exit("Unable to close %s", argv[1]);
 
 	return(0);
 }
