@@ -65,22 +65,22 @@ argv[0]);
 	verfreq = atof(argv[5]);
 
 	if ( (bufim = im_setbuf("temp.v")) == NULL )
-		error_exit("sines: Unable to set buffer image");
+		error_exit("Unable to set buffer image");
 
 	if ( im_sines(bufim, xsize, ysize, horfreq, verfreq) == -1 )
-		error_exit("sines: Unable to im_sines");
+		error_exit("Unable to im_sines");
 
 	if ( (image = im_openout(argv[1])) == NULL )
-		error_exit("sines: Unable to open %s for output", argv[1]);
+		error_exit("Unable to open %s for output", argv[1]);
 
 	if ( im_scale(bufim, image) == -1)
-		error_exit("sines: Unable to im_scale");
+		error_exit("Unable to im_scale");
 
-	if ( im_updatehist(image, argc, argv) == -1)
-		error_exit("sines: Unable to update history");
+	if ( im_updatehist(image, argv[0], argc - 1, argv + 1) == -1)
+		error_exit("Unable to update history");
 
 	if ( ( im_close( image ) == -1 )||( im_close( bufim ) == -1 ) )
-		error_exit("sines: Unable to close %s or buffer image",argv[1]);
+		error_exit("Unable to close %s or buffer image",argv[1]);
 
 	return(0);
 }
