@@ -65,19 +65,19 @@ WARNING: The program destroys the opfile if the owner has rw access on it.",
 	step = atoi(argv[3]);
 
 	if ((in= im_open(argv[1],"r")) == NULL)
-		error_exit("spatres: Unable to open %s for input", argv[1]);
+		error_exit("Unable to open %s for input", argv[1]);
 
 	if ( (out=im_open(argv[2],"w")) == NULL )
-		error_exit("spatres: Unable to open %s", argv[2]);
+		error_exit("Unable to open %s", argv[2]);
 
 	if ( im_spatres(in, out, step) == -1)
-		error_exit("spatres: Unable to im_spatres");
+		error_exit("Unable to im_spatres");
 
-	if ( im_updatehist(out, argc, argv) == -1)
-		error_exit("spatres: Unable to update history");
+	if ( im_updatehist(out, argv[0], argc - 1, argv + 1) == -1)
+		error_exit("Unable to update history");
 
 	if ( (im_close(in) == -1)||(im_close(out) == -1) )
-		error_exit("spatres: unable to close %s or %s",argv[1],argv[2]);
+		error_exit("unable to close %s or %s",argv[1],argv[2]);
 
 	return(0);
 }
